#!/bin/ksh
# Script to run the TSTRAD code in Forward mode (direct mode)
# The result is compared to a reference file. See users guide
# to note what are the "normal" differences.
# 
# This script runs only ONE test
# 
# P. Brunel Dec 2002
#

usage(){
  echo
  echo " run one RTTOV forward test"; 
  echo
  echo "usage  $0 Test_number platform instrument sat_id file_channels file_profile zenangle azangle \
        sunzenangle sunazangle surface_type water_type latitude elevation refraction ";
  echo "  test_number            the reference test number (see test plan)";
  echo "  platform	platform name (noaa, goes...)";
  echo "  instrunemt    instrument name (hirs amsua ..)   ";
  echo "  sat_id        satellite number (xx for NOAAxx)   ";
  echo "  file_channels file containing the list of channels/emissivity  ";
  echo "  file_profile  file containing the input profile   ";
  echo "  file_plevs    file containing the input pressure levels";
  echo "  zenangle      zenithal angle (deg)   ";
  echo "  azangle       azimuthal angle (deg)   ";
  echo "  solar radiation ";
  echo "  aerosols ";
  echo "  clouds ";
  echo "  surface_type  surface type   (0=land, 1=sea, 2=ice/snow) ";
  echo "  water_type    Water type (0=fresh water, 1=ocean water ";
  echo "  latitude      Latitude of sounding (degrees) ";
  echo "  elevation     Elevation of sounding (km) ";
  echo "  refraction    0=No , 1=Yes "; 
  echo "     ";
  echo "environment variables:"
  echo "  DATAIN  	directory for file channels and profiles "
  echo "  DATART	directory for coefficient files"
  echo "  REF_TEST	directory for reference test results"
  echo "  TEST		directory for test results "
  echo
  echo "environment variables default values:"
  echo "  DATAIN  	../data "
  echo "  DATART	../rtcoef"
  echo "  REF_TEST	../new_reftest"
  echo "  TEST		../new_test "
  echo
}
if [ $# -lt 14 ]
then
	usage
	exit 1
fi

# use function for rttov ids
. ./rttov_id.ksh

# Get arguments
TEST_NUMBER=$1
PLATFORM=$2
INST=$3
SATID=$4
FILE_CHANNELS=$5
FILE_PROFILE=$6
FILE_PLEVS=$7
ZENANGLE=$8
AZANGLE=$9
SUNRAD=${10}
AEROSOLS=${11}
CLOUDS=${12}
SURF_TYPE=${13}
WATER_TYPE=${14}
LATITUDE=${15}
ELEVATION=${16}
REFRACTION=${17}

# diff command ignoring case of letters and spaces
DIFF='diff -biw'

# Following variables should have been initialised and exported
# in the calling script, else default values are appling
DATAIN=${DATAIN:=../data}  
DATART=${DATART:=../rtcoef_rttov7} 
REF_TEST=${REF_TEST:=../reftest}
TEST=${TEST:=../test}

# Create directory for test results
if [ ! -d ${TEST} ]; then
  mkdir ${TEST}
fi
#
###################################################
# The followind sed command file is to change all occurence of
# .xxx or -.yyy into 0.xxx and -0.yyy to allow a good comparison
# between the refernce test and fortran output (especially for HP)
cat <<EOF > sed.cmd
s/ \./0\./g
s/ -\./-0\./g
EOF
#

CUR_DIR=`pwd`


echo " "
echo " "
echo " Test forward  ${INST} number ${TEST_NUMBER}"
echo " "

echo  "Platform   ${PLATFORM}"
echo  "Instrument ${INST}"
echo  "Satellite  ${SATID}"
echo  "File for channels ${FILE_CHANNELS}"
echo  "File for profile  ${FILE_PROFILE}"
echo  "Zenith angle ${ZENANGLE}"
echo  "Azimuth angle ${AZANGLE}"
echo  "Solar Radiation ${SUNRAD}"
echo  "Aerosols ${AEROSOLS}"
echo  "Clouds ${CLOUDS}"
echo  "Surface type ${SURF_TYPE}"
echo  "Water type ${WATER_TYPE}"
echo  "Latitude ${LATITUDE}"
echo  "Elevation ${ELEVATION}"
echo  "Refraction ${REFRACTION}"
echo  "DATART ${DATART}"

# call to function rttov_id
rttov_id ${PLATFORM} ${INST}


rm -f input.dat prof.dat
if [ ! -s ${DATAIN}/${FILE_CHANNELS} ] ; then
  echo "file ${FILE_CHANNELS}  does not exist on directory  ${DATAIN}"
  exit
fi
if [ ! -s ${DATAIN}/${FILE_PROFILE} ] ; then
  echo "file ${FILE_PROFILE}  does not exist on directory  ${DATAIN}"
  exit
fi
if [ ! -s ${DATAIN}/${FILE_PLEVS} ] ; then
  echo "file ${FILE_PLEVS}  does not exist on directory  ${DATAIN}"
  exit
fi
cp ${DATAIN}/${FILE_CHANNELS} input.dat
cp ${DATAIN}/${FILE_PROFILE}  prof.dat
cp ${DATAIN}/${FILE_PLEVS}    plevs.dat

COEF_NAME1=rtcoef_${PLATFORM}_${SATID}_${INST}_101L.dat
COEF_NAME_BIN1=rtcoef_${PLATFORM}_${SATID}_${INST}_101L.bin
COEF_NAME=rtcoef_${PLATFORM}_${SATID}_${INST}.dat
COEF_NAME_BIN=rtcoef_${PLATFORM}_${SATID}_${INST}.bin
rm -f ${COEF_NAME}
rm -f ${COEF_NAME_BIN}
if [ -s  $DATART/${COEF_NAME1} ] ; then
  ln -s $DATART/${COEF_NAME1} ${COEF_NAME}
fi
if [ -s  $DATART/${COEF_NAME_BIN1} ] ; then
  ln -s $DATART/${COEF_NAME_BIN1} ${COEF_NAME_BIN}
fi
echo
echo
echo "FWD test ${PLATFORM} ${SATID} ${INST}"

../bin/tstrad.exe <<EOF
${PLATFORM_N}   , platform name
${SATID}        , satellite number (NOAAxx = xx)
${INST_N}       , instrument number
0               , forward model only == 0, full gradient==1, radiance output ==2
1               , number of profiles
${SURF_TYPE}    , Surface type (0=land, 1=sea, 2=ice/snow)
${WATER_TYPE}   , Water type (0=fresh,1=ocean)
${AEROSOLS}     , Aerosols
${CLOUDS}       , Clouds
${ZENANGLE}     , Zenith angle
${AZANGLE}      , Azimuth angle
${SUNRAD}       , Solar radiation
${LATITUDE}     , Latitude of sounding
${ELEVATION}    , Elevation of terrain
${REFRACTION}   , Atmospheric refraction (0=no, 1=yes)
EOF

rm -f ${COEF_NAME} ${COEF_NAME_BIN}

sed -f sed.cmd print.dat > ${TEST}/tstrad_fwd_${INST}_${TEST_NUMBER}.lst
${DIFF} ${REF_TEST}/tstrad_fwd_${INST}_${TEST_NUMBER}.lst ${TEST}/tstrad_fwd_${INST}_${TEST_NUMBER}.lst > ${TEST}/tstrad_fwd_${INST}_${TEST_NUMBER}.diff

rm -f print.dat sed.cmd input.dat prof.dat plevs.dat
mv intavg_vars_wt.dat ${TEST}/intavg_vars_wt_${INST}_${TEST_NUMBER}.dat
mv intavg_temp_op.dat ${TEST}/intavg_temp_op_${INST}_${TEST_NUMBER}.dat
mv intavg_tran_op.dat ${TEST}/intavg_tran_op_${INST}_${TEST_NUMBER}.dat

date
exit

