!
Program rtcoef_select_channels
  ! Description:
  ! Main code to select channels from an RTTOV coefficient file
  ! Output is another RTTOV coefficient file (ascii or binary)
  ! It can also be used to copy an ascii file to a binary one, all channels
  !
  ! Copyright:
  !    This software was developed within the context of
  !    the EUMETSAT Satellite Application Facility on
  !    Numerical Weather Prediction (NWP SAF), under the
  !    Cooperation Agreement dated Dec 2003, between
  !    EUMETSAT and the Met Office, UK, by one or more partners
  !    within the NWP SAF. The partners in the NWP SAF are
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  !
  !    Copyright 2006, EUMETSAT, All Rights Reserved.
  !
  ! Method:
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date     Comment
  ! -------   ----     -------
  !  1.0     02/08/2006  New F90 code with structures (P Brunel)
  !  1.1     11/08/2006  Modified for RTTOV-9
  !
  ! Code Description:
  !   Language:           Fortran 90.
  !   Software Standards: "European Standards for Writing and
  !     Documenting Exchangeable Fortran 90 Code".
  !
  ! Declarations:
  ! Modules used:
  ! Imported Parameters:
  Use rttov_const, Only :   &
       & errorstatus_success

  ! Imported Type Definitions:
  Use rttov_types, Only : &
        & rttov_coef,           &
        & rttov_coef_scatt_ir,  &
        & rttov_optpar_ir

  Use parkind1, Only : jpim     ,jprb
  Implicit None

#include "rttov_opencoeff.interface"
#include "rttov_readcoeffs.interface"
#include "rttov_initcoeffs.interface"
#include "rttov_writecoef.interface"
#include "rttov_coeffname.interface"
#include "rttov_errorreport.interface"

  Type( rttov_coef ) :: coef               ! optical depth coefficients
  Type(rttov_coef_scatt_ir) :: coef_scatt_ir ! scattering coeffs
  Type(rttov_optpar_ir)  :: optp          ! optical properties

  ! Local variables
  !-------------------
  ! coeffiecnt structure for 2 instruments

  ! Logical units for input/output
  Integer(Kind=jpim) :: file_id
  Integer(Kind=jpim) :: file_out =20
  ! error return code for subroutines
  Integer(Kind=jpim) :: errorstatus
  ! if open file is for output and ascii/binary
  Logical :: for_output = .True.
  Logical :: lbinary
  ! character string for file name
  Character(len=128) :: coeffname
  ! number of channels to prosess
  Integer(Kind=jpim)          :: nchannels, i
  Integer(Kind=jpim), Pointer :: channels(:)
  Integer(Kind=jpim) :: instrument(3)
  ! Flags for cloud and aerosol
  Logical :: aerosl =.False.
  Logical :: clouds =.FALSE.

  Character(len=8) :: ascii
  !- End of header --------------------------------------------------------

  Write(*,*) 'enter ascii or binary output (a/b)  '
  Read(*,*) ascii
  If(ascii(1:1) == 'a') Then
     lbinary = .False.
  Else
     lbinary = .True.
  Endif
 
  Write(*,*) 'enter platform, satid, instrument'
  Read(*,*) instrument

  Write(*,*) 'enter the number of channels to copy (0 for all, -n for first n channels) '
  Read(*,*) nchannels
  If( nchannels > 0 ) Then
     Write(*,*) 'enter a list of channels '
     Allocate ( channels( nchannels ) )
     Read(*,*) channels(:)
  ElseIf( nchannels < 0 ) Then
     nchannels = -nchannels
     Allocate ( channels( nchannels ) )
     Do i = 1, nchannels
        channels(i) = i
     End Do
  Endif

  file_id = 0

 ! read the coefficients for the selection of channels
   Call Rttov_errorreport( errorstatus_success, 'start reading', 'main')
     If ( nchannels /= 0 ) Then
        Call rttov_readcoeffs  (errorstatus, aerosl, clouds, coef, &
     &   coef_scatt_ir, optp, instrument=instrument, file_id = file_id, channels = channels)
        Call rttov_initcoeffs  (errorstatus, aerosl, clouds,coef, &
        & coef_scatt_ir, optp)
     Else
        Call rttov_readcoeffs  (errorstatus, aerosl, clouds, coef, &
     &   coef_scatt_ir, optp, instrument=instrument,file_id = file_id)
        Call rttov_initcoeffs  (errorstatus, aerosl, clouds,coef, &
        & coef_scatt_ir, optp)

     Endif
     Call Rttov_errorreport( errorstatus, 'end reading', 'main')
     Close ( unit = file_id )

  ! Open file and store the coefficents 
  Call rttov_coeffname ( errorstatus, instrument, coeffname, lbinary=lbinary)
  Write(*,*) 'coefficient file ',coeffname
  Call rttov_opencoeff (errorstatus, coeffname, file_out, for_output=.True., lbinary=lbinary)
  Call Rttov_writecoef (errorstatus, coef, coef_scatt_ir, optp, file_out, aerosl=.False., clouds=.False., lbinary=lbinary)
  Close ( unit = file_out )

  Stop
End Program rtcoef_select_channels
