!
Program rttov_conv_coef
  ! Description:
  ! converts an ascii RTTOV coefficient file to binary format
  ! for a selected set of channels
  !
  ! Copyright:
  !    This software was developed within the context of
  !    the EUMETSAT Satellite Application Facility on
  !    Numerical Weather Prediction (NWP SAF), under the
  !    Cooperation Agreement dated 25 November 1998, between
  !    EUMETSAT and the Met Office, UK, by one or more partners
  !    within the NWP SAF. The partners in the NWP SAF are
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  !
  !    Copyright 2002, EUMETSAT, All Rights Reserved.
  !
  ! Method:
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! 14/08/2008 Add the -fchannels parameter P.Marguinaud
  !
  ! Code Description:
  !   Language:           Fortran 90.
  !   Software Standards: "European Standards for Writing and
  !     Documenting Exchangeable Fortran 90 Code".
  !
  ! Declarations:
  ! Modules used:
  ! Imported Type Definitions:

  Use rttov_types, Only :  &
        & rttov_coef      ,&
        & rttov_optpar_ir ,&
        & rttov_coef_scatt_ir 
        
  Use rttov_getoptions

  USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

  Use rttov_unix_env, Only: rttov_iargc, rttov_getenv

  Use parkind1, Only : jpim, jprb

  Use rttov_const, Only :     &
      & errorstatus_success ,&
      & errorstatus_warning ,&
      & errorstatus_fatal  

  

#include "rttov_opencoeff.interface"
#include "rttov_readcoeffs.interface"
#include "rttov_initcoeffs.interface"
#include "rttov_writecoef.interface"
#include "rttov_errorreport.interface"


  ! Local variables
  !-------------------
  ! coeffiecnt structure
  Type(rttov_coef)          :: coef        ! coefficients
  Type(rttov_optpar_ir)     :: optp
  Type(rttov_coef_scatt_ir) :: coef_scatt_ir

  Integer(Kind=jpim) :: file_id, file_id_scat, file_id_optp
  Integer(Kind=jpim) :: errorstatus
  Integer(Kind=jpim), Pointer :: channels(:) => NULL()
  Logical  ::aerosl
  Logical  ::clouds
  Character*256 :: f_coef_in = "", f_scat_in = "", f_optp_in = ""
  Character*256 :: f_coef_out = "", f_scat_out = "", f_optp_out = ""
  Character*256 :: coef_format_in = "", coef_format_out = "", fchannels = ""
  Logical :: lbinary
  Integer :: ioerr, err
  Character(len=80) :: errMessage
  Character(len=4096) :: buffer
  Character(len=*), Parameter :: NameOfRoutine = 'RTTOV_CONV_COEF'
  Integer(Kind=jpim) :: nchannels


  REAL(KIND=JPRB) :: ZHOOK_HANDLE

  !- End of header --------------------------------------------------------

  IF (LHOOK) CALL DR_HOOK(NameOfRoutine,0_jpim,ZHOOK_HANDLE)


  If( rttov_iargc() .eq. 0 ) Then
    Print *, "Usage: --format_in  FORMATTED|UNFORMATTED                   "
    Print *, "       --format_out FORMATTED|UNFORMATTED                   "
    Print *, "       --channels 1 2 3 4 5 ... | --fchannels channels.txt  "
    Print *, "       --coef_in  ... --scat_in  ... --optp_in  ...         "
    Print *, "       --coef_out ... --scat_out ... --optp_out ...         "
    Stop
  EndIf
  
  Call getoption( "--format_in",  coef_format_in  )
  Call getoption( "--format_out", coef_format_out )

  Call getoption( "--channels",  channels  )
  Call getoption( "--fchannels", fchannels )

  Call getoption( "--coef_in",  f_coef_in  )
  Call getoption( "--scat_in",  f_scat_in  )
  Call getoption( "--optp_in",  f_optp_in  )
  
  Call getoption( "--coef_out",  f_coef_out  )
  Call getoption( "--scat_out",  f_scat_out  )
  Call getoption( "--optp_out",  f_optp_out  )
  
  If( f_coef_out .eq. "" .and. f_coef_in .ne. "" ) f_coef_out = Trim(f_coef_in)//'.bin'
  If( f_scat_out .eq. "" .and. f_scat_in .ne. "" ) f_scat_out = Trim(f_scat_in)//'.bin'
  If( f_optp_out .eq. "" .and. f_optp_in .ne. "" ) f_optp_out = Trim(f_optp_in)//'.bin'

  If( ( fchannels .ne. "" ) .and. ( .not. associated( channels ) ) ) Then
    Open( 77, file = Trim(fchannels), form = 'formatted', status = 'old', iostat = ioerr )
    If( ioerr .ne. 0 ) Then
      Write( errMessage, '( "Could not open ",A, " for reading")' ) Trim(fchannels)
      Goto 999
    EndIf
    nchannels = 0_jpim
    Do 
      Read( 77, '(A)', end = 500 ) buffer
      nchannels = nchannels + CountWords( buffer )
    EndDo
 500 Continue
    Rewind( 77 )
    Allocate( channels( nchannels ), stat = err )
    If( err .ne. 0 ) Then
      Write( errMessage, '( "Could not allocate channels array")' ) 
      Goto 999
    EndIf
    Read( 77, * ) channels
    Close( 77 )
  EndIf
  
  aerosl = f_scat_out .ne. ""
  clouds = f_optp_out .ne. ""

  ! let the subroutine choose a logical unit for the file
  file_id      = 77
  file_id_scat = 78
  file_id_optp = 79

  
    Open( file_id,      File = f_coef_in, Form = Trim( coef_format_in ), &
      Status = 'old', Err = 100, Iostat = ioerr )
  If( aerosl ) &
    Open( file_id_scat, File = f_scat_in, Form = Trim( coef_format_in ), &
      Status = 'old', Err = 101, Iostat = ioerr )
  If( clouds ) &
    Open( file_id_optp, File = f_optp_in, Form = Trim( coef_format_in ), &
      Status = 'old', Err = 102, Iostat = ioerr )

  If( Associated( channels ) ) Then
     Call rttov_readcoeffs ( errorstatus, aerosl, clouds, & 
        & coef, coef_scatt_ir, optp,                      &
        & file_id          = file_id,                     &
        & file_id_scatt_ir = file_id_scat,                &
        & file_id_optp     = file_id_optp,                &
        & channels = channels )
  Else
     Call rttov_readcoeffs ( errorstatus, aerosl, clouds, & 
        & coef, coef_scatt_ir, optp,                      &
        & file_id          = file_id,                     &
        & file_id_scatt_ir = file_id_scat,                &
        & file_id_optp     = file_id_optp                 &
        & )
  Endif
  
  If( errorstatus .ne. 0_jpim ) Goto 103
  
  Call rttov_initcoeffs( errorstatus, aerosl, clouds, coef, &
      & coef_scatt_ir, optp )
      

103 If( clouds ) Close( 79 )
102 If( aerosl ) Close( 78 )
101 Close( 77 )
100 Continue

  If( ( ioerr .ne. 0_jpim ) .or. ( errorstatus .ne. 0_jpim ) ) Then
                 Print *, "Failed to read coefficient files"
                 Print *, " f_coef = ", f_coef_in
    If( aerosl ) Print *, " f_scat = ", f_scat_in
    If( clouds ) Print *, " f_optp = ", f_optp_in
    Stop 1
  EndIf


  
  lbinary = coef_format_out .eq. 'unformatted' .or. coef_format_out .eq. 'UNFORMATTED'

  Call rttov_opencoeff( errorstatus, f_coef_out, file_id, for_output = .True., lbinary = lbinary )
  Call Rttov_writecoef( errorstatus, coef, coef_scatt_ir, optp, file_id, aerosl = .False., &
    clouds = .False., lbinary = lbinary )
  Close( unit = file_id )

  If ( aerosl ) Then
    Call rttov_opencoeff( errorstatus, f_scat_out, file_id_scat, for_output = .True., lbinary = lbinary )
    Call Rttov_writecoef( errorstatus, coef, coef_scatt_ir, optp, file_id_scat, aerosl = .True., &
      clouds = .False., lbinary = lbinary )
    Close( unit = file_id_scat )
  End If

  If ( clouds ) Then
    Call rttov_opencoeff( errorstatus, f_optp_out, file_id_optp, for_output = .True., lbinary = lbinary )
    Call Rttov_writecoef( errorstatus, coef, coef_scatt_ir, optp, file_id_optp, aerosl = .False., &
      clouds = .True., lbinary = lbinary )
    Close( unit = file_id_optp )
  End If

  If( Associated( channels ) ) Deallocate( channels )

  Call rttov_dealloc_coef( errorstatus, coef, coef_scatt_ir, optp )

  IF (LHOOK) CALL DR_HOOK(NameOfRoutine,1_jpim,ZHOOK_HANDLE)
  Stop

999 Continue
    Call Rttov_ErrorReport( errorstatus_fatal, errMessage, NameOfRoutine )
    IF (LHOOK) CALL DR_HOOK(NameOfRoutine,1_jpim,ZHOOK_HANDLE)

Contains

Integer(Kind=jpim) Function CountWords( s )
  Character(len=*) :: s
  Integer(Kind=jpim) :: n, i, l
  Logical :: in
  n = 0_jpim
  in = .false.
  l = Len( Trim( s ) )
  Do i = 1, l
    If( s(i:i) .eq. ' ' ) Then
      in = .false.
    Else If( .not. in ) Then
      n = n + 1
      in = .true.
    EndIf
  EndDo
  CountWords = n
End Function

End Program rttov_conv_coef
