Subroutine rttov_opdep_9_ad(        &
      & nchannels,      &! in
      & nprofiles,      &! in
      & nlevels,        &! in
      & channels,       &! in
      & lprofiles,      &! in
      & predictors,     &! in
      & predictors_ad,  &! inout
      & aux,            &! in
      & aux_ad,         &! inout
      & coef,           &! in
      & opdp_path,      &! inout
      & opdp_path_ad,   &! inout
      & opdp_ref)        ! inout

  ! Description:
  ! Adjoint of rttov_transmit_tl
  !  To calculate optical depths for a number of channels
  !  and profiles from every pressure level to space.
  ! To interpolate optical depths on to levels of radiative transfer model
  ! (which, at present, entails only surface transmittance, as
  ! other optical depths are on *rt* levels) and to convert
  ! optical depths to transmittances.
  !
  ! Code based on OPDEPAD and RTTAUAD from previous versions of RTTOV
  ! Only one profile per call
  !
  ! Adjoint variables
  ! input transmission_ad % tau_surf and transmission_ad % tau_layer 
  ! set inside integrate_ad
  !
  ! input/output aux_ad 
  !
  ! output predictors_ad initialised inside rttov_ad (need input
  !    intent for memory allocation in calling routine)
  !
  !
  ! Copyright:
  !    This software was developed within the context of
  !    the EUMETSAT Satellite Application Facility on
  !    Numerical Weather Prediction (NWP SAF), under the
  !    Cooperation Agreement dated 25 November 1998, between
  !    EUMETSAT and the Met Office, UK, by one or more partners
  !    within the NWP SAF. The partners in the NWP SAF are
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  !
  !    Copyright 2002, EUMETSAT, All Rights Reserved.
  !
  ! Method:
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date     Comment
  ! -------   ----     -------
  !  1.0    01/06/2005  Marco Matricardi (ECMWF):
  !            --       New routine based on rttov_transmit_ad.F90.
  !                     Variable trace gases, CO2, N2O,CO and CH4
  !                     have been introduced for AIRS and IASI
  !  1.1    06/02/2007  Removed polarisation R Saunders
  !  1.2    07/04/2008  Marco Matricardi (ECMWF):
  !                     modified the routine to be consistent
  !                     with the LBLRTM coefficients
  !
  ! Code Description:
  !   Language:           Fortran 90.
  !   Software Standards: "European Standards for Writing and
  !     Documenting Exchangeable Fortran 90 Code".
  !
  ! Declarations:
  ! Modules used:

  ! Imported Parameters:

  Use rttov_const, Only: &
        & sensor_id_mw    
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

  Use rttov_types, Only :                & 
        & rttov_coef     ,               &
        & predictors_Type,               &
        & opdp_path_Type  ,              &
        & profile_aux

  Use parkind1, Only : jpim     ,jprb
  Implicit None

  !subroutine arguments:
  Integer(Kind=jpim),    Intent(in)    :: nchannels
  Integer(Kind=jpim),    Intent(in)    :: nprofiles
  Integer(Kind=jpim),    Intent(in)    :: nlevels
  Integer(Kind=jpim),    Intent(in)    :: channels(nchannels)
  Integer(Kind=jpim),    Intent(in)    :: lprofiles(nchannels)
  Type(predictors_Type), Intent(in)    :: predictors
  Type(predictors_Type), Intent(inout) :: predictors_ad
  Type(rttov_coef)     , Intent(in)    :: coef
  Type(profile_aux)   ,  Intent(in)    :: aux( nprofiles )
  Type(profile_aux)   ,  Intent(inout) :: aux_ad( nprofiles )
  Type(opdp_path_Type),  Intent(inout) :: opdp_path          ! optical depths
  Type(opdp_path_Type),  Intent(inout) :: opdp_path_ad            
  Real(Kind=jprb),       Intent(in)    :: opdp_ref(nlevels,nchannels)

!INTF

  !local variables:
  Real(Kind=jprb) :: opticaldepth_ad(nlevels,nchannels)
  Real(Kind=jprb) :: chanx

  Real(Kind=jprb),  Pointer :: debye_prof(:,:)
  Real(Kind=jprb),  Pointer :: debye_prof_ad(:,:)
  Integer(Kind=jpim) :: lev,chan,j
  Integer(Kind=jpim) :: prof

  ! cloud liquid water local variables
  Real(Kind=jprb) :: zf, zf_sq, z34_dif, z45_dif, z1_sq, z2_sq, z1_div, z2_div
  Real(Kind=jprb) :: z1_den, z2_den, zastar, z1_prod, z2_prod, z3_prod, z4_prod
  Real(Kind=jprb) :: zbstar, zbstar_sq, za2star, za2star_sq, zdiv, zgstar
  Real(Kind=jprb) :: z1f_sq_z1_sq, z2f_sq_z2_sq
  Real(Kind=jprb) :: z34_dif_ad, z45_dif_ad, z1_sq_ad, z2_sq_ad, z1_div_ad, z2_div_ad
  Real(Kind=jprb) :: z1_den_ad, z2_den_ad, zastar_ad, z1_prod_ad, z2_prod_ad, z3_prod_ad, z4_prod_ad
  Real(Kind=jprb) :: zbstar_ad, zbstar_sq_ad, za2star_ad, za2star_sq_ad, zdiv_ad, zgstar_ad
  Real(Kind=jprb) :: z1f_sq_z1_sq_ad, z2f_sq_z2_sq_ad
REAL(KIND=JPRB) :: ZHOOK_HANDLE

  !- End of header --------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('RTTOV_OPDEP_9_AD',0_jpim,ZHOOK_HANDLE)

  !----------------------------------------
  !2.Assemble layer optical depths
  !----------------------------------------
  ! note that optical depth in the calculations is negative

  opticaldepth_ad(:,:) = 0._JPRB

  Do lev = nlevels, 2, -1
    opdp_path_ad % atm_layer(lev-1,:) = opdp_path_ad % atm_layer(lev-1,:) &
                                    & + opdp_path_ad % atm_layer(lev,:)
    opticaldepth_ad(lev,:) = opticaldepth_ad(lev,:)  + opdp_path_ad % atm_layer(lev,:)
    opdp_path_ad % atm_layer(lev,:) = 0.0_JPRB
  End Do
  opticaldepth_ad(1,:) = opticaldepth_ad(1,:) + opdp_path_ad % atm_layer(1,:)

  Where( opdp_ref(:,:) > 0.0_JPRB )
     opticaldepth_ad = 0.0_JPRB
  End Where

  !--------------------
  !1.9 add liquid water (MW only)
  !--------------------
  If ( coef % id_sensor == sensor_id_mw ) Then
     Do j = 1, nchannels
        chan = channels(j)
        prof = lprofiles(j)
        debye_prof    => aux(prof) % debye_prof
        debye_prof_ad => aux_ad(prof) % debye_prof
        If( predictors % ncloud >= 1 ) Then
           Do lev = coef %  mwcldtop, nlevels

              ! Repeat direct code
              zf           = coef % frequency_ghz(chan)
              zf_sq        = zf*zf
              z1_sq        = debye_prof(1,lev) * debye_prof(1,lev)
              z2_sq        = debye_prof(2,lev) * debye_prof(2,lev)
              z34_dif      = debye_prof(3,lev) - debye_prof(4,lev)
              z45_dif      = debye_prof(4,lev) - debye_prof(5,lev)
              z1f_sq_z1_sq = zf_sq + z1_sq
              z2f_sq_z2_sq = zf_sq + z2_sq
              z1_div       = 1.0_JPRB / z1f_sq_z1_sq
              z2_div       = 1.0_JPRB / z2f_sq_z2_sq
              z1_den       = z34_dif * z1_div
              z2_den       = z45_dif * z2_div
              zastar       = debye_prof(3,lev) - zf_sq * (z1_den + z2_den)
              z1_prod      = z34_dif * debye_prof(1,lev)
              z2_prod      = z1_prod * z1_div
              z3_prod      = z45_dif * debye_prof(2,lev)
              z4_prod      = z3_prod * z2_div
              zbstar       = -zf * (z2_prod + z4_prod)
              zbstar_sq    = zbstar * zbstar
              za2star      = zastar + 2.0_JPRB
              za2star_sq   = za2star * za2star
              zdiv         = za2star_sq + zbstar_sq
              zgstar       = -3.0_JPRB * zbstar / zdiv

              ! Now compute Adjoint code
              !opticaldepth_ad(lev,j)= opticaldepth_ad(lev,j)
              zgstar_ad = opticaldepth_ad(lev,j) *&
                     & (-1.5_JPRB * zf * predictors % clw(lev,prof))  
              predictors_ad % clw(lev,prof) = predictors_ad % clw(lev,prof) +&
                     & opticaldepth_ad(lev,j) * (-1.5_JPRB * zf * zgstar)  

              zbstar_ad = -3.0_JPRB * zgstar_ad / zdiv
              zdiv_ad   =  3.0_JPRB * zgstar_ad * zbstar / (zdiv*zdiv)
              !zgstar_ad =  0.

              za2star_sq_ad = zdiv_ad
              zbstar_sq_ad  = zdiv_ad
              !zdiv_ad       = 0.

              za2star_ad    = 2.0_JPRB * za2star * za2star_sq_ad 
              !za2star_sq_ad = 0.

              zastar_ad     = za2star_ad
              !za2star_ad    = 0.

              zbstar_ad     = zbstar_ad + 2.0_JPRB * zbstar * zbstar_sq_ad
              !zbstar_sq_ad  = 0.

              z2_prod_ad    = -zf * zbstar_ad
              z4_prod_ad    = -zf * zbstar_ad
              !zbstar_ad     = 0.

              z3_prod_ad    = z2_div  * z4_prod_ad
              z2_div_ad     = z3_prod * z4_prod_ad
              !z4_prod_ad    = 0.

              z45_dif_ad           = debye_prof(2,lev)             * z3_prod_ad
              debye_prof_ad(2,lev) = debye_prof_ad(2,lev) + z45_dif* z3_prod_ad
              !z3_prod_ad           = 0.

              z1_prod_ad    = z1_div  * z2_prod_ad
              z1_div_ad     = z1_prod * z2_prod_ad
              !z2_prod_ad    = 0.

              z34_dif_ad           = debye_prof(1,lev)              * z1_prod_ad
              debye_prof_ad(1,lev) = debye_prof_ad(1,lev) + z34_dif * z1_prod_ad
              !z1_prod_ad           = 0.

              debye_prof_ad(3,lev) = debye_prof_ad(3,lev) + zastar_ad
              z1_den_ad            =              -zf_sq  * zastar_ad 
              z2_den_ad            =              -zf_sq  * zastar_ad 
              !zastar_ad            = 0.

              z2_div_ad     = z2_div_ad  + z45_dif * z2_den_ad
              z45_dif_ad    = z45_dif_ad + z2_div  * z2_den_ad
              !z2_den_ad     = 0.

              z1_div_ad     = z1_div_ad  + z34_dif * z1_den_ad
              z34_dif_ad    = z34_dif_ad + z1_div  * z1_den_ad
              !z1_den_ad     = 0.

              z2f_sq_z2_sq_ad = -z2_div_ad / (z2f_sq_z2_sq * z2f_sq_z2_sq)
              !z2_div_ad       = 0.

              z1f_sq_z1_sq_ad = -z1_div_ad / (z1f_sq_z1_sq * z1f_sq_z1_sq)
              !z1_div_ad       = 0.

              z2_sq_ad        = z2f_sq_z2_sq_ad
              !z2f_sq_z2_sq_ad = 0.

              z1_sq_ad        = z1f_sq_z1_sq_ad
              !z1f_sq_z1_sq_ad = 0.

              debye_prof_ad(4,lev) = debye_prof_ad(4,lev) + z45_dif_ad
              debye_prof_ad(5,lev) = debye_prof_ad(5,lev) - z45_dif_ad
              !z45_dif_ad           = 0.

              debye_prof_ad(3,lev) = debye_prof_ad(3,lev) + z34_dif_ad
              debye_prof_ad(4,lev) = debye_prof_ad(4,lev) - z34_dif_ad
              !z34_dif_ad           = 0.

              debye_prof_ad(2,lev) = debye_prof_ad(2,lev) + z2_sq_ad *&
                     & 2.0_JPRB * debye_prof(2,lev)  
              !z2_sq_ad             = 0.

              debye_prof_ad(1,lev) = debye_prof_ad(1,lev) + z1_sq_ad *&
                     & 2.0_JPRB * debye_prof(1,lev)  
              !z1_sq_ad             = 0.

           End Do
        Endif
     End Do
  End If

  !-----------
  !1.8 add CH4
  !----------- 

  If ( coef%nch4 > 0 ) Then
     Do j = 1, nchannels
        chan = channels(j)
        prof = lprofiles(j)
        Do lev=nlevels, 1, -1
           predictors_ad % ch4(:,lev,prof) =&
                     & predictors_ad % ch4(:,lev,prof) +&
                     & coef%ch4(lev,chan,:) * opticaldepth_ad(lev,j) 

        End Do
     End Do
   End if

  !-----------
  !1.7 add CO
  !-----------

  If ( coef%nco > 0 ) Then
     Do j = 1, nchannels
        chan = channels(j)
        chanx= coef%ff_cwn(chan)
        prof = lprofiles(j)

        if((chanx>=2000.and.chanx<=2249.75).or.                                   &
         & (chanx>=2380.25.and.chanx<=2760))then 

          Do lev=nlevels, 1, -1
  
            predictors_ad % co(1:10,lev,prof) =&
                     & predictors_ad % co(1:10,lev,prof) +&
                     & coef%co(lev,chan,1:10) * opticaldepth_ad(lev,j) 
            predictors_ad % co(12,lev,prof) =&
                     & predictors_ad % co(12,lev,prof) +&
                     & coef%co(lev,chan,11) * opticaldepth_ad(lev,j) 
            predictors_ad % co(13,lev,prof) =&
                     & predictors_ad % co(13,lev,prof) +&
                     & coef%co(lev,chan,12) * opticaldepth_ad(lev,j) 

          End Do
 
        else
          Do lev=nlevels, 1, -1

           predictors_ad % co(1:11,lev,prof) =&
                     & predictors_ad % co(1:11,lev,prof) +&
                     & coef%co(lev,chan,1:11) * opticaldepth_ad(lev,j) 

        End Do
        endif
     End Do
   End if

  !-----------
  !1.6 add N2O
  !-----------

  If ( coef%nn2o > 0 ) Then
     Do j = 1, nchannels
        chan = channels(j)
        chanx= coef%ff_cwn(chan)
        prof = lprofiles(j)
          if(chanx>=1050.and.chanx<=1350)then
          Do lev=nlevels, 1, -1

             predictors_ad % ch4(10,lev,prof)=                              &
             & predictors_ad % ch4(10,lev,prof)+coef%n2o(lev,chan,12)       * &
                                     & opticaldepth_ad(lev,j) 

             predictors_ad % ch4(1,lev,prof)=                               &
             & predictors_ad % ch4(1,lev,prof)+coef%n2o(lev,chan,11)        * &
                                     & opticaldepth_ad(lev,j) 

             predictors_ad % n2o(1:10,lev,prof) =&
                      & predictors_ad % n2o(1:10,lev,prof) +&
                      & coef%n2o(lev,chan,1:10) * opticaldepth_ad(lev,j) 

          End Do
        else if((chanx>=1995.and.chanx<=1999.75).or.                              &
                & (chanx>=2250.and.chanx<=2295))then 

          Do lev=nlevels, 1, -1
            predictors_ad % co(8,lev,prof) = predictors_ad % co(8,lev,prof)                  - &
                & coef%n2o(lev,chan,12)*opticaldepth_ad(lev,j)      * &
                & predictors % co(1,lev,prof)**3/predictors % co(8,lev,prof)**2 
   
            predictors_ad % co(1,lev,prof) = predictors_ad % co(1,lev,prof)                 + &
               & coef%n2o(lev,chan,12)*opticaldepth_ad(lev,j)      * &
               & 3*predictors % co(1,lev,prof)**2/predictors % co(8,lev,prof) 
  
            predictors_ad % co(1,lev,prof) = predictors_ad % co(1,lev,prof)                 + &
               & coef%n2o(lev,chan,11)*opticaldepth_ad(lev,j) 


            predictors_ad % n2o(1:10,lev,prof) =&
               & predictors_ad % n2o(1:10,lev,prof) +&
               & coef%n2o(lev,chan,1:10) * opticaldepth_ad(lev,j) 

          End Do

        else if(chanx>=2000.and.chanx<=2249.75)then
          
          Do lev=nlevels, 1, -1

             predictors_ad % n2o(13,lev,prof) = predictors_ad % n2o(13,lev,prof)               + &
                             & coef%n2o(lev,chan,14)*opticaldepth_ad(lev,j) 
       
             predictors_ad % n2o(12,lev,prof) = predictors_ad % n2o(12,lev,prof)              + &
                             & coef%n2o(lev,chan,13)*opticaldepth_ad(lev,j) 

             predictors_ad % co(8,lev,prof) = predictors_ad % co(8,lev,prof)                   - &
                             & coef%n2o(lev,chan,12)*opticaldepth_ad(lev,j)  * &
                             & predictors % co(1,lev,prof)**3/predictors % co(8,lev,prof)**2 

             predictors_ad % co(1,lev,prof) = predictors_ad % co(1,lev,prof)                 + &
                             & coef%n2o(lev,chan,12)*opticaldepth_ad(lev,j)     * &
                             & 3*predictors % co(1,lev,prof)**2 /predictors % co(8,lev,prof) 

             predictors_ad % co(1,lev,prof) = predictors_ad % co(1,lev,prof)                + &
                                    & coef%n2o(lev,chan,11)*opticaldepth_ad(lev,j) 

             predictors_ad % n2o(10,lev,prof) = predictors_ad % n2o(10,lev,prof)           + &
                                    & coef%n2o(lev,chan,10)*opticaldepth_ad(lev,j) 

             predictors_ad % n2o(11,lev,prof) = predictors_ad % n2o(11,lev,prof)           + &
                                    & coef%n2o(lev,chan,9)*opticaldepth_ad(lev,j) 

             predictors_ad % n2o(1:8,lev,prof) =&
                      & predictors_ad % n2o(1:8,lev,prof) +&
                      & coef%n2o(lev,chan,1:8) * opticaldepth_ad(lev,j) 

          End Do

        else if(chanx>=2380.25.and.chanx<=2660)then

          Do lev=nlevels, 1, -1

             predictors_ad % n2o(13,lev,prof) = predictors_ad % n2o(13,lev,prof)            + &
                                    & coef%n2o(lev,chan,12)*opticaldepth_ad(lev,j) 

             predictors_ad % n2o(12,lev,prof) = predictors_ad % n2o(12,lev,prof)               + &
                                    & coef%n2o(lev,chan,11)*opticaldepth_ad(lev,j) 

             predictors_ad % n2o(10,lev,prof) = predictors_ad % n2o(10,lev,prof)               + &
                                    & coef%n2o(lev,chan,10)*opticaldepth_ad(lev,j) 

             predictors_ad % n2o(11,lev,prof) = predictors_ad % n2o(11,lev,prof)              + &
                                    & coef%n2o(lev,chan,9)*opticaldepth_ad(lev,j) 


             predictors_ad % n2o(1:8,lev,prof) =&
                      & predictors_ad % n2o(1:8,lev,prof) +&
                      & coef%n2o(lev,chan,1:8) * opticaldepth_ad(lev,j) 

          End Do
        
        else
          Do lev=nlevels, 1, -1

             predictors_ad % n2o(1:10,lev,prof) =&
                      & predictors_ad % n2o(1:10,lev,prof) +&
                      & coef%n2o(lev,chan,1:10) * opticaldepth_ad(lev,j) 

          End Do
        endif
     End Do   
   End if

  !-----------
  !1.5 add CO2
  !-----------

  If ( coef%nco2 > 0 ) Then
     Do j = 1, nchannels
        chan = channels(j)
        prof = lprofiles(j)
        chanx= coef%ff_cwn(chan)

        if(chanx>=2000.and.chanx<=2249.75)then

          Do lev=nlevels, 1, -1

            predictors_ad % co2(15,lev,prof) = predictors_ad % co2(15,lev,prof)                 + &
                                     & coef%co2(lev,chan,14)*opticaldepth_ad(lev,j) 

            predictors_ad % co2(14,lev,prof) = predictors_ad % co2(14,lev,prof)                 + &
                                     & coef%co2(lev,chan,13)*opticaldepth_ad(lev,j) 

            predictors_ad % co2(13,lev,prof) = predictors_ad % co2(13,lev,prof)                 + &
                                     & coef%co2(lev,chan,12)*opticaldepth_ad(lev,j) 

            predictors_ad % co2(12,lev,prof) = predictors_ad % co2(12,lev,prof)                 + &
                                     & coef%co2(lev,chan,11)*opticaldepth_ad(lev,j) 

            predictors_ad % co2(11,lev,prof) = predictors_ad % co2(11,lev,prof)                 + &
                                    & coef%co2(lev,chan,10)*opticaldepth_ad(lev,j) 

            predictors_ad % co2(10,lev,prof) = predictors_ad % co2(10,lev,prof)                 + &
                                     & coef%co2(lev,chan,9)*opticaldepth_ad(lev,j) 

            predictors_ad % co(1,lev,prof) = predictors_ad % co(1,lev,prof)                   + &
                                      & coef%co2(lev,chan,8)*opticaldepth_ad(lev,j) 

            predictors_ad % co2(8,lev,prof) = predictors_ad % co2(8,lev,prof)                   + &
                                      & coef%co2(lev,chan,7)*opticaldepth_ad(lev,j) 

            predictors_ad % co2(7,lev,prof) = predictors_ad % co2(7,lev,prof)                   + &
                                      & coef%co2(lev,chan,6)*opticaldepth_ad(lev,j) 

            
            predictors_ad % co2(1:5,lev,prof) =&
                     & predictors_ad % co2(1:5,lev,prof) +&
                     & coef%co2(lev,chan,1:5) * opticaldepth_ad(lev,j) 
          enddo

        else if(chanx>=2250.and.chanx<=2295.or.chanx>=1995.and.chanx<=1999.75)then
       
          Do lev=nlevels, 1, -1

            predictors_ad % co2(10,lev,prof) = predictors_ad % co2(10,lev,prof)               + &
                                     & coef%co2(lev,chan,11)*opticaldepth_ad(lev,j) 

            predictors_ad % co(1,lev,prof) = predictors_ad % co(1,lev,prof)                    + &
                                    & coef%co2(lev,chan,10)*opticaldepth_ad(lev,j) 

            predictors_ad % co2(1:9,lev,prof) =&
                     & predictors_ad % co2(1:9,lev,prof) +&
                     & coef%co2(lev,chan,1:9) * opticaldepth_ad(lev,j) 
          enddo

        else if(chanx>=2295.25.and.chanx<=2380)then

          Do lev=nlevels, 1, -1
            predictors_ad % co2(1:10,lev,prof) =&
                     & predictors_ad % co2(1:10,lev,prof) +&
                     & coef%co2(lev,chan,1:10) * opticaldepth_ad(lev,j) 
          enddo

        else if(chanx>=2380.25.and.chanx<=2660)then

          Do lev=nlevels, 1, -1

            predictors_ad % co2(15,lev,prof) = predictors_ad % co2(15,lev,prof)                 + &
                                     & coef%co2(lev,chan,13)*opticaldepth_ad(lev,j) 

            predictors_ad % co2(14,lev,prof) = predictors_ad % co2(14,lev,prof)                 + &
                                     & coef%co2(lev,chan,12)*opticaldepth_ad(lev,j) 

            predictors_ad % co2(13,lev,prof) = predictors_ad % co2(13,lev,prof)                 + &
                                     & coef%co2(lev,chan,11)*opticaldepth_ad(lev,j) 

            predictors_ad % co2(12,lev,prof) = predictors_ad % co2(12,lev,prof)                 + &
                                    & coef%co2(lev,chan,10)*opticaldepth_ad(lev,j) 

            predictors_ad % co2(11,lev,prof) = predictors_ad % co2(11,lev,prof)                 + &
                                     & coef%co2(lev,chan,9)*opticaldepth_ad(lev,j) 

            predictors_ad % co2(10,lev,prof) = predictors_ad % co2(10,lev,prof)                 + &
                                      & coef%co2(lev,chan,8)*opticaldepth_ad(lev,j) 

            predictors_ad % co2(8,lev,prof) = predictors_ad % co2(8,lev,prof)                   + &
                                      & coef%co2(lev,chan,7)*opticaldepth_ad(lev,j) 

            predictors_ad % co2(7,lev,prof) = predictors_ad % co2(7,lev,prof)                   + &
                                      & coef%co2(lev,chan,6)*opticaldepth_ad(lev,j) 

            predictors_ad % co2(1:5,lev,prof) =&
                     & predictors_ad % co2(1:5,lev,prof) +&
                     & coef%co2(lev,chan,1:5) * opticaldepth_ad(lev,j) 
          enddo

        else if(chanx>=690.5.and.chanx<=1100)then

          Do lev=nlevels, 1, -1
            predictors_ad % co2(9,lev,prof)=predictors_ad % co2(9,lev,prof)               +   &
                                   coef%co2(lev,chan,9)*opticaldepth_ad(lev,j)

            predictors_ad % co2(8,lev,prof)=predictors_ad % co2(8,lev,prof)               +   &
                                   coef%co2(lev,chan,8)*opticaldepth_ad(lev,j)

            predictors_ad % co2(7,lev,prof)=predictors_ad % co2(7,lev,prof)               +   &
                                   coef%co2(lev,chan,7)*2 * predictors % co2(7,lev,prof)  *   &
                                   opticaldepth_ad(lev,j)

            predictors_ad % co2(6,lev,prof)=predictors_ad % co2(6,lev,prof)               +   &
                                   coef%co2(lev,chan,6)*2 * predictors % co2(6,lev,prof)  *   &
                                   opticaldepth_ad(lev,j)

            predictors_ad % co2(5,lev,prof)=predictors_ad % co2(5,lev,prof)               +   &
                                   coef%co2(lev,chan,5)*opticaldepth_ad(lev,j)

            predictors_ad % co2(3,lev,prof)=predictors_ad % co2(3,lev,prof)               +   &
                                   coef%co2(lev,chan,4)*2 *predictors % co2(3,lev,prof)   *   &
                                   opticaldepth_ad(lev,j)

            predictors_ad % co2(5,lev,prof)=predictors_ad % co2(5,lev,prof)               +   &
                                   coef%co2(lev,chan,3)*predictors % co2(6,lev,prof)**2   *   &
                                   opticaldepth_ad(lev,j)

            predictors_ad % co2(6,lev,prof)=predictors_ad % co2(6,lev,prof)               +   &
                                   coef%co2(lev,chan,3) *2*predictors % co2(6,lev,prof)   *   &
                                   predictors % co2(5,lev,prof)*opticaldepth_ad(lev,j)

            predictors_ad % co2(2,lev,prof)=predictors_ad % co2(2,lev,prof)               +   &
                                   coef%co2(lev,chan,2)*opticaldepth_ad(lev,j)

            predictors_ad % co2(1,lev,prof)=predictors_ad % co2(1,lev,prof)               +   &
                                   coef%co2(lev,chan,1)*opticaldepth_ad(lev,j)

          enddo

        
        else
         Do lev=nlevels, 1, -1

           predictors_ad % co2(1:9,lev,prof) =&
                     & predictors_ad % co2(1:9,lev,prof) +&
                     & coef%co2(lev,chan,1:9) * opticaldepth_ad(lev,j) 

          End Do
        endif
         
     End Do
  End If

  !------------------------------
  !1.4 add Water Vapour Continuum
  !------------------------------

  If ( coef%nwvcont > 0 ) Then
     Do j = 1, nchannels
        chan = channels(j)
        prof = lprofiles(j)
        Do lev=nlevels, 1, -1

           predictors_ad % wvcont(:,lev,prof) =&
                     & predictors_ad % wvcont(:,lev,prof) +&
                     & coef%wvcont(lev,chan,:) * opticaldepth_ad(lev,j)  
        End Do
     End Do
  End If

  !-------------
  !1.3 add ozone
  !-------------

  If ( coef%nozone > 0 ) Then
     Do j = 1, nchannels
        chan = channels(j)
        chanx= coef%ff_cwn(chan)
        prof = lprofiles(j)
        if((chanx>=2000.and.chanx<=2249.75).or.                                   &
         & (chanx>=2380.25.and.chanx<=2760))then 

          Do lev=nlevels, 1, -1
            predictors_ad % ozone(15,lev,prof) = predictors_ad % ozone(15,lev,prof)     + &
                                     & coef%ozone(lev,chan,13)*opticaldepth_ad(lev,j) 
            predictors_ad % ozone(14,lev,prof) = predictors_ad % ozone(14,lev,prof)     + &
                                     & coef%ozone(lev,chan,12)*opticaldepth_ad(lev,j) 
            predictors_ad % ozone(11,lev,prof) = predictors_ad % ozone(11,lev,prof)     + &
                                     & coef%ozone(lev,chan,11)*opticaldepth_ad(lev,j) 
            predictors_ad % ozone(10,lev,prof) = predictors_ad % ozone(10,lev,prof)     + &
                                     & coef%ozone(lev,chan,10)*opticaldepth_ad(lev,j) 
            predictors_ad % ozone(9,lev,prof) = predictors_ad % ozone(9,lev,prof)       + &
                                     & coef%ozone(lev,chan,9)*opticaldepth_ad(lev,j) 
            predictors_ad % ozone(13,lev,prof) = predictors_ad % ozone(13,lev,prof)     + &
                                     & coef%ozone(lev,chan,8)*opticaldepth_ad(lev,j) 
            predictors_ad % ozone(7,lev,prof) = predictors_ad % ozone(7,lev,prof)       + &
                                     & coef%ozone(lev,chan,7)*opticaldepth_ad(lev,j) 
            predictors_ad % ozone(6,lev,prof) = predictors_ad % ozone(6,lev,prof)       + &
                                    & coef%ozone(lev,chan,6)*opticaldepth_ad(lev,j) 
            predictors_ad % ozone(5,lev,prof) = predictors_ad % ozone(5,lev,prof)       + &
                                     & coef%ozone(lev,chan,5)*opticaldepth_ad(lev,j) 
            predictors_ad % ozone(4,lev,prof) = predictors_ad % ozone(4,lev,prof)       + &
                                     & coef%ozone(lev,chan,4)*opticaldepth_ad(lev,j) 
            predictors_ad % ozone(12,lev,prof) = predictors_ad % ozone(12,lev,prof)     + &
                                     & coef%ozone(lev,chan,3)*opticaldepth_ad(lev,j) 
            predictors_ad % ozone(2,lev,prof) = predictors_ad % ozone(2,lev,prof)       + &
                                     & coef%ozone(lev,chan,2)*opticaldepth_ad(lev,j) 
            predictors_ad % ozone(1,lev,prof) = predictors_ad % ozone(1,lev,prof)       + &
                                     & coef%ozone(lev,chan,1)*opticaldepth_ad(lev,j) 
          End Do

        else

          Do lev=nlevels, 1, -1
            predictors_ad % ozone(11,lev,prof) = predictors_ad % ozone(11,lev,prof)     + &
                                     & coef%ozone(lev,chan,11)*opticaldepth_ad(lev,j)
            predictors_ad % ozone(10,lev,prof) = predictors_ad % ozone(10,lev,prof)     + &
                                     & coef%ozone(lev,chan,10)*opticaldepth_ad(lev,j)
            predictors_ad % ozone(9,lev,prof) = predictors_ad % ozone(9,lev,prof)       + &
                                     & coef%ozone(lev,chan,9)*opticaldepth_ad(lev,j)
            predictors_ad % ozone(8,lev,prof) = predictors_ad % ozone(8,lev,prof)       + &
                                     & coef%ozone(lev,chan,8)*opticaldepth_ad(lev,j)
            predictors_ad % ozone(7,lev,prof) = predictors_ad % ozone(7,lev,prof)       + &
                                     & coef%ozone(lev,chan,7)*opticaldepth_ad(lev,j)
            predictors_ad % ozone(6,lev,prof) = predictors_ad % ozone(6,lev,prof)       + &
                                     & coef%ozone(lev,chan,6)*opticaldepth_ad(lev,j)
            predictors_ad % ozone(5,lev,prof) = predictors_ad % ozone(5,lev,prof)       + &
                                     & coef%ozone(lev,chan,5)*opticaldepth_ad(lev,j)
            predictors_ad % ozone(4,lev,prof) = predictors_ad % ozone(4,lev,prof)       + &
                                     & coef%ozone(lev,chan,4)*opticaldepth_ad(lev,j)
            predictors_ad % ozone(3,lev,prof) = predictors_ad % ozone(3,lev,prof)       + &
                                     & coef%ozone(lev,chan,3)*opticaldepth_ad(lev,j)
            predictors_ad % ozone(2,lev,prof) = predictors_ad % ozone(2,lev,prof)       + &
                                     & coef%ozone(lev,chan,2)*opticaldepth_ad(lev,j)
            predictors_ad % ozone(1,lev,prof) = predictors_ad % ozone(1,lev,prof)       + &
                                     & coef%ozone(lev,chan,1)*opticaldepth_ad(lev,j)
          End Do

        End If
     End Do
  End If

  !--------------------
  !1.2 add water vapour
  !--------------------

  Do j = 1, nchannels
     chan = channels(j)
     chanx= coef%ff_cwn(chan)
     prof = lprofiles(j)
     if(chanx>=1095.25.and.chanx<=1750.00)then
       Do lev=nlevels, 1, -1

         predictors_ad % ch4(1,lev,prof)=                                   &
         & predictors_ad % ch4(1,lev,prof)+opticaldepth_ad(lev,j)           * &
                                 & coef%watervapour(lev,chan,15)               * &
                                 & predictors % ch4(3,lev,prof) 

         predictors_ad % ch4(3,lev,prof)=                                   &
         & predictors_ad % ch4(3,lev,prof)+opticaldepth_ad(lev,j)           * &
                                 & coef%watervapour(lev,chan,15)               * &
                                 & predictors % ch4(1,lev,prof) 

         predictors_ad % ch4(1,lev,prof)=                                   &
         & predictors_ad % ch4(1,lev,prof)+opticaldepth_ad(lev,j)           * &
                                 & coef%watervapour(lev,chan,14) 

          predictors_ad % watervapour(13,lev,prof) =&
                    & predictors_ad % watervapour(13,lev,prof) +   &
                    & coef%watervapour(lev,chan,13) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(12,lev,prof) =&
                    & predictors_ad % watervapour(12,lev,prof) +   &
                    & coef%watervapour(lev,chan,12) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(11,lev,prof) =&
                    & predictors_ad % watervapour(11,lev,prof) +   &
                    & coef%watervapour(lev,chan,11) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(10,lev,prof) =&
                    & predictors_ad % watervapour(10,lev,prof) +   &
                    & coef%watervapour(lev,chan,10) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(9,lev,prof) =&
                    & predictors_ad % watervapour(9,lev,prof) +   &
                    & coef%watervapour(lev,chan,9) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(8,lev,prof) =&
                    & predictors_ad % watervapour(8,lev,prof) +   &
                    & coef%watervapour(lev,chan,8) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(7,lev,prof) =&
                    & predictors_ad % watervapour(7,lev,prof) +   &
                    & coef%watervapour(lev,chan,7) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(6,lev,prof) =&
                    & predictors_ad % watervapour(6,lev,prof) +   &
                    & coef%watervapour(lev,chan,6) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(5,lev,prof) =&
                    & predictors_ad % watervapour(5,lev,prof) +   &
                    & coef%watervapour(lev,chan,5) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(4,lev,prof) =&
                    & predictors_ad % watervapour(4,lev,prof) +   &
                    & coef%watervapour(lev,chan,4) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(3,lev,prof) =&
                    & predictors_ad % watervapour(3,lev,prof) +   &
                    & coef%watervapour(lev,chan,3) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(2,lev,prof) =&
                    & predictors_ad % watervapour(2,lev,prof) +   &
                    & coef%watervapour(lev,chan,2) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(1,lev,prof) =&
                    & predictors_ad % watervapour(1,lev,prof) +   &
                    & coef%watervapour(lev,chan,1) * opticaldepth_ad(lev,j) 
       End Do
     else if(chanx>=1900.25.and.chanx<=1994.75)then
 
       Do lev=nlevels, 1, -1
         predictors_ad % co2(1,lev,prof)=                                   &
         & predictors_ad % co2(1,lev,prof)+opticaldepth_ad(lev,j)            * &
                                 & coef%watervapour(lev,chan,14) 

          predictors_ad % watervapour(13,lev,prof) =&
                    & predictors_ad % watervapour(13,lev,prof) +   &
                    & coef%watervapour(lev,chan,13) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(12,lev,prof) =&
                    & predictors_ad % watervapour(12,lev,prof) +   &
                    & coef%watervapour(lev,chan,12) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(11,lev,prof) =&
                    & predictors_ad % watervapour(11,lev,prof) +   &
                    & coef%watervapour(lev,chan,11) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(10,lev,prof) =&
                    & predictors_ad % watervapour(10,lev,prof) +   &
                    & coef%watervapour(lev,chan,10) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(9,lev,prof) =&
                    & predictors_ad % watervapour(9,lev,prof) +   &
                    & coef%watervapour(lev,chan,9) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(8,lev,prof) =&
                    & predictors_ad % watervapour(8,lev,prof) +   &
                    & coef%watervapour(lev,chan,8) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(7,lev,prof) =&
                    & predictors_ad % watervapour(7,lev,prof) +   &
                    & coef%watervapour(lev,chan,7) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(6,lev,prof) =&
                    & predictors_ad % watervapour(6,lev,prof) +   &
                    & coef%watervapour(lev,chan,6) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(5,lev,prof) =&
                    & predictors_ad % watervapour(5,lev,prof) +   &
                    & coef%watervapour(lev,chan,5) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(4,lev,prof) =&
                    & predictors_ad % watervapour(4,lev,prof) +   &
                    & coef%watervapour(lev,chan,4) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(3,lev,prof) =&
                    & predictors_ad % watervapour(3,lev,prof) +   &
                    & coef%watervapour(lev,chan,3) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(2,lev,prof) =&
                    & predictors_ad % watervapour(2,lev,prof) +   &
                    & coef%watervapour(lev,chan,2) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(1,lev,prof) =&
                    & predictors_ad % watervapour(1,lev,prof) +   &
                    & coef%watervapour(lev,chan,1) * opticaldepth_ad(lev,j) 
       End Do

     else if(chanx>=1995.00.and.chanx<=1999.75)then

       Do lev=nlevels, 1, -1

         predictors_ad % co(1,lev,prof) = predictors_ad % co(1,lev,prof)               + &
                                      & coef%watervapour(lev,chan,15)*opticaldepth_ad(lev,j) 

         predictors_ad % co2(1,lev,prof) = predictors_ad % co2(1,lev,prof)             + &
                                      & coef%watervapour(lev,chan,14)*opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(13,lev,prof) =&
                    & predictors_ad % watervapour(13,lev,prof) +   &
                    & coef%watervapour(lev,chan,13) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(12,lev,prof) =&
                    & predictors_ad % watervapour(12,lev,prof) +   &
                    & coef%watervapour(lev,chan,12) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(11,lev,prof) =&
                    & predictors_ad % watervapour(11,lev,prof) +   &
                    & coef%watervapour(lev,chan,11) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(10,lev,prof) =&
                    & predictors_ad % watervapour(10,lev,prof) +   &
                    & coef%watervapour(lev,chan,10) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(9,lev,prof) =&
                    & predictors_ad % watervapour(9,lev,prof) +   &
                    & coef%watervapour(lev,chan,9) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(8,lev,prof) =&
                    & predictors_ad % watervapour(8,lev,prof) +   &
                    & coef%watervapour(lev,chan,8) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(7,lev,prof) =&
                    & predictors_ad % watervapour(7,lev,prof) +   &
                    & coef%watervapour(lev,chan,7) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(6,lev,prof) =&
                    & predictors_ad % watervapour(6,lev,prof) +   &
                    & coef%watervapour(lev,chan,6) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(5,lev,prof) =&
                    & predictors_ad % watervapour(5,lev,prof) +   &
                    & coef%watervapour(lev,chan,5) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(4,lev,prof) =&
                    & predictors_ad % watervapour(4,lev,prof) +   &
                    & coef%watervapour(lev,chan,4) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(3,lev,prof) =&
                    & predictors_ad % watervapour(3,lev,prof) +   &
                    & coef%watervapour(lev,chan,3) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(2,lev,prof) =&
                    & predictors_ad % watervapour(2,lev,prof) +   &
                    & coef%watervapour(lev,chan,2) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(1,lev,prof) =&
                    & predictors_ad % watervapour(1,lev,prof) +   &
                    & coef%watervapour(lev,chan,1) * opticaldepth_ad(lev,j) 
       End Do

     else if(chanx>=2000.and.chanx<=2249.75)then

       if(coef%ss_val_chn(chan)==1)then

         Do lev=nlevels, 1, -1

           predictors_ad % co(1,lev,prof) = predictors_ad % co(1,lev,prof)              + &
                       & coef%watervapour(lev,chan,15)*opticaldepth_ad(lev,j) 
           predictors_ad % co2(1,lev,prof) = predictors_ad % co2(1,lev,prof)            + &
                       & coef%watervapour(lev,chan,14)*opticaldepth_ad(lev,j) 
           predictors_ad % watervapour(17,lev,prof) =&
                       &  predictors_ad % watervapour(17,lev,prof)                    + &
                       & coef%watervapour(lev,chan,13)*opticaldepth_ad(lev,j) 
           predictors_ad % watervapour(16,lev,prof) =&
                       &  predictors_ad % watervapour(16,lev,prof)                    + &
                      & coef%watervapour(lev,chan,12)*opticaldepth_ad(lev,j) 
           predictors_ad % watervapour(11,lev,prof) =&
                       &  predictors_ad % watervapour(11,lev,prof)                    + &
                       & coef%watervapour(lev,chan,11)*opticaldepth_ad(lev,j) 
           predictors_ad % watervapour(10,lev,prof) =&
                       &  predictors_ad % watervapour(10,lev,prof)                    + &
                       & coef%watervapour(lev,chan,10)*opticaldepth_ad(lev,j) 
           predictors_ad % watervapour(15,lev,prof) =&
                       &  predictors_ad % watervapour(15,lev,prof)                   + &
                       & coef%watervapour(lev,chan,9)*opticaldepth_ad(lev,j) 
           predictors_ad % watervapour(14,lev,prof) =&
                       &  predictors_ad % watervapour(14,lev,prof)                   + &
                       & coef%watervapour(lev,chan,8)*opticaldepth_ad(lev,j) 
            predictors_ad % watervapour(7,lev,prof) =&
                    & predictors_ad % watervapour(7,lev,prof) +   &
                    & coef%watervapour(lev,chan,7) * opticaldepth_ad(lev,j) 
            predictors_ad % watervapour(6,lev,prof) =&
                    & predictors_ad % watervapour(6,lev,prof) +   &
                    & coef%watervapour(lev,chan,6) * opticaldepth_ad(lev,j) 
            predictors_ad % watervapour(5,lev,prof) =&
                    & predictors_ad % watervapour(5,lev,prof) +   &
                    & coef%watervapour(lev,chan,5) * opticaldepth_ad(lev,j) 
            predictors_ad % watervapour(4,lev,prof) =&
                    & predictors_ad % watervapour(4,lev,prof) +   &
                    & coef%watervapour(lev,chan,4) * opticaldepth_ad(lev,j) 
            predictors_ad % watervapour(3,lev,prof) =&
                    & predictors_ad % watervapour(3,lev,prof) +   &
                    & coef%watervapour(lev,chan,3) * opticaldepth_ad(lev,j) 
            predictors_ad % watervapour(2,lev,prof) =&
                    & predictors_ad % watervapour(2,lev,prof) +   &
                    & coef%watervapour(lev,chan,2) * opticaldepth_ad(lev,j) 
            predictors_ad % watervapour(1,lev,prof) =&
                    & predictors_ad % watervapour(1,lev,prof) +   &
                    & coef%watervapour(lev,chan,1) * opticaldepth_ad(lev,j) 
         End Do


       else

         Do lev=nlevels, 1, -1

           predictors_ad % co(1,lev,prof) = predictors_ad % co(1,lev,prof)            + &
                                      & coef%watervapour(lev,chan,15)*opticaldepth_ad(lev,j)

           predictors_ad % co2(1,lev,prof) = predictors_ad % co2(1,lev,prof)          + &
                                      & coef%watervapour(lev,chan,14)*opticaldepth_ad(lev,j)
           predictors_ad % watervapour(13,lev,prof) =&
                    & predictors_ad % watervapour(13,lev,prof) +   &
                    & coef%watervapour(lev,chan,13) * opticaldepth_ad(lev,j)
           predictors_ad % watervapour(12,lev,prof) =&
                    & predictors_ad % watervapour(12,lev,prof) +   &
                    & coef%watervapour(lev,chan,12) * opticaldepth_ad(lev,j)
           predictors_ad % watervapour(11,lev,prof) =&
                    & predictors_ad % watervapour(11,lev,prof) +   &
                    & coef%watervapour(lev,chan,11) * opticaldepth_ad(lev,j)
           predictors_ad % watervapour(10,lev,prof) =&
                    & predictors_ad % watervapour(10,lev,prof) +   &
                    & coef%watervapour(lev,chan,10) * opticaldepth_ad(lev,j)
           predictors_ad % watervapour(9,lev,prof) =&
                    & predictors_ad % watervapour(9,lev,prof) +   &
                    & coef%watervapour(lev,chan,9) * opticaldepth_ad(lev,j)
           predictors_ad % watervapour(8,lev,prof) =&
                    & predictors_ad % watervapour(8,lev,prof) +   &
                    & coef%watervapour(lev,chan,8) * opticaldepth_ad(lev,j)
           predictors_ad % watervapour(7,lev,prof) =&
                    & predictors_ad % watervapour(7,lev,prof) +   &
                    & coef%watervapour(lev,chan,7) * opticaldepth_ad(lev,j)
           predictors_ad % watervapour(6,lev,prof) =&
                    & predictors_ad % watervapour(6,lev,prof) +   &
                    & coef%watervapour(lev,chan,6) * opticaldepth_ad(lev,j)
           predictors_ad % watervapour(5,lev,prof) =&
                    & predictors_ad % watervapour(5,lev,prof) +   &
                    & coef%watervapour(lev,chan,5) * opticaldepth_ad(lev,j)
           predictors_ad % watervapour(4,lev,prof) =&
                    & predictors_ad % watervapour(4,lev,prof) +   &
                    & coef%watervapour(lev,chan,4) * opticaldepth_ad(lev,j)
           predictors_ad % watervapour(3,lev,prof) =&
                    & predictors_ad % watervapour(3,lev,prof) +   &
                    & coef%watervapour(lev,chan,3) * opticaldepth_ad(lev,j)
           predictors_ad % watervapour(2,lev,prof) =&
                    & predictors_ad % watervapour(2,lev,prof) +   &
                    & coef%watervapour(lev,chan,2) * opticaldepth_ad(lev,j)
           predictors_ad % watervapour(1,lev,prof) =&
                    & predictors_ad % watervapour(1,lev,prof) +   &
                    & coef%watervapour(lev,chan,1) * opticaldepth_ad(lev,j)
         End Do


       End if


     else if(chanx>=2250.and.chanx<=2295)then

       Do lev=nlevels, 1, -1

         predictors_ad % co(1,lev,prof) = predictors_ad % co(1,lev,prof)            + &
                                      & coef%watervapour(lev,chan,15)*opticaldepth_ad(lev,j) 

         predictors_ad % co2(1,lev,prof) = predictors_ad % co2(1,lev,prof)          + &
                                      & coef%watervapour(lev,chan,14)*opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(13,lev,prof) =&
                    & predictors_ad % watervapour(13,lev,prof) +   &
                    & coef%watervapour(lev,chan,13) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(12,lev,prof) =&
                    & predictors_ad % watervapour(12,lev,prof) +   &
                    & coef%watervapour(lev,chan,12) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(11,lev,prof) =&
                    & predictors_ad % watervapour(11,lev,prof) +   &
                    & coef%watervapour(lev,chan,11) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(10,lev,prof) =&
                    & predictors_ad % watervapour(10,lev,prof) +   &
                    & coef%watervapour(lev,chan,10) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(9,lev,prof) =&
                    & predictors_ad % watervapour(9,lev,prof) +   &
                    & coef%watervapour(lev,chan,9) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(8,lev,prof) =&
                    & predictors_ad % watervapour(8,lev,prof) +   &
                    & coef%watervapour(lev,chan,8) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(7,lev,prof) =&
                    & predictors_ad % watervapour(7,lev,prof) +   &
                    & coef%watervapour(lev,chan,7) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(6,lev,prof) =&
                    & predictors_ad % watervapour(6,lev,prof) +   &
                    & coef%watervapour(lev,chan,6) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(5,lev,prof) =&
                    & predictors_ad % watervapour(5,lev,prof) +   &
                    & coef%watervapour(lev,chan,5) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(4,lev,prof) =&
                    & predictors_ad % watervapour(4,lev,prof) +   &
                    & coef%watervapour(lev,chan,4) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(3,lev,prof) =&
                    & predictors_ad % watervapour(3,lev,prof) +   &
                    & coef%watervapour(lev,chan,3) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(2,lev,prof) =&
                    & predictors_ad % watervapour(2,lev,prof) +   &
                    & coef%watervapour(lev,chan,2) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(1,lev,prof) =&
                    & predictors_ad % watervapour(1,lev,prof) +   &
                    & coef%watervapour(lev,chan,1) * opticaldepth_ad(lev,j) 
       End Do

     else if(chanx>=2295.25.and.chanx<=2380)then
 
      Do lev=nlevels, 1, -1
          predictors_ad % watervapour(13,lev,prof) =&
                    & predictors_ad % watervapour(13,lev,prof) +   &
                    & coef%watervapour(lev,chan,13) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(12,lev,prof) =&
                    & predictors_ad % watervapour(12,lev,prof) +   &
                    & coef%watervapour(lev,chan,12) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(11,lev,prof) =&
                    & predictors_ad % watervapour(11,lev,prof) +   &
                    & coef%watervapour(lev,chan,11) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(10,lev,prof) =&
                    & predictors_ad % watervapour(10,lev,prof) +   &
                    & coef%watervapour(lev,chan,10) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(9,lev,prof) =&
                    & predictors_ad % watervapour(9,lev,prof) +   &
                    & coef%watervapour(lev,chan,9) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(8,lev,prof) =&
                    & predictors_ad % watervapour(8,lev,prof) +   &
                    & coef%watervapour(lev,chan,8) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(7,lev,prof) =&
                    & predictors_ad % watervapour(7,lev,prof) +   &
                    & coef%watervapour(lev,chan,7) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(6,lev,prof) =&
                    & predictors_ad % watervapour(6,lev,prof) +   &
                    & coef%watervapour(lev,chan,6) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(5,lev,prof) =&
                    & predictors_ad % watervapour(5,lev,prof) +   &
                    & coef%watervapour(lev,chan,5) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(4,lev,prof) =&
                    & predictors_ad % watervapour(4,lev,prof) +   &
                    & coef%watervapour(lev,chan,4) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(3,lev,prof) =&
                    & predictors_ad % watervapour(3,lev,prof) +   &
                    & coef%watervapour(lev,chan,3) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(2,lev,prof) =&
                    & predictors_ad % watervapour(2,lev,prof) +   &
                    & coef%watervapour(lev,chan,2) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(1,lev,prof) =&
                    & predictors_ad % watervapour(1,lev,prof) +   &
                    & coef%watervapour(lev,chan,1) * opticaldepth_ad(lev,j) 
      End Do

     else if(chanx>=2380.25.and.chanx<=2760)then

       Do lev=nlevels, 1, -1

         predictors_ad % ch4(3,lev,prof) = predictors_ad % ch4(3,lev,prof)           + &
                                      & coef%watervapour(lev,chan,15)               * &
                                   & predictors % ch4(1,lev,prof)**0.25*opticaldepth_ad(lev,j) 
         predictors_ad % ch4(1,lev,prof) = predictors_ad % ch4(1,lev,prof)           + &
                                      & coef%watervapour(lev,chan,15)*0.25          * &
                                      & predictors % ch4(1,lev,prof)**(-0.75)        * &
                                      & predictors % ch4(3,lev,prof)*opticaldepth_ad(lev,j) 
         predictors_ad % ch4(1,lev,prof) = predictors_ad % ch4(1,lev,prof)           + &
                                      & coef%watervapour(lev,chan,14)*1.25          * &
                                   & predictors % ch4(1,lev,prof)**0.25*opticaldepth_ad(lev,j) 
         predictors_ad % watervapour(19,lev,prof) =&
                          &  predictors_ad % watervapour(19,lev,prof)                    + &
                          & coef%watervapour(lev,chan,13)*opticaldepth_ad(lev,j) 
         predictors_ad % watervapour(16,lev,prof) =&
                          &  predictors_ad % watervapour(16,lev,prof)                    + &
                          & coef%watervapour(lev,chan,12)*opticaldepth_ad(lev,j) 
         predictors_ad % watervapour(11,lev,prof) =&
                          &  predictors_ad % watervapour(11,lev,prof)                    + &
                          & coef%watervapour(lev,chan,11)*opticaldepth_ad(lev,j) 
         predictors_ad % watervapour(18,lev,prof) =&
                          &  predictors_ad % watervapour(18,lev,prof)                    + &
                          & coef%watervapour(lev,chan,10)*opticaldepth_ad(lev,j) 
         predictors_ad % watervapour(15,lev,prof) =&
                          &  predictors_ad % watervapour(15,lev,prof)                    + &
                          & coef%watervapour(lev,chan,9)*opticaldepth_ad(lev,j) 
         predictors_ad % watervapour(14,lev,prof) =&
                          &  predictors_ad % watervapour(14,lev,prof)                    + &
                          & coef%watervapour(lev,chan,8)*opticaldepth_ad(lev,j) 
         predictors_ad % watervapour(7,lev,prof) =&
                    & predictors_ad % watervapour(7,lev,prof) +   &
                    & coef%watervapour(lev,chan,7) * opticaldepth_ad(lev,j) 
         predictors_ad % watervapour(6,lev,prof) =&
                    & predictors_ad % watervapour(6,lev,prof) +   &
                    & coef%watervapour(lev,chan,6) * opticaldepth_ad(lev,j) 
         predictors_ad % watervapour(5,lev,prof) =&
                    & predictors_ad % watervapour(5,lev,prof) +   &
                    & coef%watervapour(lev,chan,5) * opticaldepth_ad(lev,j) 
         predictors_ad % watervapour(4,lev,prof) =&
                    & predictors_ad % watervapour(4,lev,prof) +   &
                    & coef%watervapour(lev,chan,4) * opticaldepth_ad(lev,j) 
         predictors_ad % watervapour(3,lev,prof) =&
                    & predictors_ad % watervapour(3,lev,prof) +   &
                    & coef%watervapour(lev,chan,3) * opticaldepth_ad(lev,j) 
         predictors_ad % watervapour(2,lev,prof) =&
                    & predictors_ad % watervapour(2,lev,prof) +   &
                    & coef%watervapour(lev,chan,2) * opticaldepth_ad(lev,j) 
         predictors_ad % watervapour(1,lev,prof) =&
                    & predictors_ad % watervapour(1,lev,prof) +   &
                    & coef%watervapour(lev,chan,1) * opticaldepth_ad(lev,j) 
       End Do

     else
       Do lev=nlevels, 1, -1
          predictors_ad % watervapour(13,lev,prof) =&
                    & predictors_ad % watervapour(13,lev,prof) +   &
                    & coef%watervapour(lev,chan,13) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(12,lev,prof) =&
                    & predictors_ad % watervapour(12,lev,prof) +   &
                    & coef%watervapour(lev,chan,12) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(11,lev,prof) =&
                    & predictors_ad % watervapour(11,lev,prof) +   &
                    & coef%watervapour(lev,chan,11) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(10,lev,prof) =&
                    & predictors_ad % watervapour(10,lev,prof) +   &
                    & coef%watervapour(lev,chan,10) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(9,lev,prof) =&
                    & predictors_ad % watervapour(9,lev,prof) +   &
                    & coef%watervapour(lev,chan,9) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(8,lev,prof) =&
                    & predictors_ad % watervapour(8,lev,prof) +   &
                    & coef%watervapour(lev,chan,8) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(7,lev,prof) =&
                    & predictors_ad % watervapour(7,lev,prof) +   &
                    & coef%watervapour(lev,chan,7) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(6,lev,prof) =&
                    & predictors_ad % watervapour(6,lev,prof) +   &
                    & coef%watervapour(lev,chan,6) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(5,lev,prof) =&
                    & predictors_ad % watervapour(5,lev,prof) +   &
                    & coef%watervapour(lev,chan,5) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(4,lev,prof) =&
                    & predictors_ad % watervapour(4,lev,prof) +   &
                    & coef%watervapour(lev,chan,4) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(3,lev,prof) =&
                    & predictors_ad % watervapour(3,lev,prof) +   &
                    & coef%watervapour(lev,chan,3) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(2,lev,prof) =&
                    & predictors_ad % watervapour(2,lev,prof) +   &
                    & coef%watervapour(lev,chan,2) * opticaldepth_ad(lev,j) 
          predictors_ad % watervapour(1,lev,prof) =&
                    & predictors_ad % watervapour(1,lev,prof) +   &
                    & coef%watervapour(lev,chan,1) * opticaldepth_ad(lev,j) 
       End Do
     endif
  End Do

  !--------------------------
  !1.1 start with mixed gases
  !--------------------------

  Do j = 1, nchannels
     chan = channels(j)
     chanx= coef%ff_cwn(chan)
     prof = lprofiles(j)

     if((chanx>=2000.and.chanx<=2249.75).or.                                             &
         & (chanx>=2380.25.and.chanx<=2760))then 

       Do lev=nlevels, 1, -1
         predictors_ad % mixedgas(10,lev,prof) =                                      &
                                     & predictors_ad % mixedgas(10,lev,prof)          + &
                                     & coef%mixedgas(lev,chan,10)*opticaldepth_ad(lev,j) 
         predictors_ad % mixedgas(9,lev,prof) =                                       &
                                   & predictors_ad % mixedgas(9,lev,prof)             + &
                                    & coef%mixedgas(lev,chan,9)*opticaldepth_ad(lev,j) 
         predictors_ad % mixedgas(8,lev,prof) =                                       &
                                   & predictors_ad % mixedgas(8,lev,prof)             + &
                                    & coef%mixedgas(lev,chan,8)*opticaldepth_ad(lev,j) 
         predictors_ad % mixedgas(7,lev,prof) =                                       &
                                   & predictors_ad % mixedgas(7,lev,prof)             + &
                                    & coef%mixedgas(lev,chan,7)*opticaldepth_ad(lev,j) 
         predictors_ad % mixedgas(6,lev,prof) =                                       &
                                   & predictors_ad % mixedgas(6,lev,prof)             + &
                                    & coef%mixedgas(lev,chan,6)*opticaldepth_ad(lev,j) 
         predictors_ad % mixedgas(5,lev,prof) =                                       &
                                   & predictors_ad % mixedgas(5,lev,prof)             + &
                                    & coef%mixedgas(lev,chan,5)*opticaldepth_ad(lev,j) 
         predictors_ad % mixedgas(4,lev,prof) =                                       &
                                   & predictors_ad % mixedgas(4,lev,prof)             + &
                                    & coef%mixedgas(lev,chan,4)*opticaldepth_ad(lev,j) 
         predictors_ad % mixedgas(3,lev,prof) =                                       &
                                   & predictors_ad % mixedgas(3,lev,prof)             + &
                                    & coef%mixedgas(lev,chan,3)*opticaldepth_ad(lev,j) 
         predictors_ad % mixedgas(2,lev,prof) =                                       &
                                   & predictors_ad % mixedgas(2,lev,prof)             + &
                                    & coef%mixedgas(lev,chan,2)*opticaldepth_ad(lev,j) 
         predictors_ad % mixedgas(1,lev,prof) =                                       &
                                   & predictors_ad % mixedgas(1,lev,prof)             + &
                                    & coef%mixedgas(lev,chan,1)*opticaldepth_ad(lev,j) 
       End Do

     else

       Do lev=nlevels, 1, -1
         predictors_ad % mixedgas(8,lev,prof) =                                       &
                                   & predictors_ad % mixedgas(8,lev,prof)             + &
                                    & coef%mixedgas(lev,chan,8)*opticaldepth_ad(lev,j)
         predictors_ad % mixedgas(7,lev,prof) =                                       &
                                   & predictors_ad % mixedgas(7,lev,prof)             + &
                                    & coef%mixedgas(lev,chan,7)*opticaldepth_ad(lev,j)
         predictors_ad % mixedgas(6,lev,prof) =                                       &
                                   & predictors_ad % mixedgas(6,lev,prof)             + &
                                    & coef%mixedgas(lev,chan,6)*opticaldepth_ad(lev,j)
         predictors_ad % mixedgas(5,lev,prof) =                                       &
                                   & predictors_ad % mixedgas(5,lev,prof)             + &
                                    & coef%mixedgas(lev,chan,5)*opticaldepth_ad(lev,j)
         predictors_ad % mixedgas(4,lev,prof) =                                       &
                                   & predictors_ad % mixedgas(4,lev,prof)             + &
                                    & coef%mixedgas(lev,chan,4)*opticaldepth_ad(lev,j)
         predictors_ad % mixedgas(3,lev,prof) =                                       &
                                   & predictors_ad % mixedgas(3,lev,prof)             + &
                                    & coef%mixedgas(lev,chan,3)*opticaldepth_ad(lev,j)
         predictors_ad % mixedgas(2,lev,prof) =                                       &
                                   & predictors_ad % mixedgas(2,lev,prof)             + &
                                    & coef%mixedgas(lev,chan,2)*opticaldepth_ad(lev,j)
         predictors_ad % mixedgas(1,lev,prof) =                                       &
                                   & predictors_ad % mixedgas(1,lev,prof)             + &
                                    & coef%mixedgas(lev,chan,1)*opticaldepth_ad(lev,j)
       End Do

     endif
  End Do
IF (LHOOK) CALL DR_HOOK('RTTOV_OPDEP_9_AD',1_jpim,ZHOOK_HANDLE)

End Subroutine rttov_opdep_9_ad
