!     Compute the fraction of solar radiance that is reflected by a wind
!     roughened water surface. 
      subroutine rttov_refsun(                                                 &
      nprofiles,                                                               &
      profiles,                                                                &
      coef,                                                                    &
      aux,                                                                     &
      sunglint,                                                                &
      raytracing)                                                              


!     Description:
!     The water surface is regarded as a collection of small mirror-like facets
!     each randomly tilted with respect to the local horizon. A time passes, the
!     tilt of a facet varies under the influence of the wind. When the open 
!     ocean reflects the solar disk, these fluctuating facets produce a dancing
!     pattern known as sun glint.
!     The fraction of solar radiance reflected by a wind roughened water surface
!     is computed assuming that the slope of the facets obey to a 
!     two-dimensional Gaussian random process  whose spectrum is specified by
!     the Joint North Sea Wave Project (JONSWAP) wave spectral model. 
!     The total variance of the slope of the facet is obtained from the  
!     frequency spectrum of the surface wave and the inverse function of the 
!     dispersive relation of the full-gravity-capillary wave.
!     The shadowing of the surface of the facets on the backsides of the waves 
!     and deep in the throughs between waves is considered.
!     A coordinate system is used where the average water surface lies in the
!     X-Y plane. The coordinate system is right-handed and is located at the 
!     reflection point. The Z axis points toward the zenith and the X axis
!     points in the direction formed by the projection of the reflected ray
!     on the average water surface. 
!
!     Method:
!     K. Yoshimori, K. Itoh and Y. Ichioka:'Optical charateristics of a
!     wind-roughened water surface:a two dimensional theory'.Applied Optics,
!     Vol.34,No.27,20 September 1995, pp.6236-6247.
!
!     Owner:
!     EUMETSAT
!
!     History:
!     Version      Date        Comment
!     1            15/07/2003  Oroginal code. RTIASI-4. Marco Matricardi. ECMWF.
!
!     Code description:
!       Language:              Fortran 90.
!       Software Standards:    "European Standards for Writing and Documenting
!                              Exchangeable Fortran 90 code".
!
!
!     Module used:

  Use rttov_const, Only : &
       & deg2rad      , &
       & pi           , &
       & gravity      , &
       & surftype_land, &
       & surftype_seaice,&
       & surftype_sea
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK




      Use rttov_types, Only :   &
!     Imported Type Definitions:
     & profile_aux          ,   &
     & profile_Type         ,   &
     & raytracing_type      ,   &
     & sunglint_type        ,   &
     & rttov_coef


      Use parkind1   , Only :   &
!     Imported parameters:
     &jpim,jprb


USE PARKIND1  ,ONLY : JPRB


      IMPLICIT NONE


!     Subroutine arguments:


!       Scalar arguments with intent in:
      integer(kind=jpim), intent(in):: nprofiles           ! Number of processed profiles

!       Array arguments with intent in:
                                                ! the inclusion of solar
                                                ! radiation

      type(profile_type)    ,intent(in)   :: profiles   (nprofiles)
      type(profile_aux)     ,intent(in)   :: aux        (nprofiles)
      type(raytracing_type) ,intent(in)   :: raytracing
      type(sunglint_type)   ,intent(inout):: sunglint   (nprofiles)
      type(rttov_coef)      ,Intent(in)   :: coef 

!INTF

!     End of subroutine arguments
#include "rttov_erfcx.interface"

!       Local arrays
      real  (Kind=jprb) ::  ff (coef%ws_nomega)   ! Working space
      real  (Kind=jprb) ::  q_shad_a(nprofiles)
      reaL  (Kind=jprb) ::  q_shad_b(nprofiles)

!       Local scalars:
      integer(Kind=jpim) :: j,jj,i,np,ilevsur,ilevwrt

      real  (Kind=jprb) :: rqs               ! Effective distribution function 
      real  (Kind=jprb) :: theta_fi          ! First order shadowing factor for the reflected 
                                             ! ray
      real  (Kind=jprb) :: theta_csi         ! First ordesr shadowing factor for the incident 
                                             ! ray
                                             ! surface
      real  (Kind=jprb) :: omega_1           ! Frequency of the surface wave
      real  (Kind=jprb) :: k
      real  (Kind=jprb) :: sigma_a
      real  (Kind=jprb) :: sigma_b
      real  (Kind=jprb) :: sigma
      real  (Kind=jprb) :: bb,aa,hinc,s2,s4,sa,sb,fac,epsilon
!      real  (Kind=jprb) :: RTTOV_ERFCX
REAL(KIND=JPRB) :: ZHOOK_HANDLE



!-----End of header-------------------------------------------------------------



IF (LHOOK) CALL DR_HOOK('RTTOV_REFSUN',0_jpim,ZHOOK_HANDLE)
      epsilon  = deg2rad*0.2659_jprb        
      fac      = sqrt(2_jpim*pi)

      sunglint(:)%glint = 0._jprb
      sunglint(:)%omega = 0._jprb


      do j=1,nprofiles

        if(profiles(j)%addsolar)then

          if(profiles(j)%skin%surftype==surftype_sea)then  !Water surface type

!---------Compute the angle between the wind direction and the U axis-------

          if( profiles(j)%s2m%u >0._jprb.and. profiles(j)%s2m%v >=0._jprb)then

            sunglint(j)%wangl    =atan(profiles(j)%s2m%v/profiles(j)%s2m%u)

          else if( profiles(j)%s2m%u <0._jprb.and. profiles(j)%s2m%v >=0._jprb)then

            sunglint(j)%wangl    =pi+atan(profiles(j)%s2m%v/profiles(j)%s2m%u)

          else if( profiles(j)%s2m%u <0._jprb.and.profiles(j)%s2m%v <=0._jprb)then

            sunglint(j)%wangl    =pi+atan(profiles(j)%s2m%v/profiles(j)%s2m%u)

          else if( profiles(j)%s2m%u >=0._jprb.and. profiles(j)%s2m%v <0._jprb)then

            sunglint(j)%wangl    =2._jprb*pi+atan(profiles(j)%s2m%v/profiles(j)%s2m%u)

          else if( profiles(j)%s2m%u ==0._jprb.and. profiles(j)%s2m%v >0._jprb)then     ! N.B. This doesn't work     

            !sunglint(j)%wangl = atan(profiles(j)%s2m%v/profiles(j)%s2m%u)   ! when u = 0.0  
            sunglint(j)%wangl    =0._jprb

          else if( profiles(j)%s2m%u ==0._jprb.and. profiles(j)%s2m%v <0._jprb)then

            !sunglint(j)%wangl    =2*pi+atan(profiles(j)%s2m%v/profiles(j)%s2m%u)
            sunglint(j)%wangl    =0._jprb

          else

            sunglint(j)%wangl    =0._jprb

          endif


!-----------Compute the difference between the sun azimuth angle and the-------
!           azimuth angle of the direction formed by the projection on the
!           mean water surface of the surface-to-sun direction

            sunglint(j)%dazng=profiles(j)%sunazangle-profiles(j)%azangle

            if(sunglint(j)%dazng<0)then
              sunglint(j)%dazng=sunglint(j)%dazng+360._jprb
            ENDIF


!-----------Compute the wind speed----------------------------------------------

            sunglint(j)%windsp=sqrt(profiles(j)%s2m%u**2_jpim+profiles(j)%s2m%v**2_jpim)


!-----------Find the index of the pressure level that is closest to the surface-

            ilevwrt=aux(j) % nearestlev_surf

            if (aux(j) %pfraction_surf < 0._jprb) then
              ilevwrt=ilevwrt+1
            end if

            if(ilevwrt.gt.100)then
              ilevsur=ilevwrt-1
            else
              ilevsur=ilevwrt
            ENDIF

!-----------Compute the secant of the satellite viewing angle at the surface----

            sunglint(j)%zensat=acos(1._jprb/raytracing%pathsat(ilevsur,j))


!-----------Compute the secant of the sun zenith angle at the surface-----------

            sunglint(j)%zensun=acos(1._jprb/raytracing%pathsun(ilevsur,j))


            if(sunglint(j)%windsp==0._jprb)then

!-----------In absence of wind the water surface is a mirror-like surface-------

              IF(sunglint(j)%zensat==sunglint(j)%zensun.and.sunglint(j)%dazng==pi)then
                sunglint(j)%glint=1._jprb
              ELSE
                sunglint(j)%glint=0._jprb
              ENDIF

            else if (sunglint(j)%windsp/=0._jprb)then


!-----------Set up some parameters used to define the JONSWAP frequency -------
!           spectrum of the surface wave                                       |
!------------------------------------------------------------------------------

            sunglint(j)%x_u     = profiles(j)%s2m%wfetc*gravity/sunglint(j)%windsp**2_jpim
            sunglint(j)%alfa1   = 0.076_jprb/sunglint(j)%x_u**0.22_jprb
            k                      = 3.3_jprb
            sigma_a                = 0.07_jprb
            sigma_b                = 0.09_jprb
            sunglint(j)%omega_m = 2._jprb*pi*(3.5_jprb*(gravity/sunglint(j)%windsp)  * &
                                     (1._jprb/sunglint(j)%x_u**0.33_jprb))


!-----------Compute the total variance of the slope of the facet---------------
!                                                                              |
!           To compute the total variance of the slope of the facet (or the    |
!           total variance of the displacement of the water surface) the       |
!           knowledge of the frequency spectrum of the surface wave and the    |
!           inverse function of the dispersive relation of the full-gravity-   |
!           capillary wave is needed.                                          |
!                                                                              |
!           The dispersive relation of the full-gravity-capillary wave is      |
!           written as:                                                        |
!                                                                              |
!           omega_1(k)=k*sqrt((gravity/k+gamma*k/ro)*tanh(h*k))                   |
!                                                                              |
!           k    = wave-number                                                 |
!           gamma= surface-tension constant                                    |
!           ro   = density of water                                            |
!           h    = water depth                                                 |
!                                                                              |
!           The inverse function of the dispersive relation of the             |
!           full-gravity-capillary wave has been pre-computed numerically      |
!           using the following parameters:                                    |
!                                                                              |
!           gamma = 0.00735  [N/m]                                             |
!           ro    = 999.1    [kg/m^3]                                          |
!           h     = 50       [m]                                               |
!                                                                              |
!           The frequency spectrum (PSI) of the surface wave is computed       |
!           using the JONSWAP wave-spectral model.                             |
!                                                                              |
!           The total variance of the slope is then evaluated by integrating   |
!           the quantity (psi*k_omega**2):                                     |
!                                                                              |
!                     -infinity                                                |
!                    -                                                         |
!           gamma_sq=- (k_omega(omega_1)**2*psi(omega_1))  d omega_1           |
!                    -                                                         |
!                   -zero                                                      |
!                                                                              |
!           The integral is computed applying Simpson's rule using 302         | 
!           quadrature points.                                                 |
!                                                                              |
!------------------------------------------------------------------------------

!-----------Compute the frequency spectrum of the surface wave------------------

            DO I=1,coef%ws_nomega
              omega_1=(i-1)/10._jprb

              if(omega_1<=sunglint(j)%omega_m) sigma=sigma_a
              if(omega_1> sunglint(j)%omega_m) sigma=sigma_b

              sunglint(j)%beta(i)=exp(-(omega_1-sunglint(j)%omega_m)**2_jpim/ &
                                     (2._jprb*sigma**2_jpim*sunglint(j)%omega_m**2_jpim))

              if(i==1)then
                sunglint(j)%psi(i)=0._jprb
              else
                sunglint(j)%psi(i)=(sunglint(j)%alfa1*gravity**2_jpim/omega_1**5_jpim)*                        &
                       (exp(-(5._jprb/4._jprb)*(sunglint(j)%omega_m/omega_1)**4_jpim))*(k**sunglint(j)%beta(i))
              endif

              FF (i)=sunglint(j)%psi(i)*coef%ws_k_omega(i)**2_jpim
            ENDDO


!-----------Compute the Simpson's integral--------------------------------------

            bb    = 301._jprb
            aa    = 0._jprb
            np    = 1501
            hinc  = (bb - aa)/float(2*np)
            sa    = ff(1)
            sb    = ff(coef%ws_nomega)

            s4=0._jprb
            do i=1,nP
              jj=2*i - 1
              s4=s4+ff(jj)
            end do

            s2=0._jprb
            do i=1,nP
              jj=2*i
              s2=s2+ff(jj)
            end do
          
            sunglint(j)%gamma_sq=hinc/3.0_jprb * (sa + sb + 4.0_jprb*s4 + 2.0_jprb*s2)

!-------------------------------------------------------------------------------


!-----------Compute the rms of the slope of the facet along the X (gamma_O)---- 
!           and Y (gamma_P) axis.                                              |
!------------------------------------------------------------------------------

            sunglint(j)%wangl=sunglint(j)%wangl-profiles(j)%azangle*deg2rad


            if(sunglint(j)%wangl<0._jprb)then
              sunglint(j)%wangl=sunglint(j)%wangl+2._jprb*pi
            endif
  
            sunglint(j)%gamma_o=sqrt((2+cos(2._jprb*sunglint(j)%wangl))*sunglint(j)%gamma_sq/4._jprb)
            sunglint(j)%gamma_p=sqrt((2-cos(2._jprb*sunglint(j)%wangl))*sunglint(j)%gamma_sq/4._jprb)



!-----------Obtain angles csi and alfa------------------------------------------


            sunglint(j)%csi  = atan( tan(sunglint(j)%zensun)* cos(PI-sunglint(j)%dazng*deg2rad))
            sunglint(j)%alfa = asin( sin(sunglint(j)%zensun)* sin(PI-sunglint(j)%dazng*deg2rad))
            sunglint(j)%omega = abs(sunglint(j)%csi+sunglint(j)%zensat)/2._jprb


!-----------Compute the value of the function that represents the shadowing  --
!           of the incident and reflected ray                                  |
!------------------------------------------------------------------------------

!-----------First order shadowing (the slope of the facet is negative)---------

            sunglint(j)%gammax = tan((sunglint(j)%csi-sunglint(j)%zensat)/2._jprb)

            if((1/tan(abs(sunglint(j)%zensat))-sunglint(j)%gammax*sunglint(j)%zensat/                     &
                                               abs(sunglint(j)%zensat))>=0._jprb)then
              theta_fi=1._jprb
            else
              theta_fi=0._jprb
            endif


            if((1._jprb/tan(abs(sunglint(j)%csi))+sunglint(j)%gammax*sunglint(j)%csi/abs(sunglint(j)%csi))>=0._jprb)then
              theta_csi=1._jprb
            else
              theta_csi=0._jprb
            endif

            if(abs(sunglint(j)%csi)>PI/2._jprb)theta_csi=0._jprb


!-----------Second order shadowing (the facet cannot be seen)-------------------


            IF((sunglint(j)%zensat*sunglint(j)%csi)<=0._jprb)THEN

                IF(sunglint(j)%zensat==0._jprb.and.sunglint(j)%csi==0._jprb)THEN
                  sunglint(j)%Q_shad=1._jprb
                else if(sunglint(j)%zensat>=abs(sunglint(j)%csi))then

                  sunglint(j)%a_shad  =(1._jprb/tan(sunglint(j)%zensat))   /sunglint(j)%gamma_O
                  sunglint(j)%lambda_a=(1._jprb/(fac*sunglint(j)%a_shad))*exp(-sunglint(j)%a_shad**2_jpim/2._jprb)-       &
                              0.5_jprb*RTTOV_ERFCX(sunglint(j)%a_shad/sqrt(2._jprb))
                  Q_shad_A(j)=1._jprb/(1._jprb+sunglint(j)%lambda_a)
                  sunglint(j)%Q_shad  =Q_shad_A(j)*theta_fi

                ELSE IF(abs(sunglint(j)%csi)>sunglint(j)%ZENSAT)THEN

                  sunglint(j)%b_shad  =(1._jprb/tan(abs(sunglint(j)%csi)))/sunglint(j)%gamma_O
                  sunglint(j)%lambda_b=(1._jprb/(fac*sunglint(j)%b_shad))*exp(-sunglint(j)%b_shad**2_jpim/2._jprb)-       &
                              0.5_jprb*RTTOV_ERFCX(sunglint(j)%b_shad/sqrt(2._jprb))
                  Q_shad_B(j)=1._jprb/(1._jprb+sunglint(j)%lambda_b)
                  sunglint(j)%Q_shad  =Q_shad_B(j)*theta_csi

                ENDIF

            ELSE IF((sunglint(j)%ZENSAT*sunglint(j)%csi)>0._jprb)THEN

              sunglint(j)%a_shad=(1._jprb/tan(sunglint(j)%ZENSAT))   /sunglint(j)%gamma_O

              sunglint(j)%b_shad=(1._jprb/tan(abs(sunglint(j)%csi)))/sunglint(j)%gamma_O

              sunglint(j)%lambda_a=(1._jprb/(fac*sunglint(j)%a_shad))*exp(-sunglint(j)%a_shad**2_jpim/2._jprb)-           &
                          0.5_jprb*RTTOV_ERFCX(sunglint(j)%a_shad/sqrt(2._jprb))
              sunglint(j)%lambda_b=(1._jprb/(fac*sunglint(j)%b_shad))*exp(-sunglint(j)%b_shad**2_jpim/2._jprb)-           &
                          0.5_jprb*RTTOV_ERFCX(sunglint(j)%b_shad/sqrt(2._jprb))

              sunglint(j)%Q_shad=1._jprb/(1._jprb+sunglint(j)%lambda_a+sunglint(j)%lambda_b)

              sunglint(j)%Q_shad=sunglint(j)%Q_shad*theta_fi*theta_csi

            ENDIF

!-------------------------------------------------------------------------------


!-----------Compute the probability density that the slope of the facet at ---- 
!           a certain point is gammax when the incident ray and the ray from   | 
!           the reflection of the incident ray on the local surface do not     |
!           intersect with any other surface.                                  |
!------------------------------------------------------------------------------

            sunglint(j)%c_shad   = abs(cos(sunglint(j)%csi)+cos(sunglint(j)%ZENSAT))*sunglint(j)%gamma_P

            sunglint(j)%p_prime  = 1._jprb/(fac*sunglint(j)%c_shad)*                                   &
                                    exp(-0.5_jprb*(tan(sunglint(j)%alfa)/sunglint(j)%c_shad)**2_jpim)

            sunglint(j)%g_shad   = (1._jprb-tan((sunglint(j)%csi-sunglint(j)%ZENSAT)/2._jprb)*tan(sunglint(j)%ZENSAT))

            sunglint(j)%fac1     = 2._jprb*(cos(sunglint(j)%alfa))**2_jpim*(cos((sunglint(j)%csi-sunglint(j)%ZENSAT)/2))**2_jpim

            sunglint(j)%Pxy_gammaxy = (1._jprb/(fac*sunglint(j)%gamma_O))*                             &
                                    exp(-sunglint(j)%gammax**2_jpim/(2._jprb*sunglint(j)%gamma_O**2_jpim))

            rqs         = sunglint(j)%Q_shad*sunglint(j)%g_shad*sunglint(j)%p_prime*sunglint(j)%Pxy_gammaxy/ &
                                      sunglint(j)%fac1
              

!-------------------------------------------------------------------------------


            sunglint(j)%glint=rqs  ! *pi*epsilon**2_jpim
            endif

          endif
        endif
      enddo


IF (LHOOK) CALL DR_HOOK('RTTOV_REFSUN',1_jpim,ZHOOK_HANDLE)
      END SUBROUTINE RTTOV_REFSUN
