!
Subroutine rttov_setpredictors_7(&
      & nprofiles,  &!
      & prof,       &! in
      & geom,       &! in
      & coef,       &! in
      & aux,        &! in
      & predictors, &! out
      & raytracing ) ! out 
  ! 
  ! Description
  ! To calculate and store the profile variables (predictors) required
  ! in subsequent transmittance calculations.
  ! Code based on PRFTAU from previous versions of RTTOV
  ! Only one profile per call
  !
  ! Copyright:
  !    This software was developed within the context of
  !    the EUMETSAT Satellite Application Facility on
  !    Numerical Weather Prediction (NWP SAF), under the
  !    Cooperation Agreement dated 25 November 1998, between
  !    EUMETSAT and the Met Office, UK, by one or more partners
  !    within the NWP SAF. The partners in the NWP SAF are
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  !
  !    Copyright 2002, EUMETSAT, All Rights Reserved.
  !
  ! Method:
  ! see RTTOV7 science and validation report pages 18/19
  ! variable names are close to the documentation
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date     Comment
  ! -------   ----     -------
  !  1.0       01/12/2002  New F90 code with structures (P Brunel A Smith)
  !  1.1       04/12/2003  Optimisation (J Hague and D Salmond ECMWF)
  !  1.2       29/03/2005  Add end of header comment (J. Cameron)
  !  1.3       07/12/2005  Add surface humidity (R. Saunders)
  !  1.4       16/01/2006  Marco Matricardi (ECMWF):
  !               --       Altitude dependent local zenith angle introduced.
  !  1.5       22/08/2007  Optimised (D Salmond)
  !  1.6       12/08/2008  Zeeman effect based on Yong Han (P. Rayer)
  !
  ! Code Description:
  !   Language:           Fortran 90.
  !   Software Standards: "European Standards for Writing and
  !     Documenting Exchangeable Fortran 90 Code".
  !
  ! Declarations:
  ! Modules used:
  ! Imported Parameters:
  Use rttov_const, Only :   &
        & gravity        ,&
        & sensor_id_mw   , &
        & use_q2m 
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

  ! Imported Type Definitions:
  Use rttov_types, Only : &
        & rttov_coef     ,&
        & profile_Type   ,&
        & geometry_Type  ,&
        & profile_aux    ,&
        & predictors_Type,&
        & raytracing_type  

  Use parkind1, Only : jpim     ,jprb
USE PARKIND1  ,ONLY : JPRB
  Implicit None

  !subroutine arguments:
  Integer(Kind=jpim),    Intent(in)    :: nprofiles
  Type(profile_Type),    Intent(in)    :: prof(nprofiles)       ! profile
  Type(rttov_coef),      Intent(in)    :: coef       ! coefficients
  Type(geometry_Type),   Intent(in)    :: geom(nprofiles)       ! geometry
  Type(predictors_Type), Intent(inout) :: predictors ! predictors
  Type(profile_aux) ,    Intent(in)    :: aux(nprofiles)        ! auxillary profiles info.
  Type(raytracing_type), Intent(inout) :: raytracing

!INTF

  !local variables:
  Integer(Kind=jpim) :: level
  Integer(Kind=jpim) :: iv2, iv3, i

  ! user profile
  Real(Kind=jprb) :: t(nprofiles,prof(1) % nlevels)
  Real(Kind=jprb) :: w(nprofiles,prof(1) % nlevels)
  Real(Kind=jprb) :: o(nprofiles,prof(1) % nlevels)

  ! reference profile
  Real(Kind=jprb) :: tr(nprofiles,prof(1) % nlevels)
  Real(Kind=jprb) :: wr(nprofiles,prof(1) % nlevels)
  Real(Kind=jprb) :: or(nprofiles,prof(1) % nlevels)

  ! user - reference
  Real(Kind=jprb) :: dt(nprofiles,prof(1) % nlevels)
  Real(Kind=jprb) :: dto(nprofiles,prof(1) % nlevels)

  ! pressure weighted
  Real(Kind=jprb) :: tw(nprofiles,prof(1) % nlevels)
  Real(Kind=jprb) :: ww(nprofiles,prof(1) % nlevels)
  Real(Kind=jprb) :: ow(nprofiles,prof(1) % nlevels)

  ! intermediate variables
  Real(Kind=jprb) :: sum1(nprofiles),sum2(nprofiles),ztemp
  Real(Kind=jprb) :: deltac(prof(1) %nlevels)
  Real(Kind=jprb) :: sec_or(prof(1) %nlevels)
  Real(Kind=jprb) :: sec_wr(prof(1) %nlevels)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

  !- End of header --------------------------------------------------------

  ! 1 profile layer quantities
IF (LHOOK) CALL DR_HOOK('RTTOV_SETPREDICTORS_7',0_jpim,ZHOOK_HANDLE)
  Do i=1,nprofiles
    t(i,1) = prof(i) % t(1)
    w(i,1) = prof(i) % q(1)
  Enddo
    Do i=1,nprofiles
  Do level=2,prof(1)%nlevels
      t(i,level) = ( prof(i) % t(level-1) +  prof(i) % t(level ) ) *0.5_JPRB
      w(i,level) = ( prof(i) % q(level-1) +  prof(i) % q(level ) ) *0.5_JPRB
    Enddo
  Enddo

  If ( use_q2m )Then
    Do i=1,nprofiles
  ! include surface humidity
      iv3 = aux(i) % nearestlev_surf - 1
      iv2 = aux(i) % nearestlev_surf
      If ( iv2 <= coef % nlevels) Then
        w(i,iv2) = (prof(i) % s2m % q  + prof(i) % q(iv3)) *0.5_JPRB
      Endif
    Enddo
  Endif

  Do i=1,nprofiles
    If ( prof(i) % ozone_Data .And. coef % nozone > 0 ) Then
      o(i,1) = prof(i) % o3(1)
    Endif
  EndDo
    Do i=1,nprofiles
  Do level=2,prof(1)%nlevels
      If ( prof(i) % ozone_Data .And. coef % nozone > 0 ) Then
        o(i,level ) = ( prof(i) % o3(level-1) + prof(i) % o3(level  ) ) *0.5_JPRB
      Endif
    EndDo
  EndDo

  ! 2 calculate deviations from reference profile (layers)
  ! if no input O3 profile, set to reference value (dto =0)
    Do i=1,nprofiles
  Do level=1,prof(1)%nlevels
      dt(i,level)  = t(i,level) - coef % tstar(level)
      If ( prof(i) % ozone_Data .And. coef % nozone > 0 ) Then
       dto(i,level) = t(i,level) - coef % to3star(level)
      Else
       dto(i,level) = 0._JPRB
      Endif

  ! 3 calculate (profile / reference profile) ratios; tr wr or
      tr(i,level) = t(i,level) / coef % tstar(level)
      wr(i,level) = w(i,level) / coef % wstar(level)
  ! if no input O3 profile, set to reference value (or =1)
      If ( prof(i) % ozone_Data .And. coef % nozone > 0 ) Then
        or(i,level) = o(i,level) / coef % ostar(level)
      Else
        or(i,level) = 1._JPRB
      Endif
    EndDo
  EndDo

  ! 4 calculate profile / reference profile sums: tw ww ow
  Do i=1,nprofiles
    tw(i,1) = 0._JPRB
  Enddo
    Do i=1,nprofiles
  Do level = 2 , prof(1) % nlevels
      tw(i, level ) = tw(i, level-1 ) + coef % dpp( level ) * tr (i, level -1 )
    End Do
  End Do

  sum1 = 0._JPRB
  sum2 = 0._JPRB
    Do i=1,nprofiles
  Do level = 1, prof(1) % nlevels
      sum1(i) = sum1(i) + coef % dpp( level ) *      w       (i, level )
      sum2(i) = sum2(i) + coef % dpp( level ) * coef % wstar ( level )
      ww (i, level ) = sum1(i) / sum2(i)
    End Do
  End Do

  ! if no input O3 profile, set to reference value (ow =1)
  sum1 = 0._JPRB
  sum2 = 0._JPRB
    Do i=1,nprofiles
  Do level = 1, prof(1) % nlevels
      If ( prof(i) % ozone_Data .And. coef % nozone > 0 ) Then
        sum1(i) = sum1(i) + coef % dpp( level ) *      o       (i, level )
        sum2(i) = sum2(i) + coef % dpp( level ) * coef % ostar ( level )
        ow (i, level ) = sum1(i) / sum2(i)
      Else
        ow(i,level) = 1._JPRB
      Endif
    End Do
  End Do

  ! for other minor gases do as for O3 for testing presence of
  ! coefficients and profile values
  !

  !5) set predictors
  !--

  !5.1 mixed gases
  !---

  If(coef % id_inst /= 36)Then  

    Do i=1,nprofiles
      Do level = 1, prof(1) % nlevels
  ! No Zeeman effect
        predictors % mixedgas(1,level,i)  = raytracing % pathsat(level,i)
        predictors % mixedgas(2,level,i)  = raytracing % pathsat(level,i) * &
                                          & raytracing % pathsat(level,i) 
        predictors % mixedgas(3,level,i)  = raytracing % pathsat(level,i) * tr(i,level)
        predictors % mixedgas(4,level,i)  = raytracing % pathsat(level,i) * tr(i,level)* &
                                          & tr(i,level) 
        predictors % mixedgas(5,level,i)  = tr(i,level)
        predictors % mixedgas(6,level,i)  = tr(i,level) * tr(i,level)
        predictors % mixedgas(7,level,i)  = raytracing % pathsat(level,i) * tw(i,level)
        predictors % mixedgas(8,level,i)  = raytracing % pathsat(level,i) * tw(i,level)/ &
                                          & tr(i,level) 
        predictors % mixedgas(9,level,i)  = Sqrt(raytracing % pathsat(level,i))
        predictors % mixedgas(10,level,i) = predictors % mixedgas(9,level,i)  * &
                                          & tw(i,level)**0.25_JPRB  

        If(coef % id_inst == 3 .and. coef % IncZeeman)Then  
  ! AMSUA ch14 with Zeeman effect (coef % nmixed = 14, otherwise = 10)
  ! predictors 1 - 4 are overlapped with other channels      
          predictors % mixedgas(11,level,i) = prof(i) % cosbk**2 * raytracing % pathsat(level,i) 
          predictors % mixedgas(12,level,i) = prof(i) % Be * raytracing % pathsat(level,i)**2
          predictors % mixedgas(13,level,i) = prof(i) % Be**3 * raytracing % pathsat(level,i)
          predictors % mixedgas(14,level,i) = (prof(i) % cosbk * prof(i) % Be * &
	                                    & raytracing % pathsat(level,i))**2
        Endif           
      End Do
    End Do

  ELse   ! ssmisz predictors (Zeeman-affected channels of SSMIS allocated to sensor SSMISZ)
! NB require prof(i) % Be >0. (divisor)
    Do i=1,nprofiles
      Do level = 1, prof(1) % nlevels
        Predictors % mixedgas(1, level,i) = raytracing % pathsat(level,i)
        Predictors % mixedgas(2, level,i) = (300.0_JPRB/t(i, level)) * &
                                          & raytracing % pathsat(level,i)
        Predictors % mixedgas(3, level,i) = prof(i) % cosbk**2 * &
                                          & raytracing % pathsat(level,i)    
        Predictors % mixedgas(4, level,i) = Predictors % mixedgas(2,level,i) / prof(i) % Be
        Predictors % mixedgas(5, level,i) = Predictors % mixedgas(2,level,i) * &
                                          & prof(i) % cosbk**2 
        Predictors % mixedgas(6, level,i) = raytracing % pathsat(level,i) / prof(i) % Be
        Predictors % mixedgas(7, level,i) = Predictors % mixedgas(6,level,i) / prof(i) % Be
        Predictors % mixedgas(8, level,i) = prof(i) % Be * raytracing % pathsat(level,i)
        Predictors % mixedgas(9, level,i) = prof(i) % Be**3 * raytracing % pathsat(level,i)
        Predictors % mixedgas(10,level,i) = Predictors % mixedgas(3,level,i) * prof(i) % Be
        Predictors % mixedgas(11,level,i) = Predictors % mixedgas(10,level,i) * prof(i) % Be
      Enddo 
    Enddo

  Endif

  !5.2 water vapour  ( numbers in right hand are predictor numbers
  ! in the reference document for RTTOV7 (science and validation report)
  !----------------

    Do i=1,nprofiles
  Do level = 1, prof(1) % nlevels
      ztemp=1.0_JPRB/ww(i,level)
      sec_wr(level) = raytracing%pathsat(level,i)*wr(i,level)
      predictors% watervapour(1,level,i)  = sec_wr(level)                                        !  7
      predictors% watervapour(2,level,i)  = Sqrt( sec_wr(level) )                                !  5
      predictors% watervapour(3,level,i)  = sec_wr(level) * wr(i,level) * ztemp                  ! 12
      predictors% watervapour(4,level,i)  = sec_wr(level) * dt(i,level)                          !  4
      predictors% watervapour(5,level,i)  = sec_wr(level) * sec_wr(level)                        !  1
      predictors% watervapour(6,level,i)  = predictors% watervapour(2,level,i)   * &
                                           & dt (i,level)                                          ! 11 
      predictors% watervapour(7,level,i)  = Sqrt(predictors% watervapour(2,level,i))           ! 6
      predictors% watervapour(8,level,i)  = predictors% watervapour(2,level,i)   * &
                                           & wr(i,level) * ztemp                                   ! 13 
      predictors% watervapour(9,level,i)  = predictors% watervapour(5,level,i)   * &
                                           & sec_wr(level)                                         ! 8 
      predictors% watervapour(10,level,i) = predictors% watervapour(9,level,i)   * &
                                           & sec_wr(level)                                         ! 9 
      predictors% watervapour(11,level,i) = sec_wr(level)*dt(i,level)*Abs(dt(i,level))           ! 10
      predictors% watervapour(12,level,i) = ( raytracing%pathsat(level,i)         * &
                                           & ww(i,level) )**4                                      ! 3 
      predictors% watervapour(13,level,i) = ( raytracing%pathsat(level,i)         * &
                                           & ww(i,level) )**2         ! 2 
      ztemp=1.0_JPRB/tr(i,level)
      predictors% watervapour(14,level,i) = sec_wr(level)*wr(i,level) * ztemp                    ! 14
      predictors% watervapour(15,level,i) = sec_wr(level)*wr(i,level) * ztemp     **4
    End Do
  End Do
  !
  !5.3 ozone
  !---------

  ! if no input O3 profile, variables or, ow and dto have been set
  ! to the reference profile values (1, 1, 0)

  If ( coef % nozone > 0 ) Then
       Do i=1,nprofiles
     Do level = 1, prof(1) % nlevels
         sec_or(level) = raytracing%pathsat(level,i)*or(i,level)
         predictors% ozone(1,level,i)  = sec_or(level)
         predictors% ozone(2,level,i)  = Sqrt( sec_or(level) )
         predictors% ozone(3,level,i)  = sec_or(level) * dto(i,level)
         predictors% ozone(4,level,i)  = sec_or(level) * sec_or(level)
         predictors% ozone(5,level,i)  = predictors% ozone(2,level,i) * dto(i,level)
         predictors% ozone(6,level,i)  = sec_or(level) * or(i,level) * ow (i,level)
         predictors% ozone(7,level,i)  = predictors% ozone(2,level,i) * or(i,level)/ &
                                        & ow(i,level) 
         predictors% ozone(8,level,i)  = sec_or(level) * ow(i,level)
         predictors% ozone(9,level,i)  = sec_or(level)                          * &
                                        & Sqrt(raytracing%pathsat(level,i)*ow(i,level) ) 
         predictors% ozone(10,level,i) = raytracing%pathsat(level,i)*ow(i,level)
         predictors% ozone(11,level,i) = (raytracing%pathsat(level,i)*ow(i,level))**2
       End Do
     End Do
  Endif
  !
  !5.4 cloud
  !---------
     ztemp=1.0_JPRB/(4.3429_JPRB * gravity)
       Do i=1,nprofiles
     Do level = 1, prof(1) % nlevels
       deltac(level) = 0.1820_JPRB * 100.0_JPRB * coef % dp(level) *ztemp
         If ( prof (i)% clw_Data .And. coef % id_sensor == sensor_id_mw ) Then
           predictors% clw(level,i)         = deltac(level) * prof(i)%clw(level) * geom(i)%seczen
         Endif
       Enddo
     EndDo
       Do i=1,nprofiles
     Do level = 2, prof(1) % nlevels
          If ( prof (i)% clw_Data .And. coef % id_sensor == sensor_id_mw ) Then
             predictors% clw(level,i) = 0.5_JPRB *   &
             &  ( predictors% clw(level,i) +  deltac(level) * prof(i)%clw(level-1) *  geom(i)%seczen ) 
          Endif
       Enddo
     Enddo
     Do i=1,nprofiles
       If ( prof (i)% clw_Data .And. coef % id_sensor == sensor_id_mw ) Then
         predictors% ncloud   = 1
       Endif
     Enddo
IF (LHOOK) CALL DR_HOOK('RTTOV_SETPREDICTORS_7',1_jpim,ZHOOK_HANDLE)

End Subroutine rttov_setpredictors_7
