!
Subroutine rttov_setpredictors_7_ad( &
      & nprofiles,  &!
      & prof,        &! in
      & prof_ad,     &! inout
      & geom,        &! in
      & coef,        &! in
      & aux,         &! in
      & predictors,  &! in
      & predictors_ad,&
      & raytracing,   &
      & raytracing_ad ) ! inout  
  ! 
  ! Description
  ! RTTOV-7 Model
  ! AD of rttov_setpredictors
  ! To calculate and store the profile variables (predictors) required
  ! in subsequent transmittance calculations.
  ! Code based on PRFTAU from previous versions of RTTOV
  ! Only one profile per call
  !
  ! Copyright:
  !    This software was developed within the context of
  !    the EUMETSAT Satellite Application Facility on
  !    Numerical Weather Prediction (NWP SAF), under the
  !    Cooperation Agreement dated 25 November 1998, between
  !    EUMETSAT and the Met Office, UK, by one or more partners
  !    within the NWP SAF. The partners in the NWP SAF are
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  !
  !    Copyright 2002, EUMETSAT, All Rights Reserved.
  !
  ! Method:
  ! see RTTOV7 science and validation report pages 18/19
  ! variable names are close to the documentation
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date     Comment
  ! -------   ----     -------
  !  1.0       01/12/2002  New F90 code with structures (P Brunel A Smith)
  !  1.1       04/12/2003  Optimisation (J Hague and D Salmond ECMWF)
  !  1.2       29/03/2005  Add end of header comment (J. Cameron)
  !  1.3       07/12/2005  Add surface humidity (R. Saunders)
  !  1.4       03/03/2006  Marco Matricardi (ECMWF):
  !               --       Altitude dependent local zenith angle introduced.
  !  1.5       12/08/2008  Zeeman effect based on Yong Han (P. Rayer)
  !
  ! Code Description:
  !   Language:           Fortran 90.
  !   Software Standards: "European Standards for Writing and
  !     Documenting Exchangeable Fortran 90 Code".
  !
  ! Declarations:
  ! Modules used:
  Use rttov_const, Only :   &
        & gravity    ,&
        & use_q2m 
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

  ! Imported Type Definitions:
  Use rttov_types, Only : &
        & rttov_coef     ,&
        & profile_Type   ,&
        & geometry_Type  ,&
        & profile_aux    ,&
        & predictors_Type,&
        & raytracing_type  

  Use parkind1, Only : jpim     ,jprb
USE PARKIND1  ,ONLY : JPRB
  Implicit None

  !subroutine arguments:
  Integer(Kind=jpim),    Intent(in)     :: nprofiles
  Type(profile_Type),    Intent(in)     :: prof(nprofiles)
  Type(profile_Type),    Intent(inout)  :: prof_ad(nprofiles)
  Type(rttov_coef),      Intent(in)     :: coef
  Type(geometry_Type),   Intent(in)     :: geom(nprofiles)
  Type(predictors_Type), Intent(in)     :: predictors
  Type(predictors_Type), Intent(inout)  :: predictors_ad
  Type(raytracing_type), Intent(in)     :: raytracing
  Type(raytracing_type), Intent(inout)  :: raytracing_ad
  Type(profile_aux) ,    Intent(in)     :: aux(nprofiles)        ! auxillary profiles info.

!INTF

  !local variables:
  Integer(Kind=jpim) :: level
  Integer(Kind=jpim) :: iv2, iv3(nprofiles), i
  Real(Kind=jprb) :: ztemp

  ! user profile
  Real(Kind=jprb) :: t(nprofiles,prof(1) % nlevels)
  Real(Kind=jprb) :: w(nprofiles,prof(1) % nlevels)
  Real(Kind=jprb) :: o(nprofiles,prof(1) % nlevels)

  ! reference profile
  Real(Kind=jprb) :: tr(nprofiles,prof(1) % nlevels)
  Real(Kind=jprb) :: wr(nprofiles,prof(1) % nlevels)
  Real(Kind=Jprb) :: or(nprofiles,prof(1) % nlevels)

  ! user - reference
  Real(Kind=jprb) :: dt(nprofiles,prof(1) % nlevels)

  ! pressure weighted
  Real(Kind=jprb) :: tw(nprofiles,prof(1) % nlevels)
  Real(Kind=jprb) :: ww(nprofiles,prof(1) % nlevels)
  Real(Kind=jprb) :: ow(nprofiles,prof(1) % nlevels)

  Real(Kind=jprb) :: sum1(nprofiles),sum2(nprofiles)
  Real(Kind=jprb) :: deltac(prof(1)%nlevels)
  Real(Kind=jprb) :: sec_wr(prof(1)%nlevels)
  Real(Kind=jprb) :: sum2_ww(nprofiles,prof(1)%nlevels)
  Real(Kind=jprb) :: sum2_ow(nprofiles,prof(1)%nlevels)

  ! TL variables
  Real(Kind=jprb) :: t_ad(nprofiles,prof(1) % nlevels)
  Real(Kind=jprb) :: w_ad(nprofiles,prof(1) % nlevels)
  Real(Kind=jprb) :: o_ad(nprofiles,prof(1) % nlevels)

  Real(Kind=jprb) :: tr_ad(nprofiles,prof(1) % nlevels)
  Real(Kind=jprb) :: wr_ad(nprofiles,prof(1) % nlevels)
  Real(Kind=jprb) :: or_ad(nprofiles,prof(1) % nlevels)

  Real(Kind=jprb) :: dt_ad(nprofiles,prof(1) % nlevels)
  Real(Kind=jprb) :: dto_ad(nprofiles,prof(1) % nlevels)

  Real(Kind=jprb) :: tw_ad(nprofiles,prof(1) % nlevels)
  Real(Kind=jprb) :: ww_ad(nprofiles,prof(1) % nlevels)
  Real(Kind=jprb) :: ow_ad(nprofiles,prof(1) % nlevels)


  Real(Kind=jprb) :: sec_or_ad(prof(1)%nlevels)
  Real(Kind=jprb) :: sec_wr_ad(prof(1)%nlevels)
  Real(Kind=jprb) :: zsqrt,zrecip
REAL(KIND=JPRB) :: ZHOOK_HANDLE

  !- End of header --------------------------------------------------------

  ! profile layer quantities
  ! Direct variables
IF (LHOOK) CALL DR_HOOK('RTTOV_SETPREDICTORS_7_AD',0_jpim,ZHOOK_HANDLE)
  Do i=1,nprofiles
    t(i,1) = prof(i) % t(1)
    w(i,1) = prof(i) % q(1)
  Enddo
!CDIR NOLOOPCHG
  Do level=2,prof(1)%nlevels
    Do i=1,nprofiles
      t(i,level) = ( prof(i) % t(level-1) +  prof(i) % t(level ) ) *0.5_JPRB
      w(i,level) = ( prof(i) % q(level-1) +  prof(i) % q(level ) ) *0.5_JPRB
    Enddo
  Enddo

  If ( use_q2m )Then
    Do i=1,nprofiles
  ! include surface humidity
      iv3(i) = aux(i) % nearestlev_surf - 1
      iv2 = aux(i) % nearestlev_surf
      If ( iv2 <= coef % nlevels) Then
        w(i,iv2) = (prof(i) % s2m % q  + prof(i) % q(iv3(i))) *0.5_JPRB
      Endif
    Enddo
  Endif

  Do i=1,nprofiles
    If ( prof(i) % ozone_Data .And. coef % nozone > 0 ) Then
      o(i,1) = prof(i) % o3(1)
    Endif
  EndDo
!CDIR NOLOOPCHG
  Do level=2,prof(1)%nlevels
    Do i=1,nprofiles
      If ( prof(i) % ozone_Data .And. coef % nozone > 0 ) Then
        o(i,level ) = ( prof(i) % o3(level-1) + prof(i) % o3(level  ) ) *0.5_JPRB
      Endif
    EndDo
  EndDo

  ! 2 calculate deviations from reference profile (layers)
  ! if no input O3 profile, set to reference value (dto =0)
!CDIR NOLOOPCHG
  Do level=1,prof(1)%nlevels
    Do i=1,nprofiles
      dt(i,level)  = t(i,level) - coef % tstar(level)

  ! 3 calculate (profile / reference profile) ratios; tr wr or
      tr(i,level) = t(i,level) / coef % tstar(level)
      wr(i,level) = w(i,level) / coef % wstar(level)
  ! if no input O3 profile, set to reference value (or =1)
      If ( prof(i) % ozone_Data .And. coef % nozone > 0 ) Then
        or(i,level) = o(i,level) / coef % ostar(level)
      Else
        or(i,level) = 1._JPRB
      Endif
    EndDo
  EndDo

  ! 4 calculate profile / reference profile sums: tw ww ow
  Do i=1,nprofiles
    tw(i,1) = 0._JPRB
  Enddo
  Do level = 2 , prof(1) % nlevels
    Do i=1,nprofiles
      tw(i, level ) = tw(i, level-1 ) + coef % dpp( level ) * tr (i, level -1 )
    End Do
  End Do

  sum1 = 0._JPRB
  sum2 = 0._JPRB
  Do level = 1, prof(1) % nlevels
    Do i=1,nprofiles
      sum1(i) = sum1(i) + coef % dpp( level ) *      w       (i, level )
      sum2(i) = sum2(i) + coef % dpp( level ) * coef % wstar ( level )
      sum2_ww(i,level ) = sum2(i)
      ww (i, level ) = sum1(i) / sum2(i)
    End Do
  End Do
  ! if no input O3 profile, set to reference value (ow =1)
  sum1 = 0._JPRB
  sum2 = 0._JPRB
  Do level = 1, prof(1) % nlevels
    Do i=1,nprofiles
      If ( prof(i) % ozone_Data .And. coef % nozone > 0 ) Then
        sum1(i) = sum1(i) + coef % dpp( level ) *      o       (i, level )
        sum2(i) = sum2(i) + coef % dpp( level ) * coef % ostar ( level )
        sum2_ow( i,level ) = sum2(i)
        ow (i, level ) = sum1(i) / sum2(i)
     Else
       sum2_ow(i,level) =  0._JPRB
       ow(i,level) = 1._JPRB
      End If
    End Do
  End Do


  ! Ajoint code
  !-------------
  w_ad(:,:)      = 0._JPRB
  wr_ad(:,:)     = 0._JPRB
  ww_ad(:,:)     = 0._JPRB
  dt_ad(:,:)     = 0._JPRB

  t_ad(:,:) = 0._JPRB
  tr_ad(:,:) = 0._JPRB
  tw_ad(:,:) = 0._JPRB

  If ( coef % nozone > 0 ) Then
    o_ad(:,:)      = 0._JPRB
    or_ad(:,:)     = 0._JPRB
    ow_ad(:,:)     = 0._JPRB
    dto_ad(:,:)    = 0._JPRB
  End If

  !5.4 cloud
  !---------

     ztemp=1.0_JPRB/(4.3429_JPRB * gravity)
     Do level = 1, prof(1) % nlevels
       deltac(level) = 0.1820_JPRB * 100.0_JPRB * coef % dp(level) *ztemp
     EndDo

!CDIR NOLOOPCHG
     Do level = 2, prof(1) % nlevels
       Do i=1,nprofiles
          If ( prof (i)% clw_Data ) Then
            prof_ad(i)%clw(level-1) = prof_ad(i)%clw(level-1) + 0.5_JPRB * predictors_ad % clw(level,i) *&
            & deltac(level) * geom(i)%seczen 
            predictors_ad % clw(level,i) = 0.5_JPRB *  predictors_ad % clw(level,i)
          Endif
       Enddo
     Enddo

!CDIR NOLOOPCHG
     Do level = 1, prof(1) % nlevels
       Do i=1,nprofiles
         If ( prof (i)% clw_Data ) Then
            prof_ad(i)%clw(level) = prof_ad(i)%clw(level)  + predictors_ad % clw(level,i) *&
            & deltac(level) * geom(i)%seczen
         Endif
       Enddo
     EndDo


  !5.3 ozone
  !---------
  If ( coef % nozone > 0 ) Then
    Do level = 1,prof_ad(1) % nlevels
      Do i=1,nprofiles
        sec_or_ad(level) = 0._JPRB
  
        ! One can pack all ow_ad lines in one longer statement
        ! same for sec_or_ad and dto_ad
  
        ow_ad(i,level) = ow_ad(i,level) +  predictors_ad % ozone(11,level,i)*2       * &
                       & raytracing%pathsat(level,i)*predictors% ozone(10,level,i) 
        raytracing_ad%pathsat(level,i)  = raytracing_ad%pathsat(level,i)           + &
                       & predictors_ad % ozone(11,level,i)*2 *ow(i,level)            * &
                       & predictors% ozone(10,level,i) 
  
        ow_ad(i,level) = ow_ad(i,level) + predictors_ad % ozone(10,level,i)          * &
                       & raytracing%pathsat(level,i) 
        raytracing_ad%pathsat(level,i) = raytracing_ad%pathsat(level,i)            + &
                       & predictors_ad % ozone(10,level,i)*ow(i,level) 
  
        or_ad(i,level)= or_ad(i,level) + predictors_ad % ozone(9,level,i)            * &
                      & SQRT(raytracing%pathsat(level,i)                           * &
                      & ow(i,level))*raytracing%pathsat(level,i) 
        ow_ad(i,level) = ow_ad(i,level) + predictors_ad % ozone(9,level,i)           * &
                       & predictors% ozone(1,level,i)*0.5                         * &
                       & raytracing%pathsat(level,i)/SQRT(raytracing%pathsat(level,i)* &
                       & ow(i,level)) 
        raytracing_ad%pathsat(level,i)=raytracing_ad%pathsat(level,i)              + &
                       & predictors_ad % ozone(9,level,i)*1.5*or(i,level)            * &
                       & SQRT(predictors% ozone(10,level,i)) 
  
        or_ad(i,level)=or_ad(i,level) + predictors_ad % ozone(8,level,i)             * &
                       & predictors% ozone(10,level,i) 
        ow_ad(i,level)=ow_ad(i,level)+ predictors_ad % ozone(8,level,i)              * &
                       & raytracing%pathsat(level,i)  *or(i,level) 
        raytracing_ad%pathsat(level,i)=raytracing_ad%pathsat(level,i)              + &
                       & predictors_ad % ozone(8,level,i)*or(i,level)*ow(i,level) 
  
        or_ad(i,level)=or_ad(i,level) + predictors_ad % ozone(7,level,i)*1.5         * &
                     & predictors% ozone(2,level,i)/ow(i,level) 
  
        ow_ad(i,level) = ow_ad(i,level)  - predictors_ad % ozone(7,level,i)          * &
                       & predictors% ozone(7,level,i)/ow(i,level) 
        raytracing_ad%pathsat(level,i)=raytracing_ad%pathsat(level,i)              + &
                       & predictors_ad % ozone(7,level,i)*0.5*or(i,level)**1.5       / &
                       & (raytracing%pathsat(level,i)**0.5*ow(i,level)) 
  
        or_ad(i,level)=or_ad(i,level) + predictors_ad % ozone(6,level,i)*2           * &
                       & predictors% ozone(1,level,i)*ow(i,level) 
        ow_ad(i,level) = ow_ad(i,level) + predictors_ad % ozone(6,level,i)           * &
                       & predictors% ozone(4,level,i)/raytracing%pathsat(level,i) 
        raytracing_ad%pathsat(level,i)=raytracing_ad%pathsat(level,i)              + &
                       & predictors_ad % ozone(6,level,i)*or(i,level)**2*ow(i,level) 
   
        sec_or_ad(level)=sec_or_ad(level)+predictors_ad % ozone(5,level,i)       * &
                       & 0.5_JPRB* predictors% ozone(3,level,i)                   / &
                       & ( predictors% ozone(1,level,i) *predictors% ozone(2,level,i)) 
        dto_ad(i,level)=dto_ad(i,level)+predictors_ad % ozone(5,level,i)             * &
                       & predictors% ozone(2,level,i) 
  
        sec_or_ad(level)=sec_or_ad(level)+ predictors_ad % ozone(4,level,i)*2    * &
                       & predictors% ozone(1,level,i) 
  
        sec_or_ad(level)=sec_or_ad(level)+predictors_ad % ozone(3,level,i)       * &
                       & predictors% ozone(3,level,i) / predictors% ozone(1,level,i) 
        dto_ad(i,level)=dto_ad(i,level)+predictors_ad % ozone(3,level,i)             * &
                       & predictors% ozone(1,level,i) 
  
        sec_or_ad(level)=sec_or_ad(level)+predictors_ad % ozone(2,level,i)       * &
                       & 0.5_JPRB/ predictors% ozone(2,level,i) 
  
        sec_or_ad(level)=sec_or_ad(level)+predictors_ad % ozone(1,level,i)
  
        or_ad(i,level)=or_ad(i,level)+sec_or_ad(level) *raytracing%pathsat(level,i)
        raytracing_ad%pathsat(level,i)=raytracing_ad%pathsat(level,i)              + &
                       & sec_or_ad(level)*or(i,level) 
      End Do
    End Do
  Endif

  !5.2 water vapour  ( numbers in right hand are predictor numbers
  ! in the reference document for RTTOV7 (science and validation report)
  !----------------

  Do level = 1,prof_ad(1) % nlevels
    Do i=1,nprofiles
      sec_wr_ad(level) = 0._JPRB
      sec_wr(level)    = raytracing%pathsat(level,i)*wr(i,level) 
  
      sec_wr_ad(level)=sec_wr_ad(level)+predictors_ad % watervapour(15,level,i)  * &
                       & wr(i,level) / tr(i,level)**4 
      wr_ad(i,level)=wr_ad(i,level)+predictors_ad % watervapour(15,level,i)          * &
                       & sec_wr(level)/ tr(i,level)**4 
      tr_ad(i,level)=tr_ad(i,level)-predictors_ad % watervapour(15,level,i)*4        * &
                       & sec_wr(level) * wr(i,level) / tr(i,level)**5 
     
      sec_wr_ad(level)=sec_wr_ad(level)+predictors_ad % watervapour(14,level,i)  * &
                       & wr(i,level) / tr(i,level) 
      wr_ad(i,level)=wr_ad(i,level)+predictors_ad % watervapour(14,level,i)          * &
                       & sec_wr(level) / tr(i,level) 
      tr_ad(i,level)=tr_ad(i,level)-predictors_ad % watervapour(14,level,i)          * &
                       & sec_wr(level) * wr(i,level) / tr(i,level)**2 
  
      zsqrt=Sqrt(predictors% watervapour(13,level,i))
  
      ww_ad(i,level)=ww_ad(i,level)+predictors_ad % watervapour(13,level,i)*2        * &
                   & raytracing%pathsat(level,i)*zsqrt 
      raytracing_ad%pathsat(level,i)=raytracing_ad%pathsat(level,i)                + &
                   & predictors_ad % watervapour(13,level,i)*2 *ww(i,level) * zsqrt 
  
      ww_ad(i,level)=ww_ad(i,level)+predictors_ad % watervapour(12,level,i)*4        * &
                   & raytracing%pathsat(level,i)*predictors% watervapour(12,level,i)/ &
                   & zsqrt 
      raytracing_ad%pathsat(level,i)=raytracing_ad%pathsat(level,i)                + &
                   & predictors_ad % watervapour(12,level,i)*4 *ww(i,level)          * &
                   & predictors%watervapour(12,level,i)/zsqrt 
  
      sec_wr_ad(level)=sec_wr_ad(level)+predictors_ad % watervapour(11,level,i)  * &
                   & Abs(dt(i,level))*dt(i,level) 
      dt_ad(i,level)=dt_ad(i,level)+predictors_ad % watervapour(11,level,i)          * &
                   & Abs(dt(i,level)) *2 * sec_wr(level) 
  
      sec_wr_ad(level)=sec_wr_ad(level)+predictors_ad % watervapour(10,level,i)  * &
                   & 4 *predictors% watervapour(9,level,i) 
  
      sec_wr_ad(level)=sec_wr_ad(level)+predictors_ad % watervapour(9,level,i)   * &
                   & 3 *predictors% watervapour(5,level,i) 
  
      predictors_ad % watervapour(2,level,i)=predictors_ad % watervapour(2,level,i)+ &
                      & predictors_ad % watervapour(8,level,i)*wr(i,level) / ww(i,level) 
      wr_ad(i,level)=wr_ad(i,level)+predictors_ad % watervapour(8,level,i)           * &
                      & predictors% watervapour(2,level,i) / ww(i,level) 
      ww_ad(i,level)=ww_ad(i,level)-predictors_ad % watervapour(8,level,i)           * &
                      & predictors% watervapour(2,level,i) * wr(i,level) /ww(i,level)**2 
  
      sec_wr_ad(level)=sec_wr_ad(level)+predictors_ad % watervapour(7,level,i)   * &
                      & 0.25_JPRB / predictors% watervapour(7,level,i)**3 
  
      zrecip=1.0_JPRB/ predictors% watervapour(1,level,i)
  
      dt_ad(i,level)=dt_ad(i,level)+predictors_ad % watervapour(6,level,i)           * &
                      & predictors% watervapour(2,level,i) 
      sec_wr_ad(level)=sec_wr_ad(level)+predictors_ad % watervapour(6,level,i)   * &
                      & 0.5_JPRB* predictors% watervapour(6,level,i) *zrecip 
  
      sec_wr_ad(level)=sec_wr_ad(level)+predictors_ad % watervapour(5,level,i)*2 * &
                      & predictors% watervapour(1,level,i) 
  
      sec_wr_ad(level)=sec_wr_ad(level)+predictors_ad % watervapour(4,level,i)   * &
                      & predictors% watervapour(4,level,i) *zrecip 
      dt_ad(i,level)=dt_ad(i,level)+predictors_ad % watervapour(4,level,i)           * &
                      & predictors% watervapour(1,level,i) 
  
      sec_wr_ad(level)=sec_wr_ad(level)+predictors_ad % watervapour(3,level,i)   * &
                      & wr(i,level) / ww(i,level) 
      wr_ad(i,level)=wr_ad(i,level)+ predictors_ad % watervapour(3,level,i)          * &
                      & sec_wr(level)/ ww(i,level) 
      ww_ad(i,level)=ww_ad(i,level)-predictors_ad % watervapour(3,level,i)*wr(i,level) * &
                      & sec_wr(level)/ ww(i,level)**2 
  
      sec_wr_ad(level)=sec_wr_ad(level)+predictors_ad % watervapour(2,level,i)   * &
                       & 0.5_JPRB / predictors% watervapour(2,level,i) 
  
      sec_wr_ad(level)=sec_wr_ad(level)+predictors_ad % watervapour(1,level,i)
  
      raytracing_ad%pathsat(level,i)=raytracing_ad%pathsat(level,i)                + &
                       & sec_wr_ad(level) *wr(i,level) 
      wr_ad(i,level)=wr_ad(i,level)+sec_wr_ad(level)*raytracing%pathsat(level,i)

    End Do
  End Do

  !5.1 mixed gases
  !---------------

  If(coef % id_inst /= 36)Then  

    Do level = 2,prof_ad(1) % nlevels
      Do i=1,nprofiles
        raytracing_ad % pathsat(level,i)=                                 &
                      & raytracing_ad % pathsat(level,i)               + &
                      & predictors_ad % mixedgas(10,level,i)*0.5_JPRB  * &
                      & tw(i,level)**0.25_JPRB                         / &
                      & predictors% mixedgas(9,level,i) 
        tw_ad(i,level)=tw_ad(i,level)                      + &
                      & predictors_ad % mixedgas(10,level,i)  * &
                      & 0.25_JPRB*predictors% mixedgas(9,level,i)     / &
                      & tw(i,level)**0.75 
      End Do
    End Do

    Do level = 1,prof_ad(1) % nlevels
      Do i=1,nprofiles

        If(coef % id_inst == 3 .and. coef % IncZeeman)Then  
  ! AMSUA ch14 with Zeeman effect (coef % nmixed = 14, otherwise = 10)
          raytracing_ad % pathsat(level,i) = raytracing_ad % pathsat(level,i) + &
             & 2.0_JPRB * (prof(i) % cosbk * prof(i) % Be)**2 * &                                                        
             & raytracing % pathsat(level,i) * predictors_ad % mixedgas(14,level,i) + &                                    
             & prof(i) % Be**3 *               predictors_ad % mixedgas(13,level,i) + &                                                 
             & 2.0_JPRB * prof(i) % Be * &                                                                 
             & raytracing % pathsat(level,i) * predictors_ad % mixedgas(12,level,i) + &                                                                 
             & prof(i) % cosbk**2 *            predictors_ad % mixedgas(11,level,i)                  
          predictors_ad % mixedgas(11:14,level,i) = 0.0_JPRB                                    
        Endif           

        raytracing_ad % pathsat(level,i)=raytracing_ad % pathsat(level,i)            + &
                      & predictors_ad % mixedgas(9,level,i)*0.5_JPRB                 / &
                      & predictors% mixedgas(9,level,i) 
  
        raytracing_ad % pathsat(level,i)=raytracing_ad % pathsat(level,i)            + &
                      & predictors_ad % mixedgas(8,level,i)*tw(i,level)/tr(i,level) 
        tw_ad(i,level)=tw_ad(i,level)+predictors_ad % mixedgas(8,level,i)              * &
                      & raytracing % pathsat(level,i)  /tr(i,level) 
        tr_ad(i,level)=tr_ad(i,level)-predictors_ad % mixedgas(8,level,i)*tw(i,level)    * &
                      & raytracing % pathsat(level,i)/tr(i,level)**2._JPRB 
    
        raytracing_ad % pathsat(level,i)=raytracing_ad % pathsat(level,i)            + &
                      & predictors_ad % mixedgas(7,level,i)*tw(i,level) 
        tw_ad(i,level)=tw_ad(i,level)+predictors_ad % mixedgas(7,level,i)              * &
                      & raytracing % pathsat(level,i) 
      
        tr_ad(i,level) = tr_ad(i,level)   + predictors_ad % mixedgas(6,level,i)*2      * &
                      & predictors% mixedgas(5,level,i) 
    
        tr_ad(i,level) = tr_ad(i,level)     + predictors_ad % mixedgas(5,level,i)
    
        tr_ad(i,level) = tr_ad(i,level)   + predictors_ad % mixedgas(4,level,i)*2      * &
                      & predictors% mixedgas(3,level,i) 
        raytracing_ad % pathsat(level,i)= raytracing_ad % pathsat(level,i)           + &
                      & predictors_ad % mixedgas(4,level,i)                          * &
                      & predictors% mixedgas(5,level,i)**2 
    
        tr_ad(i,level) = tr_ad(i,level)    + predictors_ad % mixedgas(3,level,i)       * &
                      & raytracing % pathsat(level,i) 
        raytracing_ad % pathsat(level,i) = raytracing_ad % pathsat(level,i)          + &
                      & predictors_ad % mixedgas(3,level,i)*tr(i,level) 
    
        raytracing_ad % pathsat(level,i)=raytracing_ad % pathsat(level,i)            + &
                      & predictors_ad % mixedgas(2,level,i)*2*raytracing % pathsat(level,i) 
  
        raytracing_ad % pathsat(level,i)=raytracing_ad % pathsat(level,i)            + &
                      & predictors_ad % mixedgas(1,level,i) 
      End Do
    End Do


  Else   ! ssmisz predictors (Zeeman-affected channels)

    Do level = 1, prof_ad(1) % nlevels
      Do i=1,nprofiles
        Predictors_ad % mixedgas(10,level,i) = Predictors_ad % mixedgas(10,level,i) + &
                                             & Predictors_ad % mixedgas(11,level,i) * prof(i) % Be
        Predictors_ad % mixedgas(3, level,i) = Predictors_ad % mixedgas(3, level,i) + &
                                             & Predictors_ad % mixedgas(10,level,i) * prof(i) % Be
        raytracing_ad % pathsat(level,i)     = raytracing_ad % pathsat(level,i) + &
                                             & Predictors_ad % mixedgas(9, level,i) * prof(i) % Be**3 + &
                                             & Predictors_ad % mixedgas(8, level,i) * prof(i) % Be        
        Predictors_ad % mixedgas(6, level,i) = Predictors_ad % mixedgas(6, level,i) + &
                                             & Predictors_ad % mixedgas(7, level,i) / prof(i) % Be
        raytracing_ad % pathsat(level,i)     = raytracing_ad % pathsat(level,i) + &
                                             & Predictors_ad % mixedgas(6, level,i) / prof(i) % Be        
        Predictors_ad % mixedgas(2,level,i)  = Predictors_ad % mixedgas(2, level,i) + &
                                             & Predictors_ad % mixedgas(5, level,i) * prof(i) % cosbk**2
        Predictors_ad % mixedgas(2,level,i)  = Predictors_ad % mixedgas(2, level,i) + &
                                             & Predictors_ad % mixedgas(4, level,i) / prof(i) % Be
        raytracing_ad % pathsat(level,i)     = raytracing_ad % pathsat(level,i) + &
                                             & Predictors_ad % mixedgas(3, level,i) * prof(i) % cosbk**2 + &
                                             & Predictors_ad % mixedgas(2, level,i) * (300.0_JPRB / t(i, level))        
        t_ad(i,level) = t_ad(i,level) - &
                 (Predictors % mixedgas(2,level,i) / t(i, level)) * Predictors_ad % mixedgas(2, level,i)         
        raytracing_ad % pathsat(level,i) = raytracing_ad % pathsat(level,i) + Predictors_ad % mixedgas(1, level,i)

        Predictors_ad % mixedgas(1:11, level,i) = 0.0_JPRB
       Enddo 
    Enddo

  Endif

  sum1 = 0._JPRB
  Do level = prof_ad(1) % nlevels, 1, -1
    Do i=1,nprofiles
     If ( prof(i) % ozone_Data .And. coef % nozone > 0 ) Then
        sum1(i) = sum1(i) + ow_ad (i,level ) / sum2_ow(i,level)
        o_ad(i,level ) = o_ad(i,level ) + sum1(i) * coef % dpp( level )
     Else
        o_ad(i,level) = 0._JPRB
     End If
    End Do
  End Do

  sum1 = 0._JPRB
  Do level = prof_ad(1) % nlevels, 1, -1
    Do i=1,nprofiles
     sum1(i) = sum1(i) + ww_ad ( i,level ) / sum2_ww(i,level)
     w_ad(i,level ) = w_ad( i,level ) + sum1(i) * coef % dpp( level )
    End Do
  End Do

  Do level = prof_ad(1) % nlevels, 2, -1
    Do i=1,nprofiles
      tw_ad( i,level-1 ) = tw_ad( i,level-1 ) + tw_ad( i,level )
      tr_ad( i,level-1 ) = tr_ad( i,level-1 ) + tw_ad( i,level ) *&
            & coef % dpp( level ) 
    End Do
  End Do


!CDIR NOLOOPCHG
  Do level=1,prof(1)%nlevels
    Do i=1,nprofiles
      If ( prof(i) % ozone_Data .And. coef % nozone > 0 ) Then
        o_ad(i,level ) = o_ad(i,level) + or_ad(i,level) / coef % ostar(level)
      Endif
    EndDo
  EndDo


!CDIR NOLOOPCHG
  Do level=1,prof(1)%nlevels
    Do i=1,nprofiles
      w_ad(i,level) = w_ad(i,level) + wr_ad(i,level) / coef % wstar(level)
      t_ad(i,level) = t_ad(i,level) + tr_ad(i,level) / coef % tstar(level)
    Enddo
  Enddo


  Do level=1,prof(1)%nlevels
    Do i=1,nprofiles
      If ( prof(i) % ozone_Data .And. coef % nozone > 0 ) Then
       t_ad(i,level) = t_ad(i,level) + dto_ad(i,level)
      Endif
      t_ad(i,level) = t_ad(i,level) + dt_ad(i,level)
    Enddo
  Enddo


  Do level=2,prof(1)%nlevels
    Do i=1,nprofiles
     If ( prof(i) % ozone_Data .And. coef % nozone > 0 ) Then
       prof_ad(i) % o3(level-1) = prof_ad(i) % o3(level-1) + 0.5_JPRB *o_ad(i,level ) 
       prof_ad(i) % o3(level)   = prof_ad(i) % o3(level) + 0.5_JPRB *o_ad(i,level ) 
     Endif
    Enddo
  Enddo
  Do i=1,nprofiles
    If ( prof(i) % ozone_Data .And. coef % nozone > 0 ) Then
     prof_ad(i) % o3(1) = prof_ad(i) % o3(1) + o_ad(i,1)
    End If
  Enddo
  !
  ! include adjoint surface humidity
  !
  If ( use_q2m )Then
    Do i=1,nprofiles
  ! include surface humidity
      iv2 = aux(i) % nearestlev_surf
      prof_ad(i) % s2m % q = prof_ad(i) % s2m % q + 0.5_JPRB *w_ad(i,iv2)
      prof_ad(i) % q(iv3(i)) = prof_ad(i) % q(iv3(i)) + 0.5_JPRB *w_ad(i,iv2)
    End Do
    Do i=1,nprofiles
      Do level=2,iv3(i)
        prof_ad(i) % q(level-1) = prof_ad(i) % q(level-1) + 0.5_JPRB *w_ad(i,level)
        prof_ad(i) % q(level)   = prof_ad(i) % q(level) + 0.5_JPRB *w_ad(i,level )
      Enddo
    End Do
  Else
    Do level=2,prof_ad(1) % nlevels
      Do i=1,nprofiles
        prof_ad(i) % q(level-1) = prof_ad(i) % q(level-1) + 0.5_JPRB *w_ad(i,level )
        prof_ad(i) % q(level)  = prof_ad(i) % q(level) + 0.5_JPRB *w_ad(i,level )
      Enddo
    Enddo
  Endif

  Do i=1,nprofiles
    prof_ad(i) % q(1) = prof_ad(i) % q(1) + w_ad(i,1)
  End Do

  Do level=2,prof_ad(1) % nlevels
    Do i=1,nprofiles
      prof_ad(i) % t(level-1) = prof_ad(i) % t(level-1) + 0.5_JPRB *t_ad(i,level ) 
      prof_ad(i) % t(level)   = prof_ad(i) % t(level) + 0.5_JPRB *t_ad(i,level ) 
    Enddo
  Enddo
  Do i=1,nprofiles
    prof_ad(i) % t(1) = prof_ad(i) % t(1) + t_ad(i,1)
  End Do

IF (LHOOK) CALL DR_HOOK('RTTOV_SETPREDICTORS_7_AD',1_jpim,ZHOOK_HANDLE)


End Subroutine rttov_setpredictors_7_ad
