!
Subroutine rttov_tl(    &
      & errorstatus,       &! out
      & nprofiles,         &! in
      & nchannels,         &! in
      & channels,          &! in
      & lprofiles,         &! in
      & addinterp,         &! in
      & profiles,          &! in 
      & profiles_tl,       &! in 
      & coef,              &! in
      & coef_scatt_ir,     &! in
      & optp,              &! in
      & addsun,            &! in
      & addaerosl,         &! in
      & addclouds,         &! in
      & calcemis,          &! in
      & emissivity,        &! inout
      & emissivity_tl,     &! inout
      & transmission,      &! inout
      & transmission_tl,   &! inout
      & radiancedata,      &! inout
      & radiancedata_tl,   &! inout 
      & temp,              &! inout
      & temp_tl )   
  !
  ! Description:
  ! Tangent Linear of rttov_direct
  ! to compute multi-channel level to space transmittances,
  ! top of atmosphere and level to space radiances and brightness
  ! temperatures and optionally surface emissivities, for many
  ! profiles in a single call, for satellite
  ! infrared or microwave sensors. The code requires a coefficient file
  ! for each sensor for which simulated radiances are requested.
  !
  ! Copyright:
  !    This software was developed within the context of
  !    the EUMETSAT Satellite Application Facility on
  !    Numerical Weather Prediction (NWP SAF), under the
  !    Cooperation Agreement dated 25 November 1998, between
  !    EUMETSAT and the Met Office, UK, by one or more partners
  !    within the NWP SAF. The partners in the NWP SAF are
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  !
  !    Copyright 2002, EUMETSAT, All Rights Reserved.
  !
  ! Method: The methodology is described in the following:
  !
  ! Eyre J.R. and H.M. Woolf  1988 Transmittance of atmospheric gases
  ! in the microwave region: a fast model. Applied Optics 27  3244-3249
  !
  ! Eyre J.R. 1991 A fast radiative transfer model for satellite sounding
  ! systems.  ECMWF Research Dept. Tech. Memo. 176 (available from the
  ! librarian at ECMWF).
  !
  ! Saunders R.W., M. Matricardi and P. Brunel 1999 An Improved Fast Radiative
  ! Transfer Model for Assimilation of Satellite Radiance Observations.
  ! QJRMS, 125, 1407-1425.
  !
  ! Matricardi, M., F. Chevallier and S. Tjemkes 2001 An improved general
  ! fast radiative transfer model for the assimilation of radiance
  ! observations. ECMWF Research Dept. Tech. Memo. 345
  ! (available from the librarian at ECMWF).
  !
  ! Matricardi, M. 2003 RTIASI-4, a new version of the ECMWF fast radiative
  ! transfer model for the infrared atmospheric sounding interferometer.
  ! ECMWF Research Dept. Tech. Memo. 425 (available from the librarian at ECMWF)
  !
  ! Rochon Y.J., L. Garand, D.S. Turner and S. Polavarapu Jacobian mapping 
  ! between  vertical coordinate systems in data assimilation (submitted QJRMS
  ! June 2006) 
  ! 
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date     Comment
  ! -------   ----     -------
  !  1.1   01/12/2002  New F90 code with structures (P Brunel A Smith)
  !  1.2   02/01/2003  More comments added (R Saunders)
  !  1.3   24/01/2003  Error return code by input profile (P Brunel)
  !  1.4               Add WV Continuum and CO2 capability
  !  1.5   04/12/2003  Optimisation (J Hague and D Salmond ECMWF)
  !  1.6   02/06/2004  Change tests on id_comp_lvl == 7 by tests on fmv_model_ver (P. Brunel)
  !  1.7   08/12/2005  Added surface humidity to lowest mean layer q  (R Saunders) 
  !  1.8   23/01/2006  Marco Matricardi (ECMWF):
  !              --    IASI capability added.
  !              --    Variable trace gases added for IASI and AIRS.
  !              --    Solar radiation added for IASI and AIRS
  !              --    Altitude dependent local zenith angle added.
  !              --    Linear in tau approximation for RT equation added.
  !  1.9   23/01/2006 Changed intent of profiles_tl to inout (R Saunders)
  !  1.10  31/03/2006  Initialised zlev (R Saunders)
  !        06/02/2007  Removed polarisation indexing R. Saunders.
  !        29/06/2007  Introduce interpolator (P.J.Rayer)
  !              --    Profiles on USER levs -> COEF levs
  !              --    Duplicate profaux and setgeometry (USER and COEF levs)
  !              --    Predict atm/sol optical depths on COEF levs -> USER levs
  !              --    Calculate transmittance and integrate RT on USER levs
  !  1.11  11/10/2007  Move iaernum & iaertyp to profile_aux P.Marguinaud
  !                    Initialize weights_calc array
  !  1.12   16/01/2008 Facility to apply regression limits (N. Bormann)
  !  1.13   26/09/2008 Disable vectorization of the pointer association
  !                    loop and nullify temp pointers; this is for the NEC 
  !                    under -Chopt. P. Marguinaud
  !  1.12   03/10/2008 Fix bug in checking addsolar
  !
  ! Code Description:
  !   Language:           Fortran 90.
  !   Software Standards: "European Standards for Writing and
  !     Documenting Exchangeable Fortran 90 Code".
  ! A user guide and technical documentation is available at
  ! http://www.metoffice.com/research/interproj/nwpsaf/rtm/index.html
  !
  ! Declarations:
  ! Modules used:
  ! Imported Parameters:
  Use rttov_const, Only :   &
        & errorstatus_success ,&
        & errorstatus_warning ,&
        & errorstatus_fatal   ,&
        & max_optical_depth   ,&
        & sensor_id_mw        ,&
        & sensor_id_ir        ,&
        & sensor_id_hi        ,&
        & sensor_id_po        ,&
        & ncldtyp             ,&
        & ngases_max          ,&
        & apply_reg_limits
 
  USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

  ! Imported Type Definitions:
  Use rttov_types, Only :             &
        & rttov_coef                 ,&
        & rttov_coef_scatt_ir        ,&
        & transmission_scatt_ir_type ,&
        & rttov_optpar_ir            ,&
        & profile_Type               ,&
        & geometry_Type              ,&
        & predictors_Type            ,&
        & profile_aux                ,&
        & opdp_path_type             ,&
        & transmission_Type          ,&
        & radiance_Type              ,&
        & radiance_aux               ,&
        & raytracing_type            ,&
        & sunglint_type              ,&
        & ircld_type                 ,&
        & blob_type                  ,& 
        & rttov_temp    

  Use rttov_global, Only: &
        & verbose_checkinput_warnings

  Use parkind1, Only : jpim     ,jprb
  Implicit None


  !subroutine arguments:
  Integer(Kind=jpim),        Intent(in)    :: nprofiles                    ! Number of profiles
  Integer(Kind=jpim),        Intent(in)    :: nchannels                    ! Number of radiances computed (channels used * profiles)
  Integer(Kind=jpim),        Intent(out)   :: errorstatus   (nprofiles)
  Integer(Kind=jpim),        Intent(in)    :: channels      (nchannels)
  Integer(Kind=jpim),        Intent(in)    :: lprofiles     (nchannels)
  Logical,                   Intent(in)    :: addsun
  Logical,                   Intent(in)    :: addaerosl
  Logical,                   Intent(in)    :: addclouds
  Logical,                   Intent(in)    :: addinterp                  ! switch for User-Coef-User interpolations

  Type(profile_Type),        Intent(in)    :: profiles      (nprofiles)
  Type(profile_Type),        Intent(inout) :: profiles_tl   (nprofiles)

  Type(rttov_coef),          Intent(in)    :: coef
  Type(rttov_coef_scatt_ir), Intent(in)    :: coef_scatt_ir
  Type(rttov_optpar_ir),     Intent(in)    :: optp

  Logical,                   Intent(in)    :: calcemis      (nchannels)

  Real(Kind=jprb),           Intent(inout) :: emissivity    (nchannels)
  Real(Kind=jprb),           Intent(inout) :: emissivity_tl (nchannels)

  Type(transmission_Type),   Intent(inout) :: transmission               ! in because of meme allocation
  Type(transmission_Type),   Intent(inout) :: transmission_tl            ! in because of meme allocation

  Type(radiance_Type),       Intent(inout) :: radiancedata               ! in because of meme allocation
  Type(radiance_Type),       Intent(inout) :: radiancedata_tl            ! in because of meme allocation
  Type(rttov_temp),          Intent(inout), Optional :: temp, temp_tl

!INTF



#include "rttov_errorreport.interface"
#include "rttov_checkinput.interface"
#include "rttov_checkinput_tl.interface"
#include "rttov_profaux.interface"
#include "rttov_setgeometry.interface"
#include "rttov_setpredictors_7.interface"
#include "rttov_setpredictors_8.interface"
#include "rttov_setpredictors_9.interface"
#include "rttov_opdep.interface"
#include "rttov_opdep_9.interface"
#include "rttov_opdep_9_solar.interface"
#include "rttov_transmit.interface"
#include "rttov_transmit_9_solar.interface"
#include "rttov_calcemis_ir.interface"
#include "rttov_calcemis_mw.interface"
#include "rttov_integrate.interface"
#include "rttov_intavg_prof.interface"
#include "rttov_intavg_chan.interface"
#include "rttov_profaux_tl.interface"
#include "rttov_setgeometry_tl.interface"
#include "rttov_setpredictors_7_tl.interface"
#include "rttov_setpredictors_8_tl.interface"
#include "rttov_setpredictors_9_tl.interface"
#include "rttov_opdep_tl.interface"
#include "rttov_opdep_9_tl.interface"
#include "rttov_opdep_9_solar_tl.interface"
#include "rttov_transmit_tl.interface"
#include "rttov_transmit_9_solar_tl.interface"
#include "rttov_calcemis_mw_tl.interface"
#include "rttov_integrate_tl.interface"
#include "rttov_intavg_prof_tl.interface"
#include "rttov_intavg_chan_tl.interface"
#include "rttov_alloc_trans_scatt_ir.interface"
#include "rttov_alloc_prof.interface"
#include "rttov_alloc_transmission.interface"
#include "rttov_refsun.interface"
#include "rttov_refsun_tl.interface"
#include "rttov_check_temp.interface"
#include "rttov_init_profiles.interface"
#include "rttov_checkargs.interface"
#include "rttov_setpredictors_9_solar.interface" 
#include "rttov_setpredictors_9_solar_tl.interface" 
#include "rttov_cldstr.interface" 
#include "rttov_cldstr_tl.interface" 
#include "rttov_opdpscattir.interface" 
#include "rttov_opdpscattir_tl.interface" 
#include "rttov_fresnel.interface" 
#include "rttov_fresnel_tl.interface" 
#include "rttov_init_raytracing.interface"

  !local variables:
  Integer(Kind=jpim) :: i,j, prof, chan                                          ! loop index
  Integer(Kind=jpim) :: nstreams
  Integer(Kind=jpim) :: alloc_status(111)                                ! memory allocation status
  Integer(Kind=jpim) :: callnum                                          ! which interpolator call: 1=USER->COEF, 2=COEF->USER 
  Integer(Kind=jpim) :: nvar, nc1, nc2, igas                             ! no of profile elements to interpolator (USER->COEF)

  Logical            :: addcosmic                                        ! switch for adding temp of cosmic background
  Logical            :: intvar(ngases_max)
  Logical            :: sun(nchannels)                   ! solar switch located in channel sequence

  Real(Kind=jprb)    :: reflectivity(nchannels)                          ! surface reflectivity
  Real(Kind=jprb)    :: reflectivity_tl(nchannels)                       ! TL surface reflectivity

  Real(Kind=jprb)    :: fresnrefl(nchannels)
  Real(Kind=jprb)    :: fresnrefl_tl(nchannels)

  Real(Kind=jprb)    :: tau_ref          (profiles(1)%nlevels,nchannels)
  Real(Kind=jprb)    :: tau_ref_surf     (nchannels)
  Real(Kind=jprb)    :: tau_surf         (nchannels)

  Real(Kind=jprb)    :: tausun_ref       (profiles(1)%nlevels,nchannels)
  Real(Kind=jprb)    :: tausun_ref_surf  (nchannels)
  Real(Kind=jprb)    :: tausun_layer     (profiles(1)%nlevels,nchannels)
  Real(Kind=jprb)    :: tausun_surf      (nchannels)

  Real(Kind=jprb)    :: opdp_ref         (profiles(1)%nlevels,nchannels) ! layer optical depth before threshold
  Real(Kind=jprb)    :: opdp_ref_COEF    (coef%nlevels,nchannels)        ! layer optical depth before threshold
  Real(Kind=jprb)    :: od_layer         (profiles(1)%nlevels,nchannels) ! sat to layer optical depth
  Real(Kind=jprb)    :: od_layer_COEF    (coef%nlevels,nchannels)        ! sat to layer optical depth
  Real(Kind=jprb)    :: tau_layer        (profiles(1)%nlevels,nchannels) ! sat to layer transmittance

  Real(Kind=jprb)    :: opdpsun_ref      (profiles(1)%nlevels,nchannels) ! layer optical depth
  Real(Kind=jprb)    :: opdpsun_ref_COEF (coef%nlevels,nchannels)        ! layer optical depth
  Real(Kind=jprb)    :: odsun_layer      (profiles(1)%nlevels,nchannels) ! sat to layer optical depth
  Real(Kind=jprb)    :: odsun_singlelayer(profiles(1)%nlevels,nchannels) ! sat to layer optical depth
  Real(Kind=jprb)    :: od_frac          (nchannels)

  Character (len=80) :: errMessage
  Character (len=8)  :: NameOfRoutine = 'rttov_tl'

  Type(profile_Type)    :: profiles_COEF_ref (nprofiles)   
  Type(blob_type)       :: profiles_COEF_blob_ref

  Type(geometry_Type)   :: angles            (nprofiles)   ! geometry angles
  Type(geometry_Type)   :: angles_COEF       (nprofiles)   ! geometry angles


  Type(profile_aux)     :: aux_prof          (nprofiles)   ! auxillary profiles informations
  Type(profile_aux)     :: aux_prof_tl       (nprofiles)   ! TL of above aux_prof

  Type(profile_aux)     :: aux_prof_COEF     (nprofiles)   ! auxillary profiles informations
  Type(profile_aux)     :: aux_prof_COEF_tl  (nprofiles)   ! TL of above aux_prof_COEF

  Type(radiance_aux)    :: auxrad

  Type(radiance_aux)    :: auxrad_stream
  Type(radiance_aux)    :: auxrad_stream_tl



  Type(sunglint_type)   :: sunglint          (nprofiles)
  Type(sunglint_type)   :: sunglint_tl       (nprofiles)


  

  Type(transmission_scatt_ir_type)      :: transmission_scatt_ir_stream
  Type(transmission_scatt_ir_type)      :: transmission_scatt_ir_stream_tl

  Type(transmission_scatt_ir_type)      :: transmission_scatt_ir
  Type(transmission_scatt_ir_type)      :: transmission_scatt_ir_tl

  Real(Kind=jprb), target ::  zdg              (profiles(1)%nlevels,nprofiles)
  Real(Kind=jprb), target ::  zfac1_dg         (profiles(1)%nlevels,nprofiles)
  Real(Kind=jprb), target ::  zfac2_dg         (profiles(1)%nlevels,nprofiles)
  Real(Kind=jprb), target ::  zfac3_dg         (profiles(1)%nlevels,nprofiles)
  Real(Kind=jprb), target ::  zdg_tl           (profiles(1)%nlevels,nprofiles)
  Real(Kind=jprb), target ::  zfac1_dg_tl      (profiles(1)%nlevels,nprofiles)
  Real(Kind=jprb), target ::  zfac2_dg_tl      (profiles(1)%nlevels,nprofiles)
  Real(Kind=jprb), target ::  zfac3_dg_tl      (profiles(1)%nlevels,nprofiles)

  Real(Kind=jprb), target ::  zdeb   (5,profiles(1)%nlevels,nprofiles)
  Real(Kind=jprb), target ::  zdeb_COEF   (5,coef%nlevels,nprofiles)
  Real(Kind=jprb), target ::  zdeb_tl(5,profiles(1)%nlevels,nprofiles)
  Real(Kind=jprb), target ::  zdeb_COEF_tl(5,coef%nlevels,nprofiles)


  Real(Kind=jprb), target ::  surfair(nchannels)
  Real(Kind=jprb), target ::  skin   (nchannels)
  Real(Kind=jprb), target ::  cosmic (nchannels)
  Real(Kind=jprb), target ::  layer(profiles(1)%nlevels,nchannels)
  Real(Kind=jprb), target ::  up        (profiles(1)%nlevels,nchannels)
  Real(Kind=jprb), target ::  down      (profiles(1)%nlevels,nchannels)
  Real(Kind=jprb), target ::  down_cloud(profiles(1)%nlevels,nchannels)


  Real(Kind=jprb), target ::  zpsi         (coef%ws_nomega,nprofiles)
  Real(Kind=jprb), target ::  zpsi_tl      (coef%ws_nomega,nprofiles)
  Real(Kind=jprb), target ::  zbeta        (coef%ws_nomega,nprofiles)
  Real(Kind=jprb), target ::  zbeta_tl     (coef%ws_nomega,nprofiles)
  Integer(Kind=jpim) :: jn,ncha,nchb
  Type(rttov_temp), Target  :: temp1
  Type(rttov_temp), Pointer :: temp0
  Type(rttov_temp), Target  :: temp1_tl
  Type(rttov_temp), Pointer :: temp0_tl
  Integer(Kind=jpim)        :: temp_status, checkargs_status
  
  

  REAL(KIND=JPRB) :: ZHOOK_HANDLE
  !- End of header --------------------------------------------------------

!Print *, addclouds, addaerosl
!Print *, size( optp%optpaer ), size( optp%optpwcl ), size( optp%optpicl )
  !-------------
  !0. initialize
  !-------------

  IF (LHOOK) CALL DR_HOOK('RTTOV_TL',0_jpim,ZHOOK_HANDLE)
  errorstatus(:)  = errorstatus_success
  alloc_status(:) = 0

  Call rttov_checkargs(  &
      & addinterp,       &
      & addsun,          &
      & addaerosl,       &
      & addclouds,       &
      & checkargs_status &
  )
  
  If( checkargs_status .eq. errorstatus_fatal ) Then
     Write( errMessage, '( "checkargs fatal error")' )
     Call Rttov_ErrorReport( errorstatus_fatal, errMessage, NameOfRoutine )
     errorstatus(:)  = errorstatus_fatal
     IF (LHOOK) CALL DR_HOOK('RTTOV_TL',1_jpim,ZHOOK_HANDLE)
     Return
  EndIf

  Nullify( temp0 )
  Nullify( temp0_tl )

  !-----------------------------------------------------------------------
  !1. interpolator first call - input profiles from USER levs to COEF levs
  !-----------------------------------------------------------------------
  ! done immediately in direct code - for quick escape if target values are out of bounds

  Call rttov_check_temp( &
    & temp_status,                             &
    & nprofiles,                               &
    & nchannels,                               &
    & channels,                                &
    & addinterp,                               &
    & profiles(1)%nlevels,                     &
    & coef,                                    &
    & coef_scatt_ir,                           &
    & optp,                                    &
    & addsun .or. Any( profiles(:)%addsolar ), &
    & addaerosl,                               &
    & addclouds,                               &
    & 1_jpim,                                  &
    & temp0 = temp0, temp0_tl = temp0_tl,      &
    & temp1 = temp1, temp1_tl = temp1_tl,      &
    & temp2 = temp,  temp2_tl = temp_tl )
  If( temp_status .ne. errorstatus_success ) Then
    Write( errMessage, '( "rttov_check_temp fatal error ")' )
    Call Rttov_ErrorReport( errorstatus_fatal, errMessage, NameOfRoutine )
    Return
  EndIf
  

  Call rttov_init_profiles( nprofiles, temp0%profiles_COEF )



  Do i = 1 , nprofiles
    temp0%profiles_COEF(i) % nlevels =  coef % nlevels
    temp0%profiles_COEF(i) % p(:) = coef % ref_prfl_p(:)
  Enddo
  If (addinterp) Then
! set up profile variables for interpolation
! easy extension to more trace following gas id codes in rttov_const
! add more trace gases here if appropriate
      Call rttov_intavg_prof( &
            & nprofiles,                       &
            & profiles(1)%nlevels,             & !in
            & temp0%profiles_COEF(1)%nlevels,        & !in
            & profiles,                     & !in     source variables
            & temp0%profiles_COEF,                & !inout  target variables
            & errorstatus)                    !out errorstatus
      If ( Any(errorstatus == errorstatus_fatal) ) Then
        IF (LHOOK) CALL DR_HOOK('RTTOV_TL',1_jpim,ZHOOK_HANDLE)
        Return
      End If
  Else 
! COEF levs same as USER levs
    Do i = 1, nprofiles
      temp0%profiles_COEF(i)%t(:)   =    profiles(i)%t(:)
      temp0%profiles_COEF(i)%q(:)   =    profiles(i)%q(:)
      if(profiles(i) % clw_data)   temp0%profiles_COEF(i)%clw(:) =  profiles(i)%clw(:)
      if(profiles(i) % ozone_data) temp0%profiles_COEF(i)%o3(:)  =  profiles(i)%o3(:)
      if(profiles(i) % co2_data)   temp0%profiles_COEF(i)%co2(:) =  profiles(i)%co2(:)
      if(profiles(i) % n2o_data)   temp0%profiles_COEF(i)%n2o(:) =  profiles(i)%n2o(:)   
      if(profiles(i) % co_data)    temp0%profiles_COEF(i)%co(:)  =  profiles(i)%co(:)    
      if(profiles(i) % ch4_data)   temp0%profiles_COEF(i)%ch4(:) =  profiles(i)%ch4(:)  
    End Do
  EndIf
! complete profiles on COEF levs for surf, skin, cloud, aerosol, solar, angle
  Do i = 1, nprofiles
    temp0%profiles_COEF(i) % ozone_data = profiles(i) % ozone_data
    temp0%profiles_COEF(i) % co2_data   = profiles(i) % co2_data
    temp0%profiles_COEF(i) % clw_data   = profiles(i) % clw_data

    temp0%profiles_COEF(i) % n2o_data    = profiles(i) % n2o_data
    temp0%profiles_COEF(i) % co_data     = profiles(i) % co_data
    temp0%profiles_COEF(i) % ch4_data    = profiles(i) % ch4_data
    temp0%profiles_COEF(i) % aer_data    = profiles(i) % aer_data
    temp0%profiles_COEF(i) % cld_data    = profiles(i) % cld_data

    temp0%profiles_COEF(i) % S2m        = profiles(i) % S2m
    temp0%profiles_COEF(i) % skin       = profiles(i) % skin
    temp0%profiles_COEF(i) % ctp        = profiles(i) % ctp
    temp0%profiles_COEF(i) % cfraction  = profiles(i) % cfraction

! mod RWS 24/7/07
!    temp0%profiles_COEF(i) % aerosols(:,:) = profiles(i) % aerosols(:,:)
!    temp0%profiles_COEF(i) % cloud(:,:)    = profiles(i) % cloud(:,:)
!    temp0%profiles_COEF(i) % cfrac(:,:)    = profiles(i) % cfrac(:,:)

    temp0%profiles_COEF(i) % idg    = profiles(i) % idg
    temp0%profiles_COEF(i) % ish    = profiles(i) % ish

    temp0%profiles_COEF(i) % zenangle   = profiles(i) % zenangle
    temp0%profiles_COEF(i) % azangle    = profiles(i) % azangle

    temp0%profiles_COEF(i) % sunzenangle = profiles(i) % sunzenangle
    temp0%profiles_COEF(i) % sunazangle  = profiles(i) % sunazangle
    temp0%profiles_COEF(i) % latitude    = profiles(i) % latitude
    temp0%profiles_COEF(i) % elevation   = profiles(i) % elevation
    temp0%profiles_COEF(i) % addsolar    = profiles(i) % addsolar
    temp0%profiles_COEF(i) % addrefrac   = profiles(i) % addrefrac  

    temp0%profiles_COEF(i) % Be     = profiles(i) % Be  
    temp0%profiles_COEF(i) % cosbk  = profiles(i) % cosbk  
!

  End Do      ! over profiles

  Call rttov_init_raytracing ( temp0%raytracing )

  !------------------------------------------------------
  !2. check input data is within suitable physical limits
  !------------------------------------------------------
  If ( apply_reg_limits ) Then

    Call rttov_alloc_prof( alloc_status(1), nprofiles, profiles_COEF_ref, &
      coef%nlevels, coef_scatt_ir, 1_jpim, blob = profiles_COEF_blob_ref )
    If( Any(alloc_status /= 0) ) Then
      errorstatus = errorstatus_fatal
      Write( errMessage, '( "allocation of profiles_COEF_ref")' )
      Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
      IF (LHOOK) CALL DR_HOOK('RTTOV_TL',1_jpim,ZHOOK_HANDLE)
      Return
    End If

    profiles_coef_ref = temp0%profiles_coef

  End If

  Do i = 1, nprofiles

     Call rttov_checkinput( &
           & temp0%profiles_COEF( i ),&!inout
           & coef,              &!in
           & errorstatus(i)    ) !out 

  End Do

  ! test check input return code
  If ( any( errorstatus(:) == errorstatus_warning)  .and. verbose_checkinput_warnings ) Then
     Do i = 1, nprofiles
        If ( errorstatus(i) == errorstatus_warning ) Then
           Write( errMessage, '( "checkinput warning error for profile",i4)' ) i
           Call Rttov_ErrorReport (errorstatus_warning, errMessage, NameOfRoutine)
        End If
     End Do
  End If

  If ( any( errorstatus(:) == errorstatus_fatal ) ) Then
     Do i = 1, nprofiles
        If ( errorstatus(i) == errorstatus_fatal ) Then
           ! Some unphysical values; Do not run RTTOV
           Write( errMessage, '( "checkinput fatal error for profile",i4)' ) i
           Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
        End If
     End Do
     ! nothing processed so all profiles get the fatal error code
     ! user will know which profile
     errorstatus(:) = errorstatus_fatal
     IF (LHOOK) CALL DR_HOOK('RTTOV_TL',1_jpim,ZHOOK_HANDLE)
     Return
  End If

  !------------------------------------------------------------
  !3. determine cloud top, surface levels, ice cloud parameters
  !------------------------------------------------------------

  !3.1 USER levs
  !-------------
  If( coef % id_sensor == sensor_id_mw .or. coef % id_sensor == sensor_id_po ) Then
     jn=profiles(1)%nlevels
!CDIR NOVECTOR
     Do i = 1, nprofiles
       aux_prof(i) % debye_prof => zdeb(1:5,1:jn,i)
     End Do
  Endif
  If( addclouds ) Then
     jn=profiles(1)%nlevels
!CDIR NOVECTOR
     Do i = 1, nprofiles
       aux_prof(i) % dg => zdg(1:jn,i)
       aux_prof(i) % fac1_dg => zfac1_dg(1:jn,i)
       aux_prof(i) % fac2_dg => zfac2_dg(1:jn,i)
       aux_prof(i) % fac3_dg => zfac3_dg(1:jn,i)
     End Do
  Endif
  Do i = 1, nprofiles
     Call rttov_profaux( &
           & addclouds,     &! in
           & profiles(i),   &! in
           & coef,          &! in
           & aux_prof(i))    ! inout 
  End Do

   !3.2 COEF levs
  !-------------

  If( coef % id_sensor == sensor_id_mw .or. coef % id_sensor == sensor_id_po ) Then
     jn=coef%nlevels
!CDIR NOVECTOR
     Do i = 1, nprofiles
       aux_prof_COEF(i) % debye_prof => zdeb_COEF(1:5,1:jn,i)
     End Do
  Endif
  Do i = 1, nprofiles
    If(addinterp) then
      Call rttov_profaux( &
           & .false.  ,        &! no cloud computations on coef levels
           & temp0%profiles_COEF(i), &! in
           & coef,             &! in
           & aux_prof_COEF(i))  ! inout 
    Else
      aux_prof_COEF(i) % nearestlev_surf = aux_prof(i) % nearestlev_surf
      aux_prof_COEF(i) % pfraction_surf  = aux_prof(i) % pfraction_surf
      aux_prof_COEF(i) % nearestlev_ctp  = aux_prof(i) % nearestlev_ctp 
      aux_prof_COEF(i) % pfraction_ctp   = aux_prof(i) % pfraction_ctp
      aux_prof_COEF(i) % cfraction       = aux_prof(i) % cfraction 
      If( coef % id_sensor == sensor_id_mw  .or.coef % id_sensor == sensor_id_po ) Then
        aux_prof_COEF(i) % debye_prof(:,:) = aux_prof(i) % debye_prof(:,:)
      Endif
    Endif
  End Do


  !------------------------------------------------------------------
  !4. set up common geometric variables for transmittance calculation
  !------------------------------------------------------------------

  !4.1 USER levs
  !-------------

   
  Call rttov_setgeometry( &
            & nprofiles  ,    &
            & profiles   ,    &
            & aux_prof,       &! in
            & coef,           &! in
            & angles,         &
            & temp0%raytracing    )     ! out  

  !4.2 COEF levs
  !-------------

   
  If(addinterp) then
    Call rttov_setgeometry( &
            & nprofiles,      &
            & temp0%profiles_COEF,  &
            & aux_prof_COEF,  &! in
            & coef,           &! in
            & angles_COEF,    &
            & temp0%raytracing_COEF) ! out  
   Else
     Do i = 1, nprofiles
       angles_COEF(i)      = angles(i)
       temp0%raytracing_COEF % HGPL(:,i)     = temp0%raytracing % HGPL(:,i)
       temp0%raytracing_COEF % DAIR(:,i)     = temp0%raytracing % DAIR(:,i)
       temp0%raytracing_COEF % DMAIR(:,i)    = temp0%raytracing % DMAIR(:,i)
       temp0%raytracing_COEF % R(:,i)        = temp0%raytracing % R(:,i) 
       temp0%raytracing_COEF % RATOESUN(:,i) = temp0%raytracing % RATOESUN(:,i)
       temp0%raytracing_COEF % RATOESAT(:,i) = temp0%raytracing % RATOESAT(:,i)
       temp0%raytracing_COEF % ZASUN(:,i)    = temp0%raytracing % ZASUN(:,i)
       temp0%raytracing_COEF % ZASAT(:,i)    = temp0%raytracing % ZASAT(:,i)
       temp0%raytracing_COEF % INT(:,i)      = temp0%raytracing % INT(:,i)
       temp0%raytracing_COEF % HL(:,i)       = temp0%raytracing % HL(:,i)
       temp0%raytracing_COEF % PPW(:,i)      = temp0%raytracing % PPW(:,i)
       temp0%raytracing_COEF % DISPCO2(:,i)  = temp0%raytracing % DISPCO2(:,i)
       temp0%raytracing_COEF % PATHSAT(:,i)  = temp0%raytracing % PATHSAT(:,i)
       temp0%raytracing_COEF % PATHSUN(:,i)  = temp0%raytracing % PATHSUN(:,i)
       temp0%raytracing_COEF % PATHEFF(:,i)  = temp0%raytracing % PATHEFF(:,i)
       temp0%raytracing_COEF % LTICK(:,i)    = temp0%raytracing % LTICK(:,i)
     enddo
   Endif

  !---------------------------------------------------
  !5. calculate atm/sol predictors - atm/sol COEF levs
  !---------------------------------------------------


  Do i = 1, nprofiles
    temp0%predictors%clw(:,i) = 0._jprb
  EndDo  



  If (coef%fmv_model_ver == 7) Then
     Call rttov_setpredictors_7( &
           & nprofiles , &
           & temp0%profiles_COEF,         &! in
           & angles_COEF,           &! in
           & coef,                &! in
           & aux_prof_COEF,         &! in
           & temp0%predictors,       &! inout  (in because of mem allocation)
           & temp0%raytracing_COEF )       ! inout    

  Else If (coef%fmv_model_ver == 8) Then
     Call rttov_setpredictors_8( &
           & nprofiles,          &
           & temp0%profiles_COEF,         &! in
           & angles_COEF,           &! in
           & coef,                &! in
           & aux_prof_COEF,         &! in
           & temp0%predictors,       &! inout  (in because of mem allocation)
           & temp0%raytracing_COEF )       ! inout    
  Else If (coef%fmv_model_ver == 9) Then
     Call rttov_setpredictors_9( &
            & nprofiles,         &
            & temp0%profiles_COEF,         &! in
            & angles_COEF,           &! in
            & coef,                &! in
            & temp0%predictors,       &
            & temp0%raytracing_COEF ) 


     Call rttov_setpredictors_9_solar( &
          & nprofiles,                 &
          & temp0%profiles_COEF,         &! in
          & angles_COEF,           &! in
          & coef,                &! in
          & temp0%predictors,       &
          & temp0%raytracing_COEF ) 

  Else
     errorstatus(:) = errorstatus_fatal
     Write( errMessage,&
            & '( "Unexpected RTTOV compatibility version number" )' ) 
     Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
     IF (LHOOK) CALL DR_HOOK('RTTOV_TL',1_jpim,ZHOOK_HANDLE)
     Return
  End If

  !--------------------------------------------------------------------
  !6. predict atmospheric and solar optical depths - atm/sol COEF levs 
  !--------------------------------------------------------------------

  If (coef%fmv_model_ver == 9) Then
    Call rttov_opdep_9(         &
       & nchannels,           &! in
       & nprofiles,           &! in
       & coef%nlevels,        &! in
       & channels,            &! in
       & lprofiles,           &! in
       & temp0%predictors,          &! in
       & aux_prof_COEF,       &! in
       & coef,                &! in
       & temp0%opdp_path_COEF,      &! inout
       & opdp_ref_COEF)        ! inout
    Call rttov_opdep_9_solar  ( &
       & nchannels,           &! in
       & nprofiles,           &! in
       & coef%nlevels,        &! in
       & channels,            &! in
       & lprofiles,           &! in
       & profiles,            &! in    ! for solar switch 
       & temp0%predictors,          &! in
       & aux_prof_COEF,       &! in
       & coef,                &! in
       & temp0%opdp_path_COEF,      &! inout
       & opdpsun_ref_COEF)        ! inout
   else
     Call rttov_opdep(         &
       & nchannels,           &! in
       & nprofiles,           &! in
       & coef%nlevels,        &! in
       & channels,            &! in
       & lprofiles,           &! in
       & temp0%predictors,          &! in
       & aux_prof_COEF,       &! in
       & coef,                &! in
       & temp0%opdp_path_COEF,      &! inout
       & opdp_ref_COEF)        ! inout
   endif
   
  !--------------------------------------------------------------------------
  !7. interpolator second  call - optical depths from COEF levs to USER levs
  !--------------------------------------------------------------------------

  If (addinterp) Then
! locate any profile flags in the nchannels sequence 
! at present, only solar switch
   do j=1,nchannels
      prof=lprofiles(j)
      chan=channels(j)
      If(profiles(prof)%addsolar)Then
        If(coef%ss_val_chn(chan) .ne. 0_jpim)Then
          sun(j)=.true.     
        Else
          sun(j)=.false.   
        Endif
      Else
        sun(j)=.false.     
      Endif
    enddo
        Call rttov_intavg_chan( &
	   & nchannels,                       & !in 
	   & sun,                             & !in 
           & temp0%profiles_COEF(1)%nlevels,        & !in
           & profiles(1)%nlevels,             & !in
           & nprofiles,    &
           & lprofiles,    &
           & temp0%profiles_COEF,                & !inout
           & profiles,                     & !inout  only p and addsolar needed 
	   & temp0%opdp_path_COEF,               & !inout  source variable
	   & temp0%opdp_path,                    & !inout  target variable
           & errorstatus)                    !out errorstatus
        If ( Any(errorstatus == errorstatus_fatal) ) Then
          IF (LHOOK) CALL DR_HOOK('RTTOV_TL',1_jpim,ZHOOK_HANDLE)
          Return
        End If
!
  Else
    temp0%opdp_path % atm_layer(:,:) = temp0%opdp_path_COEF % atm_layer(:,:)
    temp0%opdp_path % sun_layer(:,:) = temp0%opdp_path_COEF % sun_layer(:,:)
  EndIf

  !---------------------------------------------------------------------------------------
  !8. If clouds are present,calculate the number of streams and 
  !   the cloud distribution in each stream - remains on USER levs
  !---------------------------------------------------------------------------------------

  nstreams=0_jpim

 
  Do i=1,nprofiles
    temp0%ircld%NSTREAM(i)=0_jpim
    temp0%ircld%XSTRCLR(i)=1._jprb
  enddo

  if(addclouds)then

    Call rttov_cldstr(&
      & nprofiles,    &! in
      & profiles,     &! in
      & coef,         &! in
      & temp0%ircld,        &! out
      & addclouds,    &! in
      & nstreams)      ! out 

  endif


  !---------------------------------------------------------------------------------------
  !9. Calculate optical depths of aerosols and/or clouds - USER levs
  !---------------------------------------------------------------------------------------

  if(addaerosl)then
    do i=1,nprofiles

      Allocate( aux_prof(i)%iaernum     (profiles(1)%nlevels),stat= alloc_status(7))
      Allocate( aux_prof(i)%iaertyp  (11,profiles(1)%nlevels),stat= alloc_status(8))
      Allocate( aux_prof(i)%relhum      (profiles(1)%nlevels),stat= alloc_status(9))
      Allocate( aux_prof(i)%relhumref   (profiles(1)%nlevels),stat= alloc_status(10))

      If( Any(alloc_status /= 0) ) Then
        errorstatus(:) = errorstatus_fatal
        Write( errMessage, '( "allocation of aux_prof")' )
        Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
        IF (LHOOK) CALL DR_HOOK('RTTOV_TL',1_jpim,ZHOOK_HANDLE)
        Return
      End If
    enddo
  End if

  if(addaerosl.or.addclouds)then

    Call rttov_alloc_trans_scatt_ir( alloc_status(1), transmission_scatt_ir, &
      nchannels, profiles(1) %nlevels, 1_jpim )
    Call rttov_alloc_trans_scatt_ir( alloc_status(2), transmission_scatt_ir_stream, &
      nchannels, profiles(1) %nlevels, 1_jpim, nstreams, stream = .true. )


    If( Any(alloc_status /= 0) ) Then
      errorstatus(:) = errorstatus_fatal
      Write( errMessage, '( "allocation of transmission_scatt_ir")' )
      Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
      IF (LHOOK) CALL DR_HOOK('RTTOV_TL',1_jpim,ZHOOK_HANDLE)
      Return
    End If

    transmission_scatt_ir_stream%SSA = 0._jprb


  Call rttov_opdpscattir(&
      & nchannels   , &
      & nprofiles   , &
      & profiles(1)%nlevels, &
      & channels    , &
      & lprofiles   , &
      & aux_prof    , &
      & profiles    , &
      & coef        , &
      & coef_scatt_ir,&
      & temp0%raytracing  , &
      & transmission_scatt_ir    , &
      & transmission_scatt_ir_stream,&
      & optp        , &
      & temp0%ircld       , &
      & addaerosl,    &
      & addclouds) 
  Endif


  !----------------------------------------
  !10. calculate transmittances - USER levs
  !----------------------------------------

  Call rttov_alloc_transmission( alloc_status(1), transmission, profiles(1) % nlevels, nchannels, &
  & 1_jpim, nstreams = nstreams, calc = .true. )

  If( Any(alloc_status /= 0) ) Then
    errorstatus(:) = errorstatus_fatal
    Write( errMessage, '( "allocation of transmission")' )
    Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
    IF (LHOOK) CALL DR_HOOK('RTTOV_TL',1_jpim,ZHOOK_HANDLE)
    Return
  End If


    Call rttov_transmit( &
         & addaerosl,       &! in
         & addclouds,       &! in
         & nchannels,       &! in
         & nprofiles,       &! in
         & profiles(1)%nlevels,    &! in
         & channels,        &! in
         & lprofiles,       &! in
         & aux_prof,        &! in
         & coef,            &! in
         & temp0%ircld,           &! in
         & temp0%opdp_path,       &! in
         & od_layer,        &! out
         & transmission,    &! inout
         & transmission_scatt_ir_stream, &
         & tau_ref,         &! out 
         & tau_ref_surf,    &! out 
         & tau_surf,        &! out 
         & tau_layer)

  If (coef%fmv_model_ver == 9) Then
    Call rttov_transmit_9_solar( &
         & addaerosl,       &! in
         & addclouds,       &! in
         & nchannels,        &! in
         & nprofiles,        &! in
         & profiles(1)%nlevels,    &! in
         & channels,         &! in
         & lprofiles,        &! in
         & profiles,         &! in   ! for solar switch 
         & aux_prof,         &! in
         & coef,             &! in
         & temp0%raytracing  ,     &! in
         & temp0%ircld,           &! in
         & temp0%opdp_path,       &! in
         & odsun_layer,     &! out
         & odsun_singlelayer,& ! out
         & od_frac,         &! out
         & transmission,    &! out
         & transmission_scatt_ir_stream, &! out
         & tausun_ref,     & ! out
         & tausun_ref_surf,& ! out
         & tausun_surf,    & ! out
         & tausun_layer)     ! out
 endif

  !-------------------------------------------------
  !11. calculate channel emissivity values - SURFACE
  !-------------------------------------------------

  If ( Any(calcemis) ) Then
     ! calculate surface emissivity for selected channels
     ! and reflectivity
     If ( coef % id_sensor == sensor_id_ir ) Then
        !Infrared
        Call rttov_calcemis_ir( &
              & profiles,      &! in
              & angles,        &! in
              & coef,          &! in
              & nchannels,     &! in
              & nprofiles,     &! in
              & channels,      &! in
              & lprofiles,     &! in
              & calcemis,      &! in
              & emissivity  )   ! inout 
        reflectivity(:) = 1 - emissivity(:)

        Elseif ( coef % id_sensor == sensor_id_mw .or. coef % id_sensor == sensor_id_po) Then
           !Microwave
        Call rttov_calcemis_mw ( &
              & profiles,      &! in
              & angles,        &! in
              & coef,          &! in
              & nchannels,     &! in
              & nprofiles,     &! in
              & channels,      &! in
              & lprofiles,     &! in
              & transmission,  &! in
              & calcemis,      &! in
              & emissivity,    &! inout
              & reflectivity,  &! out
              & errorstatus   ) ! out 
        If ( Any( errorstatus == errorstatus_fatal ) ) Then
           errorstatus(:) = errorstatus_fatal
           IF (LHOOK) CALL DR_HOOK('RTTOV_TL',1_jpim,ZHOOK_HANDLE)
           Return
        End If
     Else
        ! Hires
        Call rttov_calcemis_ir( &
              & profiles,      &! in
              & angles,        &! in
              & coef,          &! in
              & nchannels,     &! in
              & nprofiles,     &! in
              & channels,      &! in
              & lprofiles,     &! in
              & calcemis,      &! in
              & emissivity  )   ! inout 
        reflectivity(:) = 1 - emissivity(:)
     End If

     ! reflectivity for other channels
     Where( .Not. calcemis(:) )
        reflectivity(:) = 1 - emissivity(:)
     End Where

  Else
     ! reflectivity for all channels
     reflectivity(:) = 1 - emissivity(:)
  End If

  !-------------------------------------------------------
  !12. Compute Fresnel reflectance and sun glint - SURFACE
  !-------------------------------------------------------

     If ( coef % id_sensor == sensor_id_hi ) Then
       If (coef%fmv_model_ver == 9) Then

!CDIR NOVECTOR
         do i=1,nprofiles
           sunglint  (i) % beta     => zbeta     (1:coef%ws_nomega,i)
           sunglint  (i) % psi      => zpsi      (1:coef%ws_nomega,i)
         enddo

         call rttov_refsun( &
         & nprofiles,         &
         & profiles,          &
         & coef,              &
         & aux_prof,          &
         & sunglint,            &
         & temp0%raytracing) 

         call rttov_fresnel(  &
         & nchannels,           &
         & channels,            &
         & nprofiles,           &
         & lprofiles,           &
         & profiles,             &
         & coef,               &
         & sunglint,          &
         & fresnrefl) 

       endif
     endif

  !---------------------------------------------------------
  !13. integrate the radiative transfer equation - USER levs
  !---------------------------------------------------------

  auxrad % layer   => layer(:,:)
  auxrad % surfair => surfair(:)
  auxrad % skin    => skin(:)
  auxrad % cosmic  => cosmic(:)

  Allocate(auxrad_stream % up      (0:nstreams,profiles(1) % nlevels, nchannels) ,stat= alloc_status(1))
  Allocate(auxrad_stream % down    (0:nstreams,profiles(1) % nlevels, nchannels) ,stat= alloc_status(2))
  Allocate(auxrad_stream % down_ref(0:nstreams,profiles(1) % nlevels, nchannels) ,stat= alloc_status(3))
  Allocate(auxrad_stream % cloudy  (0:nstreams,nchannels) ,stat= alloc_status(13))

  if(addsun)then
    if(addaerosl.or.addclouds)then
      Allocate(auxrad_stream % FAC1_1    (0:nstreams,nchannels) ,stat= alloc_status(14))
      Allocate(auxrad_stream % FAC2_1    (0:nstreams,nchannels) ,stat= alloc_status(15))
      Allocate(auxrad_stream % FAC3_1    (0:nstreams,nchannels) ,stat= alloc_status(16))
      Allocate(auxrad_stream % FAC4_1    (0:nstreams,nchannels ) ,stat= alloc_status(17))
      Allocate(auxrad_stream % FAC5_1    (0:nstreams,nchannels ) ,stat= alloc_status(18))
      Allocate(auxrad_stream % FAC6_1    (0:nstreams,nchannels ) ,stat= alloc_status(19))
      Allocate(auxrad_stream % FAC7_1    (0:nstreams,nchannels) ,stat= alloc_status(20))
      Allocate(auxrad_stream % FAC1_2    (profiles(1) % nlevels,0:nstreams,nchannels ) ,stat= alloc_status(21))
      Allocate(auxrad_stream % FAC2_2    (profiles(1) % nlevels,0:nstreams,nchannels ) ,stat= alloc_status(22))
      Allocate(auxrad_stream % FAC3_2    (profiles(1) % nlevels,0:nstreams,nchannels ) ,stat= alloc_status(22))
      Allocate(auxrad_stream % FAC4_2    (profiles(1) % nlevels,0:nstreams,nchannels ) ,stat= alloc_status(24))
      Allocate(auxrad_stream % FAC5_2    (profiles(1) % nlevels,0:nstreams,nchannels ) ,stat= alloc_status(25))
      Allocate(auxrad_stream % FAC6_2    (profiles(1) % nlevels,0:nstreams,nchannels ) ,stat= alloc_status(26))
      Allocate(auxrad_stream % FAC7_2    (profiles(1) % nlevels,0:nstreams,nchannels ) ,stat= alloc_status(27))
      Allocate(auxrad_stream % FAC1_3    (0:nstreams,nchannels) ,stat= alloc_status(28))
      Allocate(auxrad_stream % FAC2_3    (0:nstreams,nchannels) ,stat= alloc_status(29))
      Allocate(auxrad_stream % FAC3_3    (0:nstreams,nchannels) ,stat= alloc_status(30))
      Allocate(auxrad_stream % FAC4_3    (0:nstreams,nchannels ),stat= alloc_status(31))
      Allocate(auxrad_stream % FAC5_3    (0:nstreams,nchannels ),stat= alloc_status(32))
      Allocate(auxrad_stream % FAC6_3    (0:nstreams,nchannels ),stat= alloc_status(33))
      Allocate(auxrad_stream % FAC7_3    (0:nstreams,nchannels) ,stat= alloc_status(34))
    endif
  endif

  addcosmic = ( coef % id_sensor == sensor_id_mw .or. coef % id_sensor == sensor_id_po)

  Call rttov_integrate( &
         & addcosmic,     &! in
         & addaerosl,     &! in
         & addclouds,     &! in
         & nchannels,     &! in
         & nprofiles,     &! in
         & nstreams,      &! in
         & angles,        &! in
         & channels,      &! in
         & lprofiles,     &! in
         & emissivity,    &! in
         & reflectivity,  &! in
         & fresnrefl,     &! in
         & sunglint,      &! in
         & transmission,  &! in
         & transmission_scatt_ir_stream,&
         & profiles,      &! in
         & aux_prof,      &! in
         & coef,          &! in
         & temp0%raytracing,    &! in
         & temp0%ircld,         &! in
         & radiancedata,  &! inout
         & auxrad      ,  &
         & auxrad_stream  ) ! inout  
  !
  ! Tangent Linear
  !----------------
  !
  !---------------------------------------------------------------------------------------
  !1. interpolator first call - input profiles from USER levs to COEF levs
  !---------------------------------------------------------------------------------------

  Call rttov_init_profiles( nprofiles, temp0_tl%profiles_COEF )

  Do i = 1 , nprofiles
    temp0_tl%profiles_COEF(i) % nlevels =  coef % nlevels
    temp0_tl%profiles_COEF(i) % p(:) = 0.0_JPRB
  Enddo

! intvar logical array set by direct code
  If (addinterp) Then
    Call rttov_intavg_prof_tl( &
            & nprofiles,                         &
            & profiles(1)%nlevels,               & !in
            & temp0%profiles_COEF(1)%nlevels,          & !in
            & profiles,                       & !in     source variables
            & profiles_tl,                    & !in     source variables
            & temp0%profiles_COEF,                  & !inout  target variables
            & temp0_tl%profiles_COEF,               & !inout  target variables
            & errorstatus)                      !out
        If ( Any(errorstatus == errorstatus_fatal) ) Then
          IF (LHOOK) CALL DR_HOOK('RTTOV_TL',1_jpim,ZHOOK_HANDLE)
          Return
        End If
  Else 
    Do i = 1, nprofiles
! COEF levs same as USER levs
      temp0_tl%profiles_COEF(i)%t(:)   =  profiles_tl(i)%t(:)
      temp0_tl%profiles_COEF(i)%q(:)   =  profiles_tl(i)%q(:)
      if(profiles(i) % clw_data)   temp0_tl%profiles_COEF(i)%clw(:) =  profiles_tl(i)%clw(:)
      if(profiles(i) % ozone_data) temp0_tl%profiles_COEF(i)%o3(:)  =  profiles_tl(i)%o3(:)
      if(profiles(i) % co2_data)   temp0_tl%profiles_COEF(i)%co2(:) =  profiles_tl(i)%co2(:)
      if(profiles(i) % n2o_data)   temp0_tl%profiles_COEF(i)%n2o(:) =  profiles_tl(i)%n2o(:)   
      if(profiles(i) % co_data)    temp0_tl%profiles_COEF(i)%co(:)  =  profiles_tl(i)%co(:)    
      if(profiles(i) % ch4_data)   temp0_tl%profiles_COEF(i)%ch4(:) =  profiles_tl(i)%ch4(:)  
    End Do
  EndIf
! complete profiles on COEF levs for surf, skin, cloud, aerosol, solar, angle
  Do i = 1, nprofiles
    temp0_tl%profiles_COEF(i) % ozone_data = profiles_tl(i) % ozone_data
    temp0_tl%profiles_COEF(i) % co2_data   = profiles_tl(i) % co2_data
    temp0_tl%profiles_COEF(i) % clw_data   = profiles_tl(i) % clw_data

    temp0_tl%profiles_COEF(i) % n2o_data    = profiles_tl(i) % n2o_data
    temp0_tl%profiles_COEF(i) % co_data     = profiles_tl(i) % co_data
    temp0_tl%profiles_COEF(i) % ch4_data    = profiles_tl(i) % ch4_data
    temp0_tl%profiles_COEF(i) % aer_data    = profiles_tl(i) % aer_data
    temp0_tl%profiles_COEF(i) % cld_data    = profiles_tl(i) % cld_data

    temp0_tl%profiles_COEF(i) % S2m        = profiles_tl(i) % S2m
    temp0_tl%profiles_COEF(i) % skin       = profiles_tl(i) % skin
    temp0_tl%profiles_COEF(i) % ctp        = profiles_tl(i) % ctp
    temp0_tl%profiles_COEF(i) % cfraction  = profiles_tl(i) % cfraction

    temp0_tl%profiles_COEF(i) % idg    = profiles_tl(i) % idg
    temp0_tl%profiles_COEF(i) % ish    = profiles_tl(i) % ish

    temp0_tl%profiles_COEF(i) % zenangle   = profiles_tl(i) % zenangle
    temp0_tl%profiles_COEF(i) % azangle    = profiles_tl(i) % azangle

    temp0_tl%profiles_COEF(i) % sunzenangle = profiles_tl(i) % sunzenangle
    temp0_tl%profiles_COEF(i) % sunazangle  = profiles_tl(i) % sunazangle
    temp0_tl%profiles_COEF(i) % latitude    = profiles_tl(i) % latitude
    temp0_tl%profiles_COEF(i) % elevation   = profiles_tl(i) % elevation
    temp0_tl%profiles_COEF(i) % addsolar    = profiles_tl(i) % addsolar
    temp0_tl%profiles_COEF(i) % addrefrac   = profiles_tl(i) % addrefrac  


  End Do      ! over profiles

  Call rttov_init_raytracing( temp0_tl%raytracing )

  !------------------------------------------------------------------
  !2. check input data is within suitable physical limits - COEF levs
  !------------------------------------------------------------------
 If ( apply_reg_limits ) Then

    Do i = 1, nprofiles

    Call rttov_checkinput_tl( &
           & profiles_COEF_ref( i ),     &!in
           & temp0_tl%profiles_COEF( i ),      &!inout
           & coef) !in 

    End Do

  End If 
 
  !------------------------------------------------------------------------
  !3. determine cloud top, surface levels, ice cloud parameters
  !------------------------------------------------------------------------

  !3.1 USER levs
  !-------------
 
  jn=profiles(1) % nlevels
  Do i = 1, nprofiles
     aux_prof_tl(i) % nearestlev_surf = 0   ! no meaning
     aux_prof_tl(i) % nearestlev_ctp  = 0   ! no meaning
     aux_prof_tl(i) % pfraction_surf  = 0._JPRB  ! calculated
     aux_prof_tl(i) % pfraction_ctp   = 0._JPRB  ! calculated   inside rttov_profaux_tl
     aux_prof_tl(i) % cfraction       = 0._JPRB  ! calculated
     ! Note that  cfraction and pfraction_ctp are set to 0 in case
     ! of MicroWaves
  End Do

  If( coef % id_sensor == sensor_id_mw .or. coef % id_sensor == sensor_id_po ) Then
!CDIR NOVECTOR
    Do i = 1, nprofiles
      aux_prof_tl(i) % debye_prof => zdeb_tl(1:5,1:jn,i)
    End Do
  Endif

  if(addclouds)then
!CDIR NOVECTOR
    Do i = 1, nprofiles
       aux_prof_tl(i) % dg      => zdg_tl(1:jn,i)
       aux_prof_tl(i) % fac1_dg => zfac1_dg_tl(1:jn,i)
       aux_prof_tl(i) % fac2_dg => zfac2_dg_tl(1:jn,i)
       aux_prof_tl(i) % fac3_dg => zfac3_dg_tl(1:jn,i)
    End Do
   endif

  Do i = 1, nprofiles
     If( coef % id_sensor == sensor_id_mw .or. coef % id_sensor == sensor_id_po) Then
        aux_prof_tl(i) % debye_prof(:,:)  = 0._JPRB
     Endif
     Call rttov_profaux_tl( &
           & addclouds,       &! in
           & profiles(i),     &! in
           & profiles_tl(i),  &! in
           & coef,            &! in
           & aux_prof(i),     &! in
           & aux_prof_tl(i))   ! inout
  End Do

  !3.2 COEF levs
  !-------------
  Do i = 1, nprofiles
      aux_prof_COEF_tl(i) % nearestlev_surf = 0   ! no meaning
      aux_prof_COEF_tl(i) % nearestlev_ctp  = 0   ! no meaning
      aux_prof_COEF_tl(i) % pfraction_surf  = 0._JPRB  ! calculated
      aux_prof_COEF_tl(i) % pfraction_ctp   = 0._JPRB  ! calculated   inside rttov_profaux_tl
      aux_prof_COEF_tl(i) % cfraction       = 0._JPRB  ! calculated
      ! Note that  cfraction and pfraction_ctp are set to 0 in case
      ! of MicroWaves
  EndDo
  jn=coef % nlevels
  If( coef % id_sensor == sensor_id_mw .or. coef % id_sensor == sensor_id_po ) Then
!CDIR NOVECTOR
    Do i = 1, nprofiles
      aux_prof_COEF_tl(i) % debye_prof => zdeb_COEF_tl(1:5,1:jn,i)
    End Do
  Endif

  Do i = 1, nprofiles
     If( coef % id_sensor == sensor_id_mw .or. coef % id_sensor == sensor_id_po) Then
        aux_prof_COEF_tl(i) % debye_prof(:,:)  = 0._JPRB
     Endif
     If(addinterp) then
       Call rttov_profaux_tl( &
           & .false.  ,          &! no cloud computations on coef levels
           & temp0%profiles_COEF(i),   &! in
           & temp0_tl%profiles_COEF(i),&! in
           & coef,               &! in
           & aux_prof_COEF(i),   &! in
           & aux_prof_COEF_tl(i)) ! inout 
     Else 
       aux_prof_COEF_tl(i) % nearestlev_surf = aux_prof_tl(i) % nearestlev_surf
       aux_prof_COEF_tl(i) % pfraction_surf  = aux_prof_tl(i) % pfraction_surf
       aux_prof_COEF_tl(i) % nearestlev_ctp  = aux_prof_tl(i) % nearestlev_ctp 
       aux_prof_COEF_tl(i) % pfraction_ctp   = aux_prof_tl(i) % pfraction_ctp
       aux_prof_COEF_tl(i) % cfraction       = aux_prof_tl(i) % cfraction 
       If( coef % id_sensor == sensor_id_mw  .or.coef % id_sensor == sensor_id_po ) Then
         aux_prof_COEF_tl(i) % debye_prof(:,:) = aux_prof_tl(i) % debye_prof(:,:)
       Endif
     Endif
  End Do

  ! TL on geometry
  !------------------------------------------------------------------------
  !4. set up common geometric variables for the RT integration
  !------------------------------------------------------------------------

  !4.1 USER levs
  !-------------



  Call rttov_setgeometry_tl( &
         & nprofiles  ,   &
         & profiles   ,       &
         & profiles_tl,   &
         & aux_prof,      &! in
         & coef,          &! in
         & angles,        &
         & temp0%raytracing,    &
         & temp0_tl%raytracing   )     ! out 


  !4.2 COEF levs
  !-------------


   If(addinterp) then
     Call rttov_setgeometry_tl( &
          & nprofiles  ,   &
          & temp0%profiles_COEF   ,       &
          & temp0_tl%profiles_COEF,   &
          & aux_prof_COEF,      &! in
          & coef,          &! in
          & angles_COEF,        &
          & temp0%raytracing_COEF,    &
          & temp0_tl%raytracing_COEF   )     ! out 
   Else

     Do i = 1, nprofiles
      angles_COEF(i)      = angles(i)
      temp0_tl%raytracing_COEF % HGPL(:,i)     = temp0_tl%raytracing % HGPL(:,i)
      temp0_tl%raytracing_COEF % DAIR(:,i)     = temp0_tl%raytracing % DAIR(:,i)
      temp0_tl%raytracing_COEF % DMAIR(:,i)    = temp0_tl%raytracing % DMAIR(:,i)
      temp0_tl%raytracing_COEF % R(:,i)        = temp0_tl%raytracing % R(:,i) 
      temp0_tl%raytracing_COEF % RATOESUN(:,i) = temp0_tl%raytracing % RATOESUN(:,i)
      temp0_tl%raytracing_COEF % RATOESAT(:,i) = temp0_tl%raytracing % RATOESAT(:,i)
      temp0_tl%raytracing_COEF % ZASUN(:,i)    = temp0_tl%raytracing % ZASUN(:,i)
      temp0_tl%raytracing_COEF % ZASAT(:,i)    = temp0_tl%raytracing % ZASAT(:,i)
      temp0_tl%raytracing_COEF % INT(:,i)      = temp0_tl%raytracing % INT(:,i)
      temp0_tl%raytracing_COEF % HL(:,i)       = temp0_tl%raytracing % HL(:,i)
      temp0_tl%raytracing_COEF % PPW(:,i)      = temp0_tl%raytracing % PPW(:,i)
      temp0_tl%raytracing_COEF % DISPCO2(:,i)  = temp0_tl%raytracing % DISPCO2(:,i)
      temp0_tl%raytracing_COEF % PATHSAT(:,i)  = temp0_tl%raytracing % PATHSAT(:,i)
      temp0_tl%raytracing_COEF % PATHSUN(:,i)  = temp0_tl%raytracing % PATHSUN(:,i)
      temp0_tl%raytracing_COEF % PATHEFF(:,i)  = temp0_tl%raytracing % PATHEFF(:,i)
      temp0_tl%raytracing_COEF % LTICK(:,i)    = temp0_tl%raytracing % LTICK(:,i)
    enddo
  Endif


  ! TL of predictors
  !---------------------------------------------------
  !5. calculate atm/sol predictors - atm/sol COEF levs
  !---------------------------------------------------



  Do i = 1, nprofiles
     temp0_tl%predictors % nlevels  = temp0%predictors % nlevels
     temp0_tl%predictors % nmixed   = temp0%predictors % nmixed
     temp0_tl%predictors % nwater   = temp0%predictors % nwater
     temp0_tl%predictors % nozone   = temp0%predictors % nozone
     temp0_tl%predictors % nwvcont  = temp0%predictors % nwvcont
     temp0_tl%predictors % nco2     = temp0%predictors % nco2
     temp0_tl%predictors % nn2o     = temp0%predictors % nn2o
     temp0_tl%predictors % nco      = temp0%predictors % nco
     temp0_tl%predictors % nch4     = temp0%predictors % nch4
     temp0_tl%predictors % ncloud   = temp0%predictors % ncloud
  End Do
     If (coef%fmv_model_ver == 7) Then
        Call rttov_setpredictors_7_tl( &
              & nprofiles,             &
              & temp0%profiles_COEF,         &! in
              & temp0_tl%profiles_COEF,      &! in
              & angles_COEF,           &! in
              & coef,                     &! in
              & aux_prof_COEF,         &! in
              & temp0%predictors,            &! in
              & temp0_tl%predictors,         &! inout
              & temp0%raytracing_COEF,       &! in
              & temp0_tl%raytracing_COEF )    ! inout  

     ElseIf (coef%fmv_model_ver == 8) Then
        Call rttov_setpredictors_8_tl( &
              & nprofiles,             &
              & temp0%profiles_COEF,         &! in
              & temp0_tl%profiles_COEF,      &! in
              & angles_COEF,           &! in
              & coef,                &! in
              & aux_prof_COEF,         &! in
              & temp0%predictors,       &! in
              & temp0_tl%predictors,    &! inout
              & temp0%raytracing_COEF,       &! in
              & temp0_tl%raytracing_COEF )    ! inout  

     ElseIf (coef%fmv_model_ver == 9) Then
        Call rttov_setpredictors_9_tl( &
               & nprofiles,            &
               & temp0%profiles_COEF,         &! in
               & temp0_tl%profiles_COEF,      &! in
               & angles_COEF,           &! in
               & coef,                &! in
               & temp0%predictors,       &! in
               & temp0_tl%predictors,    &
               & temp0%raytracing_COEF,       &
               & temp0_tl%raytracing_COEF )    ! inout 

        Call rttov_setpredictors_9_solar_tl( &
               & nprofiles,                    &
               & temp0%profiles_COEF,         &! in
               & temp0_tl%profiles_COEF,      &! in
               & angles_COEF,           &! in
               & coef,                     &! in
               & temp0%predictors,            &! in
               & temp0_tl%predictors,         &
               & temp0%raytracing_COEF,       &
               & temp0_tl%raytracing_COEF ) 

     End If


  !TL of optical depths
  !--------------------------------------------------------------------
  !6. predict atmospheric and solar optical depths - atm/sol COEF levs 
  !--------------------------------------------------------------------

  temp0_tl%opdp_path%atm_layer = 0._jprb
  temp0_tl%opdp_path%sun_layer = 0._jprb
  temp0_tl%opdp_path_COEF%atm_layer = 0._jprb
  temp0_tl%opdp_path_COEF%sun_layer = 0._jprb

  If (coef%fmv_model_ver == 9) Then
    Call rttov_opdep_9_tl(         &
       & nchannels,           &! in
       & nprofiles,           &! in
       & coef%nlevels,        &! in
       & channels,            &! in
       & lprofiles,           &! in
       & temp0%predictors,          &! in
       & temp0_tl%predictors,       &! in
       & aux_prof_COEF,       &! in
       & aux_prof_COEF_tl,    &! in
       & coef,                &! in
       & temp0%opdp_path_COEF,      &! inout
       & temp0_tl%opdp_path_COEF,   &! inout
       & opdp_ref_COEF)        ! inout
    Call rttov_opdep_9_solar_tl  ( &
       & nchannels,           &! in
       & nprofiles,           &! in
       & coef%nlevels,        &! in
       & channels,            &! in
       & lprofiles,           &! in
       & profiles,            &! in    ! for solar switch 
       & temp0%predictors,          &! in
       & temp0_tl%predictors,       &! in
       & aux_prof_COEF,       &! in
       & aux_prof_COEF_tl,    &! in
       & coef,                &! in
       & temp0%opdp_path_COEF,      &! inout
       & temp0_tl%opdp_path_COEF,   &! inout
       & opdpsun_ref_COEF)        ! inout
   else
     Call rttov_opdep_tl(         &
       & nchannels,           &! in
       & nprofiles,           &! in
       & coef%nlevels,        &! in
       & channels,            &! in
       & lprofiles,           &! in
       & temp0%predictors,          &! in
       & temp0_tl%predictors,       &! in
       & aux_prof_COEF,       &! in
       & aux_prof_COEF_tl,    &! in
       & coef,                &! in
       & temp0%opdp_path_COEF,      &! inout
       & temp0_tl%opdp_path_COEF,   &! inout
       & opdp_ref_COEF)        ! inout
   endif

  !--------------------------------------------------------------------------
  !7. interpolator second  call - optical depths from COEF levs to USER levs
  !--------------------------------------------------------------------------

    If (addinterp) Then
        Call rttov_intavg_chan_tl( &
	   & nchannels,                          & !in 
	   & sun,                                & !in 
           & temp0%profiles_COEF(1)%nlevels,           & !in
           & profiles(1)%nlevels,                & !in
           & nprofiles,    &
           & lprofiles,    &
           & temp0%profiles_COEF,                      & !in  only p and addsolar used this call 
           & profiles,                           & !in  only p and addsolar used this call 
           & profiles_tl,                        & !in  only p 
           & temp0%opdp_path_COEF,                     & !in  
           & temp0_tl%opdp_path_COEF,                  & !in    source variable
           & temp0%opdp_path,                          & !in  
           & temp0_tl%opdp_path,                       & !inout target variable
           & errorstatus)                          !out
        If ( Any(errorstatus == errorstatus_fatal) ) Then
          IF (LHOOK) CALL DR_HOOK('RTTOV_TL',1_jpim,ZHOOK_HANDLE)
          Return
        End If
  Else
!    temp0%opdp_path    % atm_layer(:,:) = temp0%opdp_path_COEF    % atm_layer(:,:)
    temp0_tl%opdp_path % atm_layer(:,:) = temp0_tl%opdp_path_COEF % atm_layer(:,:)
!    temp0%opdp_path    % sun_layer(:,:) = temp0%opdp_path_COEF    % sun_layer(:,:)
    temp0_tl%opdp_path % sun_layer(:,:) = temp0_tl%opdp_path_COEF % sun_layer(:,:)
  EndIf

!! testop omitted


  ! TL of ircld
  !--------------------------------------------------------------------
  !8. If clouds are present,calculate the number of streams and
  !   the cloud distribution  in each stream - remains on USER levs
  !--------------------------------------------------------------------

 

  do i=1,nprofiles
    temp0_tl%ircld%XSTRCLR(i)=0._jprb
  enddo

  if(addclouds)then


     Call rttov_cldstr_tl(&
      & nprofiles,    &! in
      & profiles,     &! in
      & profiles_tl,  &
      & coef,         &! in
      & temp0%ircld,        &
      & temp0_tl%ircld,     &
      & addclouds)         ! out 

  endif


  ! TL of opdpscattir
  !----------------------------------------------------------------------------
  !9. Calculate optical depths of aerosols and/or clouds - USER levs
  !----------------------------------------------------------------------------

  if(addaerosl)then
    Do i=1,nprofiles

      Allocate( aux_prof_tl(i)%relhum   (profiles(1)%nlevels),stat= alloc_status(9))

      If( Any(alloc_status /= 0) ) Then
        errorstatus(:) = errorstatus_fatal
        Write( errMessage, '( "allocation of aux_prof")' )
        Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
        IF (LHOOK) CALL DR_HOOK('RTTOV_TL',1_jpim,ZHOOK_HANDLE)
        Return
      End If
    enddo

  End if

  if(addaerosl.or.addclouds)then

    Call rttov_alloc_trans_scatt_ir( alloc_status(1), transmission_scatt_ir_tl, &
      nchannels, profiles(1) %nlevels, 1_jpim )
    Call rttov_alloc_trans_scatt_ir( alloc_status(2), transmission_scatt_ir_stream_tl, &
      nchannels, profiles(1) %nlevels, 1_jpim, nstreams, stream = .true. )



    If( Any(alloc_status /= 0) ) Then
      errorstatus(:) = errorstatus_fatal
      Write( errMessage, '( "allocation of transmission_scatt_ir_stream_tl ")' ) 
      Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
      IF (LHOOK) CALL DR_HOOK('RTTOV_TL',1_jpim,ZHOOK_HANDLE)
      Return
    End If

    transmission_scatt_ir_stream_tl%SSA = 0._jprb



  Call rttov_opdpscattir_tl(&
      & nchannels   , &
      & nprofiles   , &
      & profiles(1)%nlevels, &
      & channels    , &
      & lprofiles   , &
      & aux_prof    , &
      & aux_prof_tl , &
      & profiles    , &
      & profiles_tl , &
      & coef        , &
      & coef_scatt_ir,&
      & temp0%raytracing  , &
      & temp0_tl%raytracing,&
      & transmission_scatt_ir    , &
      & transmission_scatt_ir_tl,&
      & transmission_scatt_ir_stream,&
      & transmission_scatt_ir_stream_tl,&
      & optp        , &
      & temp0%ircld,        &
      & temp0_tl%ircld,      &
      & addaerosl,   &
      & addclouds) 
  Endif

  ! TL of transmit
  !----------------------------------------
  !10. calculate transmittances - USER levs
  !----------------------------------------

  Call rttov_alloc_transmission( alloc_status(1), transmission_tl, profiles(1) % nlevels, nchannels, &
  & 1_jpim, nstreams = nstreams, calc = .true. )

  If( Any(alloc_status /= 0) ) Then
    errorstatus(:) = errorstatus_fatal
    Write( errMessage, '( "allocation of transmission")' )
    Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
    IF (LHOOK) CALL DR_HOOK('RTTOV_TL',1_jpim,ZHOOK_HANDLE)
    Return
  End If


  !TL of optical depths and transmittances
    Call rttov_transmit_tl( &
         & addaerosl,                     &! in
         & addclouds,                     &! in
         & nchannels,                     &! in
         & nprofiles,                     &! in
         & profiles(1)%nlevels,           &! in
         & channels,                      &! in
         & lprofiles,                     &! in
         & aux_prof,                      &! in
         & aux_prof_tl,                   &! in
         & coef,                          &! in
         & temp0%ircld,                         &! in
         & temp0%opdp_path,                     &! inout
         & temp0_tl%opdp_path,                  &! inout
         & od_layer,                      &! in
         & transmission,                  &! in
         & transmission_tl,               &! inout
         & transmission_scatt_ir_stream  ,&! inout
         & transmission_scatt_ir_stream_tl, &!inout  
         & tau_ref,                       &! in
         & tau_ref_surf,                  &! in
         & tau_surf,                      &! in
         & tau_layer)                      ! in

   If (coef%fmv_model_ver == 9) Then
    Call rttov_transmit_9_solar_tl( &
         & addaerosl,        &! in
         & addclouds,        &! in
         & nchannels,        &! in
         & nprofiles,        &! in
         & profiles(1)%nlevels,    &! in
         & channels,         &! in
         & lprofiles,        &! in
         & profiles,         &! in
         & aux_prof,         &! in
         & aux_prof_tl,      &! in
         & coef,             &! in
         & temp0%raytracing  ,     &! in
         & temp0_tl%raytracing  ,  &! in
         & temp0%ircld,            &! in
         & temp0%opdp_path,         &! in
         & temp0_tl%opdp_path,      &! in
         & odsun_layer,       & ! out
         & odsun_singlelayer, & ! out
         & od_frac,           &! out
         & transmission,      &! in
         & transmission_tl,   &
         & transmission_scatt_ir_stream,&
         & transmission_scatt_ir_stream_tl,&  
         & tausun_ref,        & !in
         & tausun_ref_surf,   & !in
         & tausun_layer,      & !in
         & tausun_surf)         !in
  endif
  
  ! TL of emissivity
  !-------------------------------------------------
  !11. calculate channel emissivity values - SURFACE
  !-------------------------------------------------

  If ( Any(calcemis) ) Then
     ! calculate surface emissivity for selected channels
     ! and reflectivity
     If ( coef % id_sensor == sensor_id_ir ) Then
        !Infrared
        ! nothing to do
        reflectivity_tl(:) =  - emissivity_tl(:)

        Elseif ( coef % id_sensor == sensor_id_mw .or. coef % id_sensor == sensor_id_po) Then
           !Microwave
        Call rttov_calcemis_mw_tl ( &
              & profiles,        &! in
              & profiles_tl,     &! in
              & angles,          &! in
              & coef,            &! in
              & nchannels,       &! in
              & nprofiles,       &! in
              & channels,        &! in
              & lprofiles,       &! in
              & transmission,    &! in
              & transmission_tl, &! in
              & calcemis,        &! in
              & emissivity_tl,   &! inout
              & reflectivity_tl ) ! out 
     Else
        ! Hires
        reflectivity_tl(:) = - emissivity_tl(:)
     End If

     ! reflectivity for other channels
     Where( .Not. calcemis(:) )
        reflectivity_tl(:) = - emissivity_tl(:)
     End Where

  Else
     ! reflectivity for all channels
     reflectivity_tl(:) = - emissivity_tl(:)
  End If

   ! TL of reflectance and sun glint
  !-------------------------------------------------------
  !12. Compute Fresnel reflectance and sun glint - SURFACE
  !-------------------------------------------------------
  
  If ( coef % id_sensor == sensor_id_hi ) Then
     If (coef%fmv_model_ver == 9) Then

!CDIR NOVECTOR
         do i=1,nprofiles
           sunglint_tl(i) % beta     => zbeta_tl  (1:coef%ws_nomega,i)
           sunglint_tl(i) % psi      => zpsi_tl   (1:coef%ws_nomega,i)
         enddo

         call rttov_refsun_tl( &
          & nprofiles,       &
          & profiles,        &
          & profiles_tl,     &
          & coef,            &
          & aux_prof,        &
          & sunglint,        &
          & sunglint_tl,     &
          & temp0%raytracing,      &
          & temp0_tl%raytracing)     

         call rttov_fresnel_tl(&
         & nchannels,         &
         & channels,          &
         & nprofiles,         &
         & lprofiles,         &
         & profiles,          &
         & coef,              &
         & sunglint,          &
         & sunglint_tl,       &
         & fresnrefl,         &
         & fresnrefl_tl) 

     endif
  endif

  ! TL of RTE
  !---------------------------------------------------------
  !13. integrate the radiative transfer equation - USER levs
  !---------------------------------------------------------

    Allocate(auxrad_stream_tl % cloudy  (0:nstreams,nchannels) ,stat= alloc_status(12))

    If( Any(alloc_status /= 0) ) Then
        errorstatus(:) = errorstatus_fatal
        Write( errMessage, '( "allocation of auxrad_stream_tl")' )
        Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
        IF (LHOOK) CALL DR_HOOK('RTTOV_TL',1_jpim,ZHOOK_HANDLE)
        Return
    endif

  addcosmic = ( coef % id_sensor == sensor_id_mw .or. coef % id_sensor == sensor_id_po)

  Call rttov_integrate_tl( &
         & addcosmic,     &! in
         & addaerosl,     &! in
         & addclouds,     &! in
         & nchannels,     &! in
         & nprofiles,     &! in
         & nstreams,      &! in
         & angles,        &! in
         & channels,      &! in
         & lprofiles,     &! in
         & emissivity,    &! in
         & emissivity_tl,    &! in
         & reflectivity,     &! in
         & reflectivity_tl,  &! in
         & fresnrefl   ,     &! in
         & fresnrefl_tl   ,  &! in
         & sunglint,         &! in
         & sunglint_tl,      &! in
         & transmission,     &! in
         & transmission_tl,  &! in
         & transmission_scatt_ir_stream,&
         & transmission_scatt_ir_stream_tl,&
         & profiles,         &! in
         & profiles_tl,      &! in
         & aux_prof,         &! in
         & aux_prof_tl,      &! in
         & coef,             &! in
         & temp0%raytracing,       &! in
         & temp0_tl%raytracing,    &! in
         & temp0%ircld,            &! in
         & temp0_tl%ircld,         &! in
         & radiancedata,     &! in
         & auxrad ,          &! in
         & auxrad_stream,    &! in
         & auxrad_stream_tl, &! in
         & radiancedata_tl  ) ! inout  
 
  !--------------------
  !14. deallocate memory
  !--------------------



  if(addaerosl)then
    Do i=1,nprofiles
      DeAllocate( aux_prof_tl(i)%relhum   ,stat= alloc_status(26))
    enddo
  endif

  if(addaerosl.or.addclouds)then
    Call rttov_alloc_trans_scatt_ir( alloc_status(1), transmission_scatt_ir_tl, &
      nchannels, profiles(1) %nlevels, 0_jpim )
    Call rttov_alloc_trans_scatt_ir( alloc_status(2), transmission_scatt_ir_stream_tl, &
      nchannels, profiles(1) %nlevels, 0_jpim )

    If( Any(alloc_status /= 0) ) Then
      errorstatus(:) = errorstatus_fatal
      Write( errMessage, '( "deallocation of scatt_ir_tl")' )
      Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
      IF (LHOOK) CALL DR_HOOK('RTTOV_TL',1_jpim,ZHOOK_HANDLE)
      Return
    endif

  endif




    Call rttov_alloc_transmission( alloc_status(1), transmission_tl, profiles(1) % nlevels, nchannels, &
    & 0_jpim, nstreams = nstreams, calc = .true. )

    If( Any(alloc_status /= 0) ) Then
      errorstatus(:) = errorstatus_fatal
      Write( errMessage, '( "deallocation of transmission_tl" )' )
      Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
      IF (LHOOK) CALL DR_HOOK('RTTOV_TL',1_jpim,ZHOOK_HANDLE)
      Return
    endif





  if(addaerosl)then
    do i=1,nprofiles
      DeAllocate( aux_prof(i)%iaernum  ,stat= alloc_status(24))
      DeAllocate( aux_prof(i)%iaertyp  ,stat= alloc_status(25))
      DeAllocate( aux_prof(i)%relhum   ,stat= alloc_status(26))
      DeAllocate( aux_prof(i)%relhumref,stat= alloc_status(26))
    enddo
  endif

  if(addaerosl.or.addclouds)then

    Call rttov_alloc_trans_scatt_ir( alloc_status(1), transmission_scatt_ir, &
      nchannels, profiles(1) %nlevels, 0_jpim )
    Call rttov_alloc_trans_scatt_ir( alloc_status(2), transmission_scatt_ir_stream, &
      nchannels, profiles(1) %nlevels, 0_jpim )


  endif



    Call rttov_alloc_transmission( alloc_status(1), transmission, profiles(1) % nlevels, nchannels, &
    & 0_jpim, nstreams = nstreams, calc = .true. )


    DeAllocate(auxrad_stream % up     ,stat= alloc_status(79))
    DeAllocate(auxrad_stream % down   ,stat= alloc_status(80))
    DeAllocate(auxrad_stream % down_ref   ,stat= alloc_status(80))
    DeAllocate(auxrad_stream % cloudy ,stat= alloc_status(90))
    DeAllocate(auxrad_stream_tl % cloudy ,stat= alloc_status(90))

    if(addsun)then
      if(addaerosl.or.addclouds)then
        DeAllocate(auxrad_stream % FAC1_1    ,stat= alloc_status(91))
        DeAllocate(auxrad_stream % FAC2_1    ,stat= alloc_status(92))
        DeAllocate(auxrad_stream % FAC3_1    ,stat= alloc_status(93))
        DeAllocate(auxrad_stream % FAC4_1    ,stat= alloc_status(94))
        DeAllocate(auxrad_stream % FAC5_1    ,stat= alloc_status(95))
        DeAllocate(auxrad_stream % FAC6_1    ,stat= alloc_status(96))
        DeAllocate(auxrad_stream % FAC7_1    ,stat= alloc_status(97))
        DeAllocate(auxrad_stream % FAC1_2    ,stat= alloc_status(98))
        DeAllocate(auxrad_stream % FAC2_2    ,stat= alloc_status(99))
        DeAllocate(auxrad_stream % FAC3_2    ,stat= alloc_status(100))
        DeAllocate(auxrad_stream % FAC4_2    ,stat= alloc_status(101))
        DeAllocate(auxrad_stream % FAC5_2    ,stat= alloc_status(102))
        DeAllocate(auxrad_stream % FAC6_2    ,stat= alloc_status(103))
        DeAllocate(auxrad_stream % FAC7_2    ,stat= alloc_status(104))
        DeAllocate(auxrad_stream % FAC1_3    ,stat= alloc_status(105))
        DeAllocate(auxrad_stream % FAC2_3    ,stat= alloc_status(106))
        DeAllocate(auxrad_stream % FAC3_3    ,stat= alloc_status(107))
        DeAllocate(auxrad_stream % FAC4_3    ,stat= alloc_status(108))
        DeAllocate(auxrad_stream % FAC5_3    ,stat= alloc_status(109))
        DeAllocate(auxrad_stream % FAC6_3    ,stat= alloc_status(100))
        DeAllocate(auxrad_stream % FAC7_3    ,stat= alloc_status(111))
      endif
    endif

    If( Any(alloc_status /= 0) ) Then
      errorstatus(:) = errorstatus_fatal
      Write( errMessage, '( "deallocation of raytracing")' )
      Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
      IF (LHOOK) CALL DR_HOOK('RTTOV_TL',1_jpim,ZHOOK_HANDLE)
      Return
    endif

  If ( apply_reg_limits ) Then
    Call rttov_alloc_prof( alloc_status(2), nprofiles, profiles_COEF_ref, &
      coef%nlevels, coef_scatt_ir, 0_jpim, blob = profiles_COEF_blob_ref )
  End If 

  If( Any(alloc_status /= 0) ) Then
    errorstatus = errorstatus_fatal
    Write( errMessage, '( "deallocation of profiles_COEF")' )
    Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
    IF (LHOOK) CALL DR_HOOK('RTTOV_TL',1_jpim,ZHOOK_HANDLE)
    Return
  End If



  Call rttov_check_temp( &
    & temp_status,                             &
    & nprofiles,                               &
    & nchannels,                               &
    & channels,                                &
    & addinterp,                               &
    & profiles(1)%nlevels,                     &
    & coef,                                    &
    & coef_scatt_ir,                           &
    & optp,                                    &
    & addsun .or. Any( profiles(:)%addsolar ), &
    & addaerosl,                               &
    & addclouds,                               &
    & 0_jpim,                                  &
    & temp0 = temp0, temp0_tl = temp0_tl,      &
    & temp1 = temp1, temp1_tl = temp1_tl,      &
    & temp2 = temp,  temp2_tl = temp_tl )

  If( temp_status .ne. errorstatus_success ) Then
    Write( errMessage, '( "rttov_check_temp fatal error ")' )
    Call Rttov_ErrorReport( errorstatus_fatal, errMessage, NameOfRoutine )
    Return
  EndIf

  IF (LHOOK) CALL DR_HOOK('RTTOV_TL',1_jpim,ZHOOK_HANDLE)

End Subroutine rttov_tl
